@@ -17,33 +17,15 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 """
-import inkex, os, re, simplepath, cubicsuperpath
+import inkex, os, simplepath, cubicsuperpath
 from ffgeom import *
 
-uuconv = {'in':90.0, 'pt':1.25, 'px':1, 'mm':3.5433070866, 'cm':35.433070866, 'pc':15.0}
-def unittouu(string):
-    unit = re.compile('(%s)$' % '|'.join(uuconv.keys()))
-    param = re.compile(r'(([-+]?[0-9]+(\.[0-9]*)?|[-+]?\.[0-9]+)([eE][-+]?[0-9]+)?)')
-
-    p = param.match(string)
-    u = unit.search(string)    
-    if p:
-        retval = float(p.string[p.start():p.end()])
-    else:
-        retval = 0.0
-    if u:
-        try:
-            return retval * uuconv[u.string[u.start():u.end()]]
-        except KeyError:
-            pass
-    return retval
-
 class Project(inkex.Effect):
     def __init__(self):
             inkex.Effect.__init__(self)
     def effect(self):
         if len(self.options.ids) < 2:
-            inkex.debug("Requires two selected paths. The second must be exctly four nodes long.")
+            inkex.debug("Requires two selected paths. The second must be exactly four nodes long.")
             exit()            
             
         #obj is selected second
@@ -69,7 +51,7 @@ class Project(inkex.Effect):
                 self.q[query] = float(f.read())
                 f.close()
             #glean document height from the SVG
-            docheight = unittouu(inkex.xml.xpath.Evaluate('/svg/@height',self.document)[0].value)
+            docheight = inkex.unittouu(inkex.xml.xpath.Evaluate('/svg/@height',self.document)[0].value)
             #Flip inkscapes transposed renderer coords
             self.q['y'] = docheight - self.q['y'] - self.q['height']
 
@@ -96,4 +78,4 @@ class Project(inkex.Effect):
         return [p['x'],p['y']]    
 
 e = Project()
-e.affect()
\ No newline at end of file
+e.affect()
