@@ -8,9 +8,10 @@
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *   bulia byak <buliabyak@users.sf.net>
  *
+ * Copyright (C) 2006      Johan Engelen <johan@shouraizou.nl>
+ * Copyright (C) 2002      Mitsuru Oka
  * Copyright (C) 2000-2002 Lauris Kaplinski
  * Copyright (C) 2000-2001 Ximian, Inc.
- * Copyright (C) 2002 Mitsuru Oka
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
@@ -39,6 +40,7 @@
 #include "desktop.h"
 #include "desktop-style.h"
 #include "context-fns.h"
+#include "verbs.h"
 
 #include "arc-context.h"
 
@@ -324,10 +326,12 @@ static gint sp_arc_context_root_handler(SPEventContext *event_context, GdkEvent
                 case GDK_Shift_R:
                 case GDK_Meta_L:  // Meta is when you press Shift+Alt (at least on my machine)
                 case GDK_Meta_R:
-                    sp_event_show_modifier_tip(event_context->defaultMessageContext(), event,
-                                               _("<b>Ctrl</b>: make circle or integer-ratio ellipse, snap arc/segment angle"),
-                                               _("<b>Shift</b>: draw around the starting point"),
-                                               NULL);
+                    if (!dragging) {
+                        sp_event_show_modifier_tip(event_context->defaultMessageContext(), event,
+                                                   _("<b>Ctrl</b>: make circle or integer-ratio ellipse, snap arc/segment angle"),
+                                                   _("<b>Shift</b>: draw around the starting point"),
+                                                   NULL);
+                    }
                     break;
                 case GDK_Up:
                 case GDK_Down:
@@ -401,6 +405,8 @@ static void sp_arc_drag(SPArcContext *ac, NR::Point pt, guint state)
         Inkscape::GC::release(repr);
         ac->item->transform = SP_ITEM(desktop->currentRoot())->getRelativeTransform(desktop->currentLayer());
         ac->item->updateRepr();
+
+        sp_canvas_force_full_redraw_after_interruptions(desktop->canvas, 5);
     }
 
     NR::Rect const r = Inkscape::snap_rectangular_box(desktop, ac->item, pt, ac->center, state);
@@ -425,8 +431,11 @@ static void sp_arc_finish(SPArcContext *ac)
 
         SP_OBJECT(ac->item)->updateRepr();
 
+        sp_canvas_end_forced_full_redraws(desktop->canvas);
+        
         sp_desktop_selection(desktop)->set(ac->item);
-        sp_document_done(sp_desktop_document(desktop));
+        sp_document_done(sp_desktop_document(desktop), SP_VERB_CONTEXT_ARC, 
+                         _("Create ellipse"));
 
         ac->item = NULL;
     }
