@@ -33,28 +33,11 @@
 
 /* Root item handler */
 
-
 int sp_desktop_root_handler(SPCanvasItem *item, GdkEvent *event, SPDesktop *desktop)
 {
     return sp_event_context_root_handler(desktop->event_context, event);
 }
 
-/*
- * fixme: this conatins a hack, to deal with deleting a view, which is
- * completely on another view, in which case active_desktop will not be updated
- *
- */
-
-int sp_desktop_item_handler(SPCanvasItem *item, GdkEvent *event, gpointer data)
-{
-    gpointer ddata = gtk_object_get_data(GTK_OBJECT(item->canvas), "SPDesktop");
-    g_return_val_if_fail(ddata != NULL, FALSE);
-
-    SPDesktop *desktop = static_cast<SPDesktop*>(ddata);
-
-    return sp_event_context_item_handler(desktop->event_context, SP_ITEM(data), event);
-}
-
 
 static gint sp_dt_ruler_event(GtkWidget *widget, GdkEvent *event, SPDesktopWidget *dtw, bool horiz)
 {
@@ -123,7 +106,8 @@ static gint sp_dt_ruler_event(GtkWidget *widget, GdkEvent *event, SPDesktopWidge
                     sp_repr_set_svg_double(repr, "position", guide_pos_dt);
                     SP_OBJECT_REPR(desktop->namedview)->appendChild(repr);
                     Inkscape::GC::release(repr);
-                    sp_document_done(sp_desktop_document(desktop));
+                    sp_document_done(sp_desktop_document(desktop), SP_VERB_NONE, 
+                                     _("Create guide"));
                 }
                 desktop->set_coordinate_status(event_dt);
             }
@@ -196,13 +180,16 @@ gint sp_dt_guide_event(SPCanvasItem *item, GdkEvent *event, gpointer data)
                     NR::Point const event_dt(desktop->w2d(event_w));
                     if (sp_canvas_world_pt_inside_window(item->canvas, event_w)) {
                         sp_guide_moveto(*guide, sp_guide_position_from_pt(guide, event_dt), true);
+                        sp_document_done(sp_desktop_document(desktop), SP_VERB_NONE,
+                                     _("Move guide"));
                     } else {
                         /* Undo movement of any attached shapes. */
                         sp_guide_moveto(*guide, guide->position, false);
                         sp_guide_remove(guide);
+                        sp_document_done(sp_desktop_document(desktop), SP_VERB_NONE,
+                                     _("Delete guide"));
                     }
                     moved = false;
-                    sp_document_done(sp_desktop_document(desktop));
                     desktop->set_coordinate_status(event_dt);
                     desktop->setPosition (event_dt);
                 }
