@@ -24,6 +24,8 @@
 #include "style.h"
 #include "prefs-utils.h"
 #include "sp-use.h"
+#include "sp-filter.h"
+#include "sp-gaussian-blur.h"
 #include "sp-flowtext.h"
 #include "sp-flowregion.h"
 #include "sp-flowdiv.h"
@@ -1009,6 +1011,72 @@ objects_query_fontfamily (GSList *objects, SPStyle *style_res)
     }
 }
 
+/**
+ * Write to style_res the average blurring of a list of objects.
+ */
+int
+objects_query_blur (GSList *objects, SPStyle *style_res)
+{
+   if (g_slist_length(objects) == 0) {
+        /* No objects, set empty */
+        return QUERY_STYLE_NOTHING;
+    }
+
+    float blur_sum = 0;
+    float blur_prev = -1;
+    bool same_blur = true;
+    guint blur_items = 0;
+    guint items = 0;
+    
+    for (GSList const *i = objects; i != NULL; i = i->next) {
+        SPObject *obj = SP_OBJECT (i->data);
+        SPStyle *style = SP_OBJECT_STYLE (obj);
+        if (!style) continue;
+
+        NR::Matrix i2d = sp_item_i2d_affine (SP_ITEM(obj));
+
+        items ++;
+
+        //if object has a filter
+        if (style->filter.set && style->filter.filter) {
+            //cycle through filter primitives
+            for(int i=0; i<style->filter.filter->_primitive_count; i++)
+            {
+                SPFilterPrimitive *primitive = style->filter.filter->_primitives[i];
+                //if primitive is gaussianblur
+                if(SP_IS_GAUSSIANBLUR(primitive)) {
+                    SPGaussianBlur * spblur = SP_GAUSSIANBLUR(primitive);
+                    float num = spblur->stdDeviation.getNumber();
+                    blur_sum += num * NR::expansion(i2d);
+                    if (blur_prev != -1 && fabs (num - blur_prev) > 1e-2) // rather low tolerance because difference in blur radii is much harder to notice than e.g. difference in sizes
+                        same_blur = false;
+                    blur_prev = num;
+                    //TODO: deal with opt number, for the moment it's not necessary to the ui.
+                    blur_items ++;
+                }
+            }
+        }
+
+    }
+
+    if (items > 0) {
+        if (blur_items > 0)
+            blur_sum /= blur_items;
+        style_res->filter_gaussianBlur_deviation.value = blur_sum;
+    }
+
+    if (items == 0) {
+        return QUERY_STYLE_NOTHING;
+    } else if (items == 1) {
+        return QUERY_STYLE_SINGLE;
+    } else {
+        if (same_blur)
+            return QUERY_STYLE_MULTIPLE_SAME;
+        else
+            return QUERY_STYLE_MULTIPLE_AVERAGED;
+    }
+}
+
 /**
  * Query the given list of objects for the given property, write
  * the result to style, return appropriate flag.
@@ -1039,8 +1107,10 @@ sp_desktop_query_style_from_list (GSList *list, SPStyle *style, int property)
         return objects_query_fontstyle (list, style);
     } else if (property == QUERY_STYLE_PROPERTY_FONTNUMBERS) {
         return objects_query_fontnumbers (list, style);
-    }
 
+    } else if (property == QUERY_STYLE_PROPERTY_BLUR) {
+        return objects_query_blur (list, style);
+    }
     return QUERY_STYLE_NOTHING;
 }
 
@@ -1078,8 +1148,9 @@ sp_desktop_query_style_all (SPDesktop *desktop, SPStyle *query)
         int result_strokecap = sp_desktop_query_style (desktop, query, QUERY_STYLE_PROPERTY_STROKECAP);
         int result_strokejoin = sp_desktop_query_style (desktop, query, QUERY_STYLE_PROPERTY_STROKEJOIN);
         int result_opacity = sp_desktop_query_style (desktop, query, QUERY_STYLE_PROPERTY_MASTEROPACITY);
-
-        return (result_family != QUERY_STYLE_NOTHING && result_fstyle != QUERY_STYLE_NOTHING && result_fnumbers != QUERY_STYLE_NOTHING && result_fill != QUERY_STYLE_NOTHING && result_stroke != QUERY_STYLE_NOTHING && result_opacity != QUERY_STYLE_NOTHING && result_strokewidth != QUERY_STYLE_NOTHING && result_strokemiterlimit != QUERY_STYLE_NOTHING && result_strokecap != QUERY_STYLE_NOTHING && result_strokejoin != QUERY_STYLE_NOTHING);
+        int result_blur = sp_desktop_query_style (desktop, query, QUERY_STYLE_PROPERTY_BLUR);
+        
+        return (result_family != QUERY_STYLE_NOTHING && result_fstyle != QUERY_STYLE_NOTHING && result_fnumbers != QUERY_STYLE_NOTHING && result_fill != QUERY_STYLE_NOTHING && result_stroke != QUERY_STYLE_NOTHING && result_opacity != QUERY_STYLE_NOTHING && result_strokewidth != QUERY_STYLE_NOTHING && result_strokemiterlimit != QUERY_STYLE_NOTHING && result_strokecap != QUERY_STYLE_NOTHING && result_strokejoin != QUERY_STYLE_NOTHING && result_blur != QUERY_STYLE_NOTHING);
 }
 
 
