@@ -16,6 +16,9 @@
 
 #include <display/nr-arena.h>
 #include <display/nr-arena-shape.h>
+#include "display/nr-filter.h"
+#include "display/nr-filter-gaussian.h"
+#include "display/nr-filter-types.h"
 #include <libnr/n-art-bpath.h>
 #include <libnr/nr-path.h>
 #include <libnr/nr-pixops.h>
@@ -26,6 +29,10 @@
 #include <livarot/float-line.h>
 #include <livarot/int-line.h>
 #include <style.h>
+/* prefs-utils used for deciding, whether to run filtering test or not */
+#include "prefs-utils.h"
+#include "sp-filter.h"
+#include "sp-gaussian-blur.h"
 
 //int  showRuns=0;
 void nr_pixblock_render_shape_mask_or(NRPixBlock &m,Shape* theS);
@@ -85,6 +92,10 @@ nr_arena_shape_class_init(NRArenaShapeClass *klass)
     item_class->pick = nr_arena_shape_pick;
 }
 
+/**
+ * Initializes the arena shape, setting all parameters to null, 0, false,
+ * or other defaults
+ */
 static void
 nr_arena_shape_init(NRArenaShape *shape)
 {
@@ -131,6 +142,9 @@ nr_arena_shape_finalize(NRObject *object)
     ((NRObjectClass *) shape_parent_class)->finalize(object);
 }
 
+/**
+ * Retrieves the markers from the item
+ */
 static NRArenaItem *
 nr_arena_shape_children(NRArenaItem *item)
 {
@@ -139,6 +153,11 @@ nr_arena_shape_children(NRArenaItem *item)
     return shape->markers;
 }
 
+/**
+ * Attaches child to item, and if ref is not NULL, sets it and ref->next as
+ * the prev and next items.  If ref is NULL, then it sets the item's markers
+ * as the next items.
+ */
 static void
 nr_arena_shape_add_child(NRArenaItem *item, NRArenaItem *child, NRArenaItem *ref)
 {
@@ -153,6 +172,10 @@ nr_arena_shape_add_child(NRArenaItem *item, NRArenaItem *child, NRArenaItem *ref
     nr_arena_item_request_update(item, NR_ARENA_ITEM_STATE_ALL, FALSE);
 }
 
+/**
+ * Removes child from the shape.  If there are no prev items in 
+ * the child, it sets items' markers to the next item in the child.
+ */
 static void
 nr_arena_shape_remove_child(NRArenaItem *item, NRArenaItem *child)
 {
@@ -167,6 +190,12 @@ nr_arena_shape_remove_child(NRArenaItem *item, NRArenaItem *child)
     nr_arena_item_request_update(item, NR_ARENA_ITEM_STATE_ALL, FALSE);
 }
 
+/**
+ * Detaches child from item, and if there are no previous items in child, it 
+ * sets item's markers to the child.  It then attaches the child back onto the item.
+ * If ref is null, it sets the markers to be the next item, otherwise it uses
+ * the next/prev items in ref.
+ */
 static void
 nr_arena_shape_set_child_position(NRArenaItem *item, NRArenaItem *child, NRArenaItem *ref)
 {
@@ -191,6 +220,9 @@ void nr_arena_shape_update_stroke(NRArenaShape *shape, NRGC* gc, NRRectL *area);
 void nr_arena_shape_update_fill(NRArenaShape *shape, NRGC *gc, NRRectL *area, bool force_shape = false);
 void nr_arena_shape_add_bboxes(NRArenaShape* shape,NRRect &bbox);
 
+/**
+ * Updates the arena shape 'item' and all of its children, including the markers.
+ */
 static guint
 nr_arena_shape_update(NRArenaItem *item, NRRectL *area, NRGC *gc, guint state, guint reset)
 {
@@ -236,6 +268,8 @@ nr_arena_shape_update(NRArenaItem *item, NRRectL *area, NRGC *gc, guint state, g
     bbox.x0 = bbox.y0 = NR_HUGE;
     bbox.x1 = bbox.y1 = -NR_HUGE;
 
+    bool outline = (NR_ARENA_ITEM(shape)->arena->rendermode == RENDERMODE_OUTLINE);
+
     if (shape->curve) {
         NRBPath bp;
         /* fixme: */
@@ -243,7 +277,7 @@ nr_arena_shape_update(NRArenaItem *item, NRRectL *area, NRGC *gc, guint state, g
         bbox.x1 = bbox.y1 = -NR_HUGE;
         bp.path = SP_CURVE_BPATH(shape->curve);
         nr_path_matrix_bbox_union(&bp, gc->transform, &bbox);
-        if (shape->_stroke.paint.type() != NRArenaShape::Paint::NONE) {
+        if (shape->_stroke.paint.type() != NRArenaShape::Paint::NONE || outline) {
             float width, scale;
             scale = NR_MATRIX_DF_EXPANSION(&gc->transform);
             width = MAX(0.125, shape->_stroke.width * scale);
@@ -292,11 +326,13 @@ nr_arena_shape_update(NRArenaItem *item, NRRectL *area, NRGC *gc, guint state, g
         shape->stroke_painter = NULL;
     }
 
-    if (!shape->curve || !shape->style) return NR_ARENA_ITEM_STATE_ALL;
-    if (sp_curve_is_empty(shape->curve)) return NR_ARENA_ITEM_STATE_ALL;
-    if ( ( shape->_fill.paint.type() == NRArenaShape::Paint::NONE ) &&
-         ( shape->_stroke.paint.type() == NRArenaShape::Paint::NONE ) )
+    if (!shape->curve || 
+        !shape->style ||
+        sp_curve_is_empty(shape->curve) ||
+        (( shape->_fill.paint.type() == NRArenaShape::Paint::NONE ) &&
+         ( shape->_stroke.paint.type() == NRArenaShape::Paint::NONE && !outline) ))
     {
+        item->bbox = shape->approx_bbox;
         return NR_ARENA_ITEM_STATE_ALL;
     }
 
@@ -479,13 +515,19 @@ nr_arena_shape_update_stroke(NRArenaShape *shape,NRGC* gc, NRRectL *area)
 
     float const scale = NR_MATRIX_DF_EXPANSION(&gc->transform);
 
-    if (NR_ARENA_ITEM(shape)->arena->rendermode == RENDERMODE_OUTLINE ||
+    bool outline = (NR_ARENA_ITEM(shape)->arena->rendermode == RENDERMODE_OUTLINE);
+
+    if (outline ||
         ((shape->_stroke.paint.type() != NRArenaShape::Paint::NONE) &&
          ( fabs(shape->_stroke.width * scale) > 0.01 ))) { // sinon c'est 0=oon veut pas de bord
 
-        float width = MAX(0.125, shape->_stroke.width * scale);
-        if (NR_ARENA_ITEM(shape)->arena->rendermode == RENDERMODE_OUTLINE)
+        float style_width = MAX(0.125, shape->_stroke.width * scale);
+        float width;
+        if (outline) {
             width = 0.5; // 1 pixel wide, independent of zoom
+        } else {
+            width = style_width;
+        }
 
         NR::Matrix  cached_to_new = NR::identity();
 
@@ -499,6 +541,10 @@ nr_arena_shape_update_stroke(NRArenaShape *shape,NRGC* gc, NRRectL *area)
             }
             if (0 != isometry && !is_inner_area(shape->cached_sarea, *area))
                 isometry = 0;
+            if (0 != isometry && width != shape->cached_width) { 
+                // if this happens without setting style, we have just switched to outline or back
+                isometry = 0; 
+            } 
         }
 
         if ( isometry == 0 ) {
@@ -517,16 +563,16 @@ nr_arena_shape_update_stroke(NRArenaShape *shape,NRGC* gc, NRRectL *area)
             padded_area.x1 += (NR::ICoord)width;
             padded_area.y0 -= (NR::ICoord)width;
             padded_area.y1 += (NR::ICoord)width;
-            if ((style->stroke_dash.n_dash && NR_ARENA_ITEM(shape)->arena->rendermode != RENDERMODE_OUTLINE) || is_inner_area(padded_area, NR_ARENA_ITEM(shape)->bbox)) {
-                thePath->Convert((NR_ARENA_ITEM(shape)->arena->rendermode != RENDERMODE_OUTLINE) ? 1.0 : 4.0);
+            if ((style->stroke_dash.n_dash && !outline) || is_inner_area(padded_area, NR_ARENA_ITEM(shape)->bbox)) {
+                thePath->Convert((outline) ? 4.0 : 1.0);
                 shape->cached_spartialy = false;
             }
             else {
-                thePath->Convert(&padded_area, (NR_ARENA_ITEM(shape)->arena->rendermode != RENDERMODE_OUTLINE) ? 1.0 : 4.0);
+                thePath->Convert(&padded_area, (outline) ? 4.0 : 1.0);
                 shape->cached_spartialy = true;
             }
 
-            if (style->stroke_dash.n_dash && NR_ARENA_ITEM(shape)->arena->rendermode != RENDERMODE_OUTLINE) {
+            if (style->stroke_dash.n_dash && !outline) {
                 thePath->DashPolylineFromStyle(style, scale, 1.0);
             }
 
@@ -555,7 +601,7 @@ nr_arena_shape_update_stroke(NRArenaShape *shape,NRGC* gc, NRRectL *area)
                     break;
             }
 
-            if (NR_ARENA_ITEM(shape)->arena->rendermode == RENDERMODE_OUTLINE) {
+            if (outline) {
                 butt = butt_straight;
                 join = join_straight;
             }
@@ -564,13 +610,15 @@ nr_arena_shape_update_stroke(NRArenaShape *shape,NRGC* gc, NRRectL *area)
                             0.5*width*shape->_stroke.mitre_limit);
 
 
-            if (NR_ARENA_ITEM(shape)->arena->rendermode == RENDERMODE_OUTLINE) {
+            if (outline) {
                 // speeds it up, but uses evenodd for the stroke shape (which does not matter for 1-pixel wide outline)
                 shape->cached_stroke->Copy(theShape);
             } else {
                 shape->cached_stroke->ConvertToShape(theShape, fill_nonZero);
             }
 
+            shape->cached_width = width;
+
             shape->cached_sctm=gc->transform;
             shape->cached_sarea = *area;
             delete thePath;
@@ -666,6 +714,10 @@ nr_arena_shape_add_bboxes(NRArenaShape* shape, NRRect &bbox)
         }
     }
 }
+
+/**
+ * Renders the item.  Markers are just composed into the parent buffer.
+ */
 static unsigned int
 nr_arena_shape_render(NRArenaItem *item, NRRectL *area, NRPixBlock *pb, unsigned int flags)
 {
@@ -694,12 +746,21 @@ nr_arena_shape_render(NRArenaItem *item, NRRectL *area, NRPixBlock *pb, unsigned
         }
     }
 
+    bool outline = (NR_ARENA_ITEM(shape)->arena->rendermode == RENDERMODE_OUTLINE);
+
     SPStyle const *style = shape->style;
-    if ( shape->fill_shp && NR_ARENA_ITEM(shape)->arena->rendermode != RENDERMODE_OUTLINE) {
+    if ( shape->fill_shp && !outline) {
         NRPixBlock m;
         guint32 rgba;
 
         nr_pixblock_setup_fast(&m, NR_PIXBLOCK_MODE_A8, area->x0, area->y0, area->x1, area->y1, TRUE);
+
+        // if memory allocation failed, abort render
+        if (m.data.px == NULL) {
+            nr_pixblock_release (&m);
+            return (item->state);
+        }
+
         m.visible_area = pb->visible_area; 
         nr_pixblock_render_shape_mask_or(m,shape->fill_shp);
         m.empty = FALSE;
@@ -731,13 +792,19 @@ nr_arena_shape_render(NRArenaItem *item, NRRectL *area, NRPixBlock *pb, unsigned
         guint32 rgba;
 
         nr_pixblock_setup_fast(&m, NR_PIXBLOCK_MODE_A8, area->x0, area->y0, area->x1, area->y1, TRUE);
+
+        // if memory allocation failed, abort render
+        if (m.data.px == NULL) {
+            nr_pixblock_release (&m);
+            return (item->state);
+        }
+
         m.visible_area = pb->visible_area; 
         nr_pixblock_render_shape_mask_or(m, shape->stroke_shp);
         m.empty = FALSE;
 
-        if (shape->_stroke.paint.type() == NRArenaShape::Paint::COLOR ||
-            NR_ARENA_ITEM(shape)->arena->rendermode == RENDERMODE_OUTLINE) {
-            if ( NR_ARENA_ITEM(shape)->arena->rendermode == RENDERMODE_OUTLINE) {
+        if (shape->_stroke.paint.type() == NRArenaShape::Paint::COLOR || outline) {
+            if (outline) {
                 rgba = NR_ARENA_ITEM(shape)->arena->outlinecolor;
             } else if ( item->render_opacity ) {
                 rgba = sp_color_get_rgba32_falpha(&shape->_stroke.paint.color(),
@@ -790,6 +857,13 @@ nr_arena_shape_clip(NRArenaItem *item, NRRectL *area, NRPixBlock *pb)
 
         /* fixme: We can OR in one step (Lauris) */
         nr_pixblock_setup_fast(&m, NR_PIXBLOCK_MODE_A8, area->x0, area->y0, area->x1, area->y1, TRUE);
+
+        // if memory allocation failed, abort 
+        if (m.data.px == NULL) {
+            nr_pixblock_release (&m);
+            return (item->state);
+        }
+
         m.visible_area = pb->visible_area; 
         nr_pixblock_render_shape_mask_or(m,shape->fill_shp);
 
@@ -798,7 +872,7 @@ nr_arena_shape_clip(NRArenaItem *item, NRRectL *area, NRPixBlock *pb)
             s = NR_PIXBLOCK_PX(&m) + (y - area->y0) * m.rs;
             d = NR_PIXBLOCK_PX(pb) + (y - area->y0) * pb->rs;
             for (int x = area->x0; x < area->x1; x++) {
-                *d = NR_A7_NORMALIZED(*s,*d);
+                *d = NR_COMPOSEA_111(*s, *d);
                 d ++;
                 s ++;
             }
@@ -850,18 +924,20 @@ nr_arena_shape_pick(NRArenaItem *item, NR::Point p, double delta, unsigned int /
         }
     }
 
+    bool outline = (NR_ARENA_ITEM(shape)->arena->rendermode == RENDERMODE_OUTLINE);
+
     if (item->state & NR_ARENA_ITEM_STATE_RENDER) {
         if (shape->fill_shp && (shape->_fill.paint.type() != NRArenaShape::Paint::NONE)) {
             if (shape->fill_shp->PtWinding(p) > 0 ) return item;
         }
-        if (shape->stroke_shp && (shape->_stroke.paint.type() != NRArenaShape::Paint::NONE)) {
+        if (shape->stroke_shp && (shape->_stroke.paint.type() != NRArenaShape::Paint::NONE || outline)) {
             if (shape->stroke_shp->PtWinding(p) > 0 ) return item;
         }
         if (delta > 1e-3) {
             if (shape->fill_shp && (shape->_fill.paint.type() != NRArenaShape::Paint::NONE)) {
                 if (distanceLessThanOrEqual(shape->fill_shp, p, delta)) return item;
             }
-            if (shape->stroke_shp && (shape->_stroke.paint.type() != NRArenaShape::Paint::NONE)) {
+            if (shape->stroke_shp && (shape->_stroke.paint.type() != NRArenaShape::Paint::NONE || outline)) {
                 if (distanceLessThanOrEqual(shape->stroke_shp, p, delta)) return item;
             }
         }
@@ -878,7 +954,7 @@ nr_arena_shape_pick(NRArenaItem *item, NR::Point p, double delta, unsigned int /
                 if (wind & 0x1) return item;
             }
         }
-        if (shape->_stroke.paint.type() != NRArenaShape::Paint::NONE) {
+        if (shape->_stroke.paint.type() != NRArenaShape::Paint::NONE || outline) {
             /* fixme: We do not take stroke width into account here (Lauris) */
             if (dist < delta) return item;
         }
@@ -1082,6 +1158,41 @@ nr_arena_shape_set_style(NRArenaShape *shape, SPStyle *style)
     }
     shape->setMitreLimit(style->stroke_miterlimit.value);
 
+    //if shape has a filter
+    if (style->filter.set && style->filter.filter) 
+    {
+        shape->filter = new NR::Filter();
+        shape->filter->set_x(style->filter.filter->x);
+        shape->filter->set_y(style->filter.filter->y);
+        shape->filter->set_width(style->filter.filter->width);
+        shape->filter->set_height(style->filter.filter->height);
+        
+        //go through all SP filter primitives
+        for(int i=0; i<style->filter.filter->_primitive_count; i++)
+        {
+            SPFilterPrimitive *primitive = style->filter.filter->_primitives[i];
+            //if primitive is gaussianblur
+            if(SP_IS_GAUSSIANBLUR(primitive)) {
+                NR::FilterGaussian * gaussian = (NR::FilterGaussian *) shape->filter->add_primitive(NR::NR_FILTER_GAUSSIANBLUR);
+                SPGaussianBlur * spblur = SP_GAUSSIANBLUR(primitive);
+                float num = spblur->stdDeviation.getNumber();
+                if( num>=0.0 )
+                {
+                    float optnum = spblur->stdDeviation.getOptNumber();
+                    if( optnum>=0.0 )
+                        gaussian->set_deviation((double) num, (double) optnum);
+                    else
+                        gaussian->set_deviation((double) num);
+                }
+            }
+        }
+    }
+    else
+    {
+        //no filter set for this shape
+        shape->filter = NULL;
+    }
+
     nr_arena_item_request_update(shape, NR_ARENA_ITEM_STATE_ALL, FALSE);
 }
 
@@ -1139,10 +1250,8 @@ shape_run_A8_OR(raster_info &dest,void */*data*/,int st,float vst,int en,float v
             unsigned int c0_24=(int)sv;
             c0_24&=0xFF;
             while (len > 0) {
-                unsigned int da;
                 /* Draw */
-                da = NR_A7(c0_24,d[0]);
-                d[0] = NR_PREMUL_SINGLE(da);
+                d[0] = NR_COMPOSEA_111(c0_24,d[0]);
                 d += 1;
                 len -= 1;
             }
@@ -1153,10 +1262,8 @@ shape_run_A8_OR(raster_info &dest,void */*data*/,int st,float vst,int en,float v
             sv*=256;
             unsigned int c0_24=(int)sv;
             c0_24&=0xFF;
-            unsigned int da;
             /* Draw */
-            da = NR_A7(c0_24,d[0]);
-            d[0] = NR_PREMUL_SINGLE(da);
+            d[0] = NR_COMPOSEA_111(c0_24,d[0]);
         } else {
             dv/=len;
             sv+=0.5*dv; // correction trapezoidale
@@ -1165,12 +1272,11 @@ shape_run_A8_OR(raster_info &dest,void */*data*/,int st,float vst,int en,float v
             int c0_24 = static_cast<int>(CLAMP(sv, 0, 16777216));
             int s0_24 = static_cast<int>(dv);
             while (len > 0) {
-                unsigned int ca, da;
+                unsigned int ca;
                 /* Draw */
                 ca = c0_24 >> 16;
                 if ( ca > 255 ) ca=255;
-                da = NR_A7(ca,d[0]);
-                d[0] = NR_PREMUL_SINGLE(da);
+                d[0] = NR_COMPOSEA_111(ca,d[0]);
                 d += 1;
                 c0_24 += s0_24;
                 c0_24 = CLAMP(c0_24, 0, 16777216);
