@@ -17,19 +17,13 @@
  *
  */
 
-#include <math.h>
-#include "sp-canvas.h"
+#include "display-forward.h"
 #include "sp-canvas-util.h"
 #include "sp-ctrlline.h"
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
-#include <libnr/nr-point.h>
-#include <libnr/nr-matrix.h>
-#include <libnr/nr-point-ops.h>
-#include <libnr/nr-point-matrix-ops.h>
-#include <libnr/nr-rect-l.h>
 #include <livarot/Shape.h>
 #include <livarot/Path.h>
 
@@ -103,7 +97,7 @@ sp_ctrlline_destroy (GtkObject *object)
         delete ctrlline->shp;
         ctrlline->shp = NULL;
     }
-  
+
     if (GTK_OBJECT_CLASS (parent_class)->destroy)
         (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
@@ -113,18 +107,14 @@ sp_ctrlline_render (SPCanvasItem *item, SPCanvasBuf *buf)
 {
     SPCtrlLine *ctrlline = SP_CTRLLINE (item);
 
-    if (buf->is_bg) {
-        sp_canvas_clear_buffer (buf);
-        buf->is_bg = FALSE;
-        buf->is_buf = TRUE;
-    }
     NRRectL  area;
     area.x0=buf->rect.x0;
     area.x1=buf->rect.x1;
     area.y0=buf->rect.y0;
     area.y1=buf->rect.y1;
+
     if (ctrlline->shp) {
-        sp_canvas_buf_ensure_buf (buf);
+        sp_canvas_prepare_buffer (buf);
         nr_pixblock_render_ctrl_rgba (ctrlline->shp,ctrlline->rgba,area,(char*)buf->buf, buf->buf_rowstride);
     }
 }
@@ -151,8 +141,13 @@ sp_ctrlline_update (SPCanvasItem *item, NR::Matrix const &affine, unsigned int f
     Path* thePath = new Path;
     thePath->MoveTo(NR::Point(cl->s.x, cl->s.y) * affine);
     thePath->LineTo(NR::Point(cl->e.x, cl->e.y) * affine);
-  
-    thePath->Convert(1.0);
+
+    NRRectL  area;
+    area.x0=(NR::ICoord)(double)item->x1;
+    area.x1=(NR::ICoord)(double)item->x2;
+    area.y0=(NR::ICoord)(double)item->y1;
+    area.y1=(NR::ICoord)(double)item->y2;
+    thePath->Convert(&area, 1.0);
     if ( cl->shp == NULL ) cl->shp=new Shape;
     thePath->Stroke(cl->shp,false,0.5,join_straight,butt_straight,20.0,false);
     cl->shp->CalcBBox();
@@ -168,7 +163,7 @@ sp_ctrlline_update (SPCanvasItem *item, NR::Matrix const &affine, unsigned int f
         }
     }
     delete thePath;
-  
+
     item->x1 = (int)dbox.x0;
     item->y1 = (int)dbox.y0;
     item->x2 = (int)dbox.x1;
@@ -219,9 +214,9 @@ sp_ctrlline_set_coords (SPCtrlLine *cl, const NR::Point start, const NR::Point e
   Local Variables:
   mode:c++
   c-file-style:"stroustrup"
-  c-file-offsets:((innamespace . 0)(inline-open . 0))
+  c-file-offsets:((innamespace . 0)(inline-open . 0)(case-label . +))
   indent-tabs-mode:nil
   fill-column:99
   End:
 */
-// vim: filetype=c++:expandtab:shiftwidth=4:tabstop=8:softtabstop=4 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
