@@ -27,6 +27,7 @@
 #include "gc-finalized.h"
 #include "gc-anchored.h"
 #include <glibmm/ustring.h>
+#include "verbs.h"
 
 namespace Avoid {
 class Router;
@@ -42,6 +43,7 @@ namespace Inkscape {
     struct Application;
     class Selection; 
     class UndoStackObserver;
+    class EventLog;
     namespace XML {
         class Document;
         class Node;
@@ -113,6 +115,8 @@ struct SPDocument : public Inkscape::GC::Managed<>,
 	void addUndoObserver(Inkscape::UndoStackObserver& observer);
 	void removeUndoObserver(Inkscape::UndoStackObserver& observer);
 
+        bool _updateDocument();
+
 private:
 	SPDocument(SPDocument const &); // no copy
 	void operator=(SPDocument const &); // no assign
@@ -137,6 +141,9 @@ SPDocument *sp_document_new_dummy();
 SPDocument *sp_document_ref (SPDocument *doc);
 SPDocument *sp_document_unref (SPDocument *doc);
 
+
+SPDocument *sp_document_create(Inkscape::XML::Document *rdoc, gchar const *uri, gchar const *base, gchar const *name, unsigned int keepalive);
+
 /*
  * Access methods
  */
@@ -166,8 +173,8 @@ void sp_document_set_height (SPDocument * document, gdouble height, const SPUnit
  * Undo & redo
  */
 
-void sp_document_set_undo_sensitive (SPDocument * document, gboolean sensitive);
-gboolean sp_document_get_undo_sensitive (SPDocument const * document);
+void sp_document_set_undo_sensitive (SPDocument * document, bool sensitive);
+bool sp_document_get_undo_sensitive (SPDocument const * document);
 
 void sp_document_clear_undo (SPDocument * document);
 void sp_document_clear_redo (SPDocument * document);
@@ -183,8 +190,8 @@ void sp_document_request_modified (SPDocument *doc);
 gint sp_document_ensure_up_to_date (SPDocument *doc);
 
 /* Save all previous actions to stack, as one undo step */
-void sp_document_done (SPDocument *document);
-void sp_document_maybe_done (SPDocument *document, const gchar *key);
+void sp_document_done (SPDocument *document, unsigned int event_type, Glib::ustring event_description);
+void sp_document_maybe_done (SPDocument *document, const gchar *keyconst, unsigned int event_type, Glib::ustring event_description);
 void sp_document_reset_key (Inkscape::Application *inkscape, SPDesktop *desktop, GtkObject *base);
 
 /* Cancel (and revert) current unsaved actions */
