@@ -135,7 +135,7 @@ bool CssParser::match(int pos, char *str)
 {
     while (*str)
        {
-       if (get(pos++) != *str++)
+       if (get(pos++) != (XMLCh) *str++)
            return false;
        }
    return true;
@@ -192,7 +192,7 @@ int CssParser::skipwhite(int p)
             return -1;
             }
         }
-    else if (!isspace(get(p)))
+    else if (!isWhitespace(get(p)))
         break;
     else
         p++;
@@ -1622,7 +1622,7 @@ bool CssParser::parseFile(const DOMString &fileName)
         int ch = fgetc(f);
         if (ch<0)
             break;
-        str.push_back(ch);
+        str.push_back((XMLCh)ch);
         }
     fclose(f);
 
