@@ -402,7 +402,7 @@ public:
      * This is the big one. Called by the xpath-dom api to fetch
      * nodes from a DOM tree.
      */
-    NodeList evaluate(const Node *root, const DOMString &str);
+    NodeList evaluate(const NodePtr root, const DOMString &str);
 
 
 
@@ -515,7 +515,16 @@ private:
     /**
      * Add a newly derived token to the token list;
      */
-    void tokAdd(Token *token);
+    void tokAdd(const Token &token);
+
+    void tokAdd(int type);
+
+    void tokAdd(int type, long val);
+
+    void tokAdd(int type, double val);
+
+    void tokAdd(int type, const DOMString &val);
+
 
     /**
      * The grammar definitions marked [1]-[39] are directly
