@@ -11,39 +11,24 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 
-#include <string.h>
 #include <signal.h>
 #include <errno.h>
 
 #include "libnr/n-art-bpath.h"
-#include "libnr/nr-point-matrix-ops.h"
-#include "libnr/nr-rect.h"
-#include "libnr/nr-matrix.h"
-#include "libnr/nr-matrix-fns.h"
-#include "libnr/nr-path.h"
-#include "libnr/nr-pixblock.h"
-#include "display/canvas-bpath.h"
 #include "sp-item.h"
 
-#include <glib.h>
-#include <gtk/gtkdialog.h>
-#include <gtk/gtkbox.h>
-#include <gtk/gtkstock.h>
 
-#include <glibmm/i18n.h>
-#include "enums.h"
-#include "document.h"
 #include "style.h"
-#include "sp-paint-server.h"
-#include "inkscape_version.h"
 
 #include "latex-pstricks.h"
 
 #include <unit-constants.h>
 
-#include "extension/extension.h"
 #include "extension/system.h"
 #include "extension/print.h"
 
@@ -82,6 +67,8 @@ PrintLatex::begin (Inkscape::Extension::Print *mod, SPDocument *doc)
     FILE *osf, *osp;
     const gchar * fn;
 
+    os.setf(std::ios::fixed);
+
     fn = mod->get_param_string("destination");
 
     osf = NULL;
@@ -229,6 +216,8 @@ PrintLatex::fill(Inkscape::Extension::Print *mod,
         Inkscape::SVGOStringStream os;
         float rgb[3];
 
+        os.setf(std::ios::fixed);
+
         sp_color_get_rgb_floatv(&style->fill.value.color, rgb);
         os << "{\n\\newrgbcolor{curcolor}{" << rgb[0] << " " << rgb[1] << " " << rgb[2] << "}\n";
 
@@ -254,6 +243,8 @@ PrintLatex::stroke (Inkscape::Extension::Print *mod, const NRBPath *bpath, const
         Inkscape::SVGOStringStream os;
         float rgb[3];
 
+        os.setf(std::ios::fixed);
+
         sp_color_get_rgb_floatv(&style->stroke.value.color, rgb);
         os << "{\n\\newrgbcolor{curcolor}{" << rgb[0] << " " << rgb[1] << " " << rgb[2] << "}\n";
 
@@ -342,6 +333,8 @@ PrintLatex::textToPath(Inkscape::Extension::Print * ext)
     return ext->get_param_bool("textToPath");
 }
 
+#include "clear-n_.h"
+
 void
 PrintLatex::init (void)
 {
@@ -350,7 +343,7 @@ PrintLatex::init (void)
 	/* SVG in */
     ext = Inkscape::Extension::build_from_mem(
 		"<inkscape-extension>\n"
-			"<name>LaTeX Print</name>\n"
+			"<name>" N_("LaTeX Print") "</name>\n"
 			"<id>" SP_MODULE_KEY_PRINT_LATEX "</id>\n"
         		"<param name=\"destination\" type=\"string\"></param>\n"
                         "<param name=\"textToPath\" type=\"boolean\">TRUE</param>\n"
