@@ -7,6 +7,7 @@
  * Authors:
  *   Ted Gould <ted@gould.cx>
  *
+ * Copyright (C) 2006 Johan Engelen <johan@shouraizou.nl>
  * Copyright (C) 2002-2004 Authors
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
@@ -103,8 +104,8 @@ open(Extension *key, gchar const *filename)
     /* This kinda overkill as most of these are already set, but I want
        to make sure for this release -- TJG */
     Inkscape::XML::Node *repr = sp_document_repr_root(doc);
-    gboolean saved = sp_document_get_undo_sensitive(doc);
-    sp_document_set_undo_sensitive(doc, FALSE);
+    bool saved = sp_document_get_undo_sensitive(doc);
+    sp_document_set_undo_sensitive(doc, false);
     repr->setAttribute("sodipodi:modified", NULL);
     sp_document_set_undo_sensitive(doc, saved);
 
@@ -244,12 +245,25 @@ save(Extension *key, SPDocument *doc, gchar const *filename, bool setextension,
         throw Output::no_overwrite();
     }
 
-    if (official) {
-        sp_document_set_uri(doc, fileName);
-    }
-
     omod->save(doc, fileName);
 
+    if (official) {
+        // save the filename for next use
+        sp_document_set_uri(doc, fileName);
+        bool saved = sp_document_get_undo_sensitive(doc);
+        sp_document_set_undo_sensitive (doc, false);
+            // also save the extension for next use
+            Inkscape::XML::Node *repr = sp_document_repr_root(doc);
+            repr->setAttribute("inkscape:output_extension", omod->get_id());
+            // set the "dataloss" attribute if the chosen extension is lossy
+    	    repr->setAttribute("inkscape:dataloss", NULL);
+            if ( omod->causes_dataloss() ) {
+	            repr->setAttribute("inkscape:dataloss", "true");
+            }
+            repr->setAttribute("sodipodi:modified", NULL);
+    	sp_document_set_undo_sensitive (doc, saved);
+    }
+    
     g_free(fileName);
     return;
 }
