@@ -8,9 +8,10 @@
  *   Chema Celorio <chema@celorio.com>
  *   bulia byak <buliabyak@users.sf.net>
  *
+ * Copyright (C) 2006 Johan Engelen <johan@shouraizou.nl>
  * Copyright (C) 1999-2005 Authors
- * Copyright (C) 2001-2002 Ximian, Inc.
  * Copyright (C) 2004 David Turner
+ * Copyright (C) 2001-2002 Ximian, Inc.
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
@@ -43,7 +44,7 @@
 #include "print.h"
 #include "file.h"
 #include "message-stack.h"
-#include "dialogs/filedialog.h"
+#include "ui/dialog/filedialog.h"
 #include "prefs-utils.h"
 #include "path-prefix.h"
 
@@ -66,13 +67,6 @@
 #include "jabber_whiteboard/session-manager.h"
 #endif
 
-/**
- * 'Current' paths.  Used to remember which directory
- * had the last file accessed.
- * Static globals are evil.  This will be gone soon
- * as C++ification continues
- */
-static gchar *import_path = NULL;
 
 //#define INK_DUMP_FILENAME_CONV 1
 #undef INK_DUMP_FILENAME_CONV
@@ -92,9 +86,12 @@ void dump_ustr(Glib::ustring const &ustr);
  * Create a blank document and add it to the desktop
  */
 SPDesktop*
-sp_file_new(gchar const *templ)
+sp_file_new(const Glib::ustring &templ)
 {
-    SPDocument *doc = sp_document_new(templ, TRUE, true);
+    char *templName = NULL;
+    if (templ.size()>0)
+        templName = (char *)templ.c_str();
+    SPDocument *doc = sp_document_new(templName, TRUE, true);
     g_return_val_if_fail(doc != NULL, NULL);
 
     SPDesktop *dt;
@@ -109,6 +106,7 @@ sp_file_new(gchar const *templ)
         sp_create_window(dtw, TRUE);
         dt = static_cast<SPDesktop*>(dtw->view);
         sp_namedview_window_from_document(dt);
+        sp_namedview_update_layers_from_document(dt);
     }
     return dt;
 }
@@ -137,7 +135,7 @@ sp_file_new_default()
         sources.pop_front();
     }
 
-    return sp_file_new(NULL);
+    return sp_file_new("");
 }
 
 
@@ -167,11 +165,13 @@ sp_file_exit()
  *  will replace the empty one.
  */
 bool
-sp_file_open(gchar const *uri, Inkscape::Extension::Extension *key, bool add_to_recent, bool replace_empty)
+sp_file_open(const Glib::ustring &uri,
+             Inkscape::Extension::Extension *key,
+             bool add_to_recent, bool replace_empty)
 {
-    SPDocument *doc;
+    SPDocument *doc = NULL;
     try {
-        doc = Inkscape::Extension::open(key, uri);
+        doc = Inkscape::Extension::open(key, uri.c_str());
     } catch (Inkscape::Extension::Input::no_extension_found &e) {
         doc = NULL;
     } catch (Inkscape::Extension::Input::open_failed &e) {
@@ -202,12 +202,8 @@ sp_file_open(gchar const *uri, Inkscape::Extension::Extension *key, bool add_to_
         // everyone who cares now has a reference, get rid of ours
         sp_document_unref(doc);
         // resize the window to match the document properties
-        // (this may be redundant for new windows... if so, move to the "virgin"
-        //  section above)
-#ifdef WITH_INKBOARD
-		desktop->whiteboard_session_manager()->setDesktop(desktop);
-#endif
         sp_namedview_window_from_document(desktop);
+        sp_namedview_update_layers_from_document(desktop);
 
         if (add_to_recent) {
             prefs_set_recent_file(SP_DOCUMENT_URI(doc), SP_DOCUMENT_NAME(doc));
@@ -215,7 +211,7 @@ sp_file_open(gchar const *uri, Inkscape::Extension::Extension *key, bool add_to_
 
         return TRUE;
     } else {
-        gchar *safeUri = Inkscape::IO::sanitizeString(uri);
+        gchar *safeUri = Inkscape::IO::sanitizeString(uri.c_str());
         gchar *text = g_strdup_printf(_("Failed to load the requested file %s"), safeUri);
         sp_ui_error_dialog(text);
         g_free(text);
@@ -355,7 +351,7 @@ void dump_ustr(Glib::ustring const &ustr)
     g_message("---------------");
 }
 
-static Inkscape::UI::Dialogs::FileOpenDialog *openDialogInstance = NULL;
+static Inkscape::UI::Dialog::FileOpenDialog *openDialogInstance = NULL;
 
 /**
  *  Display an file Open selector.  Open a document if OK is pressed.
@@ -364,134 +360,86 @@ static Inkscape::UI::Dialogs::FileOpenDialog *openDialogInstance = NULL;
 void
 sp_file_open_dialog(gpointer object, gpointer data)
 {
-    gchar *open_path2 = NULL;
 
-    gchar *open_path = g_strdup(prefs_get_string_attribute("dialogs.open", "path"));
-    if (open_path != NULL && open_path[0] == '\0') {
-        g_free(open_path);
-        open_path = NULL;
-    }
-    if (open_path && !Inkscape::IO::file_test(open_path, (GFileTest)(G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))) {
-        g_free(open_path);
-        open_path = NULL;
-    }
-    if (open_path == NULL)
-        open_path = g_strconcat(g_get_home_dir(), G_DIR_SEPARATOR_S, NULL);
+    //# Get the current directory for finding files
+    Glib::ustring open_path;
+    char *attr = (char *)prefs_get_string_attribute("dialogs.open", "path");
+    if (attr)
+        open_path = attr;
 
-    if (!openDialogInstance) {
-        openDialogInstance =
-              Inkscape::UI::Dialogs::FileOpenDialog::create(
-                 (char const *)open_path,
-                 Inkscape::UI::Dialogs::SVG_TYPES,
-                 (char const *)_("Select file to open"));
-    }
-    bool const success = openDialogInstance->show();
-    gchar *fileName = ( success
-                        ? g_strdup(openDialogInstance->getFilename())
-                        : NULL );
-    Inkscape::Extension::Extension *selection =
-            openDialogInstance->getSelectionType();
-    g_free(open_path);
 
-    if (!success) return;
+    //# Test if the open_path directory exists  
+    if (!Inkscape::IO::file_test(open_path.c_str(),
+              (GFileTest)(G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)))
+        open_path = "";
 
-    // Code to check & open iff multiple files.
-    Glib::SListHandle<Glib::ustring> flist=openDialogInstance->getFilenames();
-    GSList *list=flist.data();
-
-    if(g_slist_length(list)>1)
-    {
-        gchar *fileName=NULL;
-
-        while(list!=NULL)
+    //# If no open path, default to our home directory
+    if (open_path.size() < 1)
         {
-
-#ifdef INK_DUMP_FILENAME_CONV
-            g_message(" FileName: %s",(const char *)list->data);
-#endif
-
-            fileName=(gchar *)g_strdup((gchar *)list->data);
-
-            if (fileName && !g_file_test(fileName,G_FILE_TEST_IS_DIR)) {
-                gsize bytesRead = 0;
-                gsize bytesWritten = 0;
-                GError *error = NULL;
-#ifdef INK_DUMP_FILENAME_CONV
-                dump_str( fileName, "A file pre  is " );
-#endif
-                gchar *newFileName = g_filename_to_utf8(fileName,
-                                                -1,
-                                                        &bytesRead,
-                                                        &bytesWritten,
-                                                        &error);
-                if ( newFileName != NULL ) {
-                    g_free(fileName);
-                    fileName = newFileName;
-#ifdef INK_DUMP_FILENAME_CONV
-                    dump_str( fileName, "A file post is " );
-#endif
-                } else {
-                    // TODO: bulia, please look over
-                    g_warning( "ERROR CONVERTING OPEN FILENAME TO UTF-8" );
-                }
-
-#ifdef INK_DUMP_FILENAME_CONV
-                g_message("Opening File %s\n",fileName);
-#endif
-
-                sp_file_open(fileName, selection);
-                g_free(fileName);
-            }
-            else
-            {
-                g_message("Cannot Open Directory %s\n",fileName);
-            }
-
-            list=list->next;
+        open_path = g_get_home_dir();
+        open_path.append(G_DIR_SEPARATOR_S);
         }
 
+    //# Create a dialog if we don't already have one
+    if (!openDialogInstance) {
+        openDialogInstance =
+              Inkscape::UI::Dialog::FileOpenDialog::create(
+                 open_path,
+                 Inkscape::UI::Dialog::SVG_TYPES,
+                 (char const *)_("Select file to open"));
+    }
+
+    //# Show the dialog
+    bool const success = openDialogInstance->show();
+    if (!success)
+        return;
+
+    //# User selected something.  Get name and type
+    Glib::ustring fileName = openDialogInstance->getFilename();
+    Inkscape::Extension::Extension *selection =
+            openDialogInstance->getSelectionType();
+
+    //# Code to check & open iff multiple files.
+    std::vector<Glib::ustring> flist=openDialogInstance->getFilenames();
+
+    //# Iterate through filenames if more than 1
+    if (flist.size() > 1)
+        {
+        for (unsigned int i=1 ; i<flist.size() ; i++)
+            {
+            Glib::ustring fName = flist[i];
+
+            if (Glib::file_test(fileName, Glib::FILE_TEST_IS_DIR)) {
+            Glib::ustring newFileName = Glib::filename_to_utf8(fName);
+            if ( newFileName.size() > 0 )
+                fName = newFileName;
+            else
+                g_warning( "ERROR CONVERTING OPEN FILENAME TO UTF-8" );
+
+#ifdef INK_DUMP_FILENAME_CONV
+            g_message("Opening File %s\n",fileName);
+#endif
+            sp_file_open(fileName, selection);
+            }
+        }
         return;
     }
 
 
-    if (fileName) {
-        gsize bytesRead = 0;
-        gsize bytesWritten = 0;
-        GError *error = NULL;
-#ifdef INK_DUMP_FILENAME_CONV
-        dump_str( fileName, "A file pre  is " );
-#endif
-        gchar *newFileName = g_filename_to_utf8(fileName,
-                                                -1,
-                                                &bytesRead,
-                                                &bytesWritten,
-                                                &error);
-        if ( newFileName != NULL ) {
-            g_free(fileName);
+    if (fileName.size() > 0) {
+
+        Glib::ustring newFileName = Glib::filename_to_utf8(fileName);
+
+        if ( newFileName.size() > 0)
             fileName = newFileName;
-#ifdef INK_DUMP_FILENAME_CONV
-            dump_str( fileName, "A file post is " );
-#endif
-        } else {
-            // TODO: bulia, please look over
+        else
             g_warning( "ERROR CONVERTING OPEN FILENAME TO UTF-8" );
-        }
 
-
-        if ( !g_utf8_validate(fileName, -1, NULL) ) {
-            // TODO: bulia, please look over
-            g_warning( "INPUT FILENAME IS NOT UTF-8" );
-        }
-
-
-        open_path = g_dirname(fileName);
-        open_path2 = g_strconcat(open_path, G_DIR_SEPARATOR_S, NULL);
-        prefs_set_string_attribute("dialogs.open", "path", open_path2);
-        g_free(open_path);
-        g_free(open_path2);
+        open_path = fileName;
+        open_path.append(G_DIR_SEPARATOR_S);
+        prefs_set_string_attribute("dialogs.open", "path", open_path.c_str());
 
         sp_file_open(fileName, selection);
-        g_free(fileName);
     }
 
     return;
@@ -514,7 +462,8 @@ sp_file_vacuum()
 
     unsigned int diff = vacuum_document (doc);
 
-    sp_document_done(doc);
+    sp_document_done(doc, SP_VERB_FILE_VACUUM, 
+                     _("Vacuum &lt;defs&gt;"));
 
     SPDesktop *dt = SP_ACTIVE_DESKTOP;
     if (diff > 0) {
@@ -536,19 +485,23 @@ sp_file_vacuum()
 
 /**
  * This 'save' function called by the others below
+ *
+ * \param    official  whether to set :output_module and :modified in the
+ *                     document; is true for normal save, false for temporary saves
  */
 static bool
-file_save(SPDocument *doc, gchar const *uri, Inkscape::Extension::Extension *key, bool saveas)
+file_save(SPDocument *doc, const Glib::ustring &uri,
+          Inkscape::Extension::Extension *key, bool saveas, bool official)
 {
-    if (!doc || !uri) //Safety check
-        return FALSE;
+    if (!doc || uri.size()<1) //Safety check
+        return false;
 
     try {
-        Inkscape::Extension::save(key, doc, uri,
-                                  saveas && prefs_get_int_attribute("dialogs.save_as", "append_extension", 1),
-                                  saveas, TRUE); // save officially, with inkscape: attributes set
+        Inkscape::Extension::save(key, doc, uri.c_str(),
+                 saveas && prefs_get_int_attribute("dialogs.save_as", "append_extension", 1),
+                 saveas, official); 
     } catch (Inkscape::Extension::Output::no_extension_found &e) {
-        gchar *safeUri = Inkscape::IO::sanitizeString(uri);
+        gchar *safeUri = Inkscape::IO::sanitizeString(uri.c_str());
         gchar *text = g_strdup_printf(_("No Inkscape extension found to save document (%s).  This may have been caused by an unknown filename extension."), safeUri);
         SP_ACTIVE_DESKTOP->messageStack()->flash(Inkscape::ERROR_MESSAGE, _("Document not saved."));
         sp_ui_error_dialog(text);
@@ -556,7 +509,7 @@ file_save(SPDocument *doc, gchar const *uri, Inkscape::Extension::Extension *key
         g_free(safeUri);
         return FALSE;
     } catch (Inkscape::Extension::Output::save_failed &e) {
-        gchar *safeUri = Inkscape::IO::sanitizeString(uri);
+        gchar *safeUri = Inkscape::IO::sanitizeString(uri.c_str());
         gchar *text = g_strdup_printf(_("File %s could not be saved."), safeUri);
         SP_ACTIVE_DESKTOP->messageStack()->flash(Inkscape::ERROR_MESSAGE, _("Document not saved."));
         sp_ui_error_dialog(text);
@@ -568,175 +521,159 @@ file_save(SPDocument *doc, gchar const *uri, Inkscape::Extension::Extension *key
     }
 
     SP_ACTIVE_DESKTOP->messageStack()->flash(Inkscape::NORMAL_MESSAGE, _("Document saved."));
-    return TRUE;
+    return true;
 }
 
-static Inkscape::UI::Dialogs::FileSaveDialog *saveDialogInstance = NULL;
+
+
+
+
+static Inkscape::UI::Dialog::FileSaveDialog *saveDialogInstance = NULL;
 
 /**
  *  Display a SaveAs dialog.  Save the document if OK pressed.
+ *
+ * \param    ascopy  (optional) wether to set the documents->uri to the new filename or not
  */
-gboolean
-sp_file_save_dialog(SPDocument *doc)
+bool
+sp_file_save_dialog(SPDocument *doc, bool is_copy)
 {
-    Inkscape::XML::Node *repr = sp_document_repr_root(doc);
-    gchar const *default_extension = NULL;
-    gchar *save_loc;
-    Inkscape::Extension::Output *extension;
-    gchar *save_path = NULL;
 
-    default_extension = repr->attribute("inkscape:output_extension");
-    if (default_extension == NULL) {
-        default_extension = prefs_get_string_attribute("dialogs.save_as", "default");
-    }
-    //g_warning("%s: extension name: '%s'", __FUNCTION__, default_extension);
+    Inkscape::XML::Node *repr = sp_document_repr_root(doc);
+
+    Inkscape::Extension::Output *extension;
+
+    //# Get the default extension name
+    Glib::ustring default_extension;
+    char *attr = (char *)repr->attribute("inkscape:output_extension");
+    if (!attr)
+        attr = (char *)prefs_get_string_attribute("dialogs.save_as", "default");
+    if (attr)
+        default_extension = attr;
+    //g_message("%s: extension name: '%s'", __FUNCTION__, default_extension);
+
+    Glib::ustring save_path;
+    Glib::ustring save_loc;
 
     if (doc->uri == NULL) {
+        char formatBuf[256];
         int i = 1;
-        char const *filename_extension;
-        char *temp_filename;
 
-        extension = dynamic_cast<Inkscape::Extension::Output *>(Inkscape::Extension::db.get(default_extension));
+        Glib::ustring filename_extension = ".svg";
+        extension = dynamic_cast<Inkscape::Extension::Output *>
+              (Inkscape::Extension::db.get(default_extension.c_str()));
         //g_warning("%s: extension ptr: 0x%x", __FUNCTION__, (unsigned int)extension);
-        if (extension == NULL) {
-            filename_extension = ".svg";
-        } else {
+        if (extension)
             filename_extension = extension->get_extension();
-        }
 
-        save_path = g_strdup(prefs_get_string_attribute("dialogs.save_as", "path"));
-        if (save_path != NULL && save_path[0] == '\0') {
-            g_free(save_path);
-            save_path = NULL;
-        }
-        if (save_path && !Inkscape::IO::file_test(save_path, (GFileTest)(G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))) {
-            g_free(save_path);
-            save_path = NULL;
-        }
-        if (save_path == NULL)
-            save_path = g_strdup(g_get_home_dir());
-        temp_filename = g_strdup_printf(_("drawing%s"), filename_extension);
-        save_loc = g_build_filename(save_path, temp_filename, NULL);
-        g_free(temp_filename);
+        attr = (char *)prefs_get_string_attribute("dialogs.save_as", "path");
+        if (attr)
+            save_path = attr;
 
-        while (Inkscape::IO::file_test(save_loc, G_FILE_TEST_EXISTS)) {
-            g_free(save_loc);
-            temp_filename = g_strdup_printf(_("drawing-%d%s"), i++, filename_extension);
-            save_loc = g_build_filename(save_path, temp_filename, NULL);
-            g_free(temp_filename);
+        if (!Inkscape::IO::file_test(save_path.c_str(),
+              (GFileTest)(G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)))
+            save_path = "";
+
+        if (save_path.size()<1)
+            save_path = g_get_home_dir();
+
+        save_loc = save_path;
+        save_loc.append(G_DIR_SEPARATOR_S);
+        snprintf(formatBuf, 255, _("drawing%s"), filename_extension.c_str());
+        save_loc.append(formatBuf);
+
+        while (Inkscape::IO::file_test(save_loc.c_str(), G_FILE_TEST_EXISTS)) {
+            save_loc = save_path;
+            save_loc.append(G_DIR_SEPARATOR_S);
+            snprintf(formatBuf, 255, _("drawing-%d%s"), i++, filename_extension.c_str());
+            save_loc.append(formatBuf);
         }
     } else {
-        save_loc = g_path_get_dirname(doc->uri); /* \todo should use a getter */
+        save_loc = Glib::build_filename(Glib::path_get_dirname(doc->uri),
+                                        Glib::path_get_basename(doc->uri));
     }
 
-    { // convert save_loc from utf-8 to locale
-      // is this needed any more, now that everything is handled in
-      // Inkscape::IO?
-        gsize bytesRead = 0;
-        gsize bytesWritten = 0;
-        GError* error = NULL;
-#ifdef INK_DUMP_FILENAME_CONV
-        dump_str( save_loc, "B file pre  is " );
-#endif
-        gchar* save_loc_local = g_filename_from_utf8( save_loc, -1, &bytesRead, &bytesWritten, &error);
+    // convert save_loc from utf-8 to locale
+    // is this needed any more, now that everything is handled in
+    // Inkscape::IO?
+    Glib::ustring save_loc_local = Glib::filename_from_utf8(save_loc);
 
-        if ( save_loc_local != NULL ) {
-            g_free(save_loc);
-            save_loc = save_loc_local;
-#ifdef INK_DUMP_FILENAME_CONV
-            dump_str( save_loc, "B file post is " );
-#endif
-        } else {
-            //g_warning( "Error converting save filename stored in the file to locale encoding.");
-        }
+    if ( save_loc_local.size() > 0) 
+        save_loc = save_loc_local;
+
+    //# Show the SaveAs dialog
+    char const * dialog_title;
+    if (is_copy) {
+        dialog_title = (char const *) _("Select file to save a copy to");
+    } else {
+        dialog_title = (char const *) _("Select file to save to");
     }
-
-    if (!saveDialogInstance) {
+    if (!saveDialogInstance)
         saveDialogInstance =
-             Inkscape::UI::Dialogs::FileSaveDialog::create(
-                 (char const *) save_loc,
-                 Inkscape::UI::Dialogs::SVG_TYPES,
+             Inkscape::UI::Dialog::FileSaveDialog::create(
+                 save_loc,
+                 Inkscape::UI::Dialog::SVG_TYPES,
                  (char const *) _("Select file to save to"),
                  default_extension
             );
-    } // FIXME: else (i.e. if reshowing an already shown dialog) save_loc is not used, it thus always displays the previously opened dir
+    else
+        saveDialogInstance->change_path(save_loc);
+    saveDialogInstance->change_title(dialog_title);
+    
     bool success = saveDialogInstance->show();
-    char *fileName = ( success
-                       ? g_strdup(saveDialogInstance->getFilename())
-                       : NULL );
+    if (!success)
+        return success;
+
+    Glib::ustring fileName = saveDialogInstance->getFilename();
+
     Inkscape::Extension::Extension *selectionType =
         saveDialogInstance->getSelectionType();
-    g_free(save_loc);
-    g_free(save_path);
-    if (!success) {
-        return success;
-    }
 
-    if (fileName && *fileName) {
-        gsize bytesRead = 0;
-        gsize bytesWritten = 0;
-        GError *error = NULL;
-#ifdef INK_DUMP_FILENAME_CONV
-        dump_str( fileName, "C file pre  is " );
-#endif
-        gchar *newFileName = g_filename_to_utf8(fileName,
-                                                -1,
-                                                &bytesRead,
-                                                &bytesWritten,
-                                                &error);
-        if ( newFileName != NULL ) {
-            g_free(fileName);
+
+    if (fileName.size() > 0) {
+        Glib::ustring newFileName = Glib::filename_to_utf8(fileName);
+
+        if ( newFileName.size()>0 )
             fileName = newFileName;
-#ifdef INK_DUMP_FILENAME_CONV
-            dump_str( fileName, "C file post is " );
-#endif
-        } else {
+        else
             g_warning( "Error converting save filename to UTF-8." );
-        }
 
-        if (!g_utf8_validate(fileName, -1, NULL)) {
-            // TODO: bulia, please look over
-            g_warning( "The filename is not UTF-8." );
-        }
+        success = file_save(doc, fileName, selectionType, TRUE, !is_copy);
 
-        success = file_save(doc, fileName, selectionType, TRUE);
-
-        if (success) {
+        if (success)
             prefs_set_recent_file(SP_DOCUMENT_URI(doc), SP_DOCUMENT_NAME(doc));
-        }
 
-        save_path = g_dirname(fileName);
-        prefs_set_string_attribute("dialogs.save_as", "path", save_path);
-        g_free(save_path);
+        save_path = Glib::path_get_dirname(fileName);
+        prefs_set_string_attribute("dialogs.save_as", "path", save_path.c_str());
 
-        g_free(fileName);
         return success;
-    } else {
-        return FALSE;
     }
+
+
+    return false;
 }
 
 
 /**
  * Save a document, displaying a SaveAs dialog if necessary.
  */
-gboolean
+bool
 sp_file_save_document(SPDocument *doc)
 {
-    gboolean success = TRUE;
+    bool success = true;
 
     Inkscape::XML::Node *repr = sp_document_repr_root(doc);
 
     gchar const *fn = repr->attribute("sodipodi:modified");
     if (fn != NULL) {
-        if (doc->uri == NULL
-            || repr->attribute("inkscape:output_extension") == NULL)
+        if ( doc->uri == NULL
+            || repr->attribute("inkscape:output_extension") == NULL )
         {
-            return sp_file_save_dialog(doc);
+            return sp_file_save_dialog(doc, FALSE);
         } else {
             fn = g_strdup(doc->uri);
             gchar const *ext = repr->attribute("inkscape:output_extension");
-            success = file_save(doc, fn, Inkscape::Extension::db.get(ext), FALSE);
+            success = file_save(doc, fn, Inkscape::Extension::db.get(ext), FALSE, TRUE);
             g_free((void *) fn);
         }
     } else {
@@ -770,11 +707,23 @@ sp_file_save_as(gpointer object, gpointer data)
     if (!SP_ACTIVE_DOCUMENT)
         return false;
     sp_namedview_document_from_window(SP_ACTIVE_DESKTOP);
-    return sp_file_save_dialog(SP_ACTIVE_DOCUMENT);
+    return sp_file_save_dialog(SP_ACTIVE_DOCUMENT, FALSE);
 }
 
 
 
+/**
+ *  Save a copy of a document, always displaying a sort of SaveAs dialog.
+ */
+bool
+sp_file_save_a_copy(gpointer object, gpointer data)
+{
+    if (!SP_ACTIVE_DOCUMENT)
+        return false;
+    sp_namedview_document_from_window(SP_ACTIVE_DESKTOP);
+    return sp_file_save_dialog(SP_ACTIVE_DOCUMENT, TRUE);
+}
+
 
 /*######################
 ## I M P O R T
@@ -784,14 +733,15 @@ sp_file_save_as(gpointer object, gpointer data)
  *  Import a resource.  Called by sp_file_import()
  */
 void
-file_import(SPDocument *in_doc, gchar const *uri, Inkscape::Extension::Extension *key)
+file_import(SPDocument *in_doc, const Glib::ustring &uri,
+               Inkscape::Extension::Extension *key)
 {
     SPDesktop *desktop = SP_ACTIVE_DESKTOP;
 
     //DEBUG_MESSAGE( fileImport, "file_import( in_doc:%p uri:[%s], key:%p", in_doc, uri, key );
     SPDocument *doc;
     try {
-        doc = Inkscape::Extension::open(key, uri);
+        doc = Inkscape::Extension::open(key, uri.c_str());
     } catch (Inkscape::Extension::Input::no_extension_found &e) {
         doc = NULL;
     } catch (Inkscape::Extension::Input::open_failed &e) {
@@ -898,10 +848,11 @@ file_import(SPDocument *in_doc, gchar const *uri, Inkscape::Extension::Extension
         }
 
         sp_document_unref(doc);
-        sp_document_done(in_doc);
+        sp_document_done(in_doc, SP_VERB_FILE_IMPORT,
+                         _("Import"));
 
     } else {
-        gchar *text = g_strdup_printf(_("Failed to load the requested file %s"), uri);
+        gchar *text = g_strdup_printf(_("Failed to load the requested file %s"), uri.c_str());
         sp_ui_error_dialog(text);
         g_free(text);
     }
@@ -910,7 +861,7 @@ file_import(SPDocument *in_doc, gchar const *uri, Inkscape::Extension::Extension
 }
 
 
-static Inkscape::UI::Dialogs::FileOpenDialog *importDialogInstance = NULL;
+static Inkscape::UI::Dialog::FileOpenDialog *importDialogInstance = NULL;
 
 /**
  *  Display an Open dialog, import a resource if OK pressed.
@@ -918,60 +869,44 @@ static Inkscape::UI::Dialogs::FileOpenDialog *importDialogInstance = NULL;
 void
 sp_file_import(GtkWidget *widget)
 {
+    static Glib::ustring import_path;
+
     SPDocument *doc = SP_ACTIVE_DOCUMENT;
     if (!doc)
         return;
 
     if (!importDialogInstance) {
         importDialogInstance =
-             Inkscape::UI::Dialogs::FileOpenDialog::create(
-                 (char const *)import_path,
-                 Inkscape::UI::Dialogs::IMPORT_TYPES,
+             Inkscape::UI::Dialog::FileOpenDialog::create(
+                 import_path,
+                 Inkscape::UI::Dialog::IMPORT_TYPES,
                  (char const *)_("Select file to import"));
     }
+
     bool success = importDialogInstance->show();
-    char *fileName = ( success
-                       ? g_strdup(importDialogInstance->getFilename())
-                       : NULL );
+    if (!success)
+        return;
+
+    //# Get file name and extension type
+    Glib::ustring fileName = importDialogInstance->getFilename();
     Inkscape::Extension::Extension *selection =
         importDialogInstance->getSelectionType();
 
-    if (!success) return;
-    if (fileName) {
-        gsize bytesRead = 0;
-        gsize bytesWritten = 0;
-        GError *error = NULL;
-#ifdef INK_DUMP_FILENAME_CONV
-        dump_str( fileName, "D file pre  is " );
-#endif
-        gchar *newFileName = g_filename_to_utf8( fileName,
-                                                 -1,
-                                                 &bytesRead,
-                                                 &bytesWritten,
-                                                 &error);
-        if ( newFileName != NULL ) {
-            g_free(fileName);
+
+    if (fileName.size() > 0) {
+ 
+        Glib::ustring newFileName = Glib::filename_to_utf8(fileName);
+
+        if ( newFileName.size() > 0)
             fileName = newFileName;
-#ifdef INK_DUMP_FILENAME_CONV
-            dump_str( fileName, "D file post is " );
-#endif
-        } else {
-            // TODO: bulia, please look over
+        else
             g_warning( "ERROR CONVERTING OPEN FILENAME TO UTF-8" );
-        }
 
-
-        if (!g_utf8_validate(fileName, -1, NULL)) {
-            // TODO: bulia, please look over
-            g_warning( "INPUT FILENAME IS NOT UTF-8" );
-        }
-
-        g_free(import_path);
-        import_path = g_dirname(fileName);
-        if (import_path) import_path = g_strconcat(import_path, G_DIR_SEPARATOR_S, NULL);
+        import_path = fileName;
+        if (import_path.size()>0)
+            import_path.append(G_DIR_SEPARATOR_S);
 
         file_import(doc, fileName, selection);
-        g_free(fileName);
     }
 
     return;
@@ -983,209 +918,143 @@ sp_file_import(GtkWidget *widget)
 ## E X P O R T
 ######################*/
 
+//#define NEW_EXPORT_DIALOG
+
+
+
+#ifdef NEW_EXPORT_DIALOG
+
+static Inkscape::UI::Dialog::FileExportDialog *exportDialogInstance = NULL;
+
+/**
+ *  Display an Export dialog, export as the selected type if OK pressed
+ */
+bool
+sp_file_export_dialog(void *widget)
+{
+    //# temp hack for 'doc' until we can switch to this dialog
+    SPDocument *doc = SP_ACTIVE_DOCUMENT;
+
+    Glib::ustring export_path; 
+    Glib::ustring export_loc; 
+
+    Inkscape::XML::Node *repr = sp_document_repr_root(doc);
+
+    Inkscape::Extension::Output *extension;
+
+    //# Get the default extension name
+    Glib::ustring default_extension;
+    char *attr = (char *)repr->attribute("inkscape:output_extension");
+    if (!attr)
+        attr = (char *)prefs_get_string_attribute("dialogs.save_as", "default");
+    if (attr)
+        default_extension = attr;
+    //g_message("%s: extension name: '%s'", __FUNCTION__, default_extension);
+
+    if (doc->uri == NULL)
+        {
+        char formatBuf[256];
+
+        Glib::ustring filename_extension = ".svg";
+        extension = dynamic_cast<Inkscape::Extension::Output *>
+              (Inkscape::Extension::db.get(default_extension.c_str()));
+        //g_warning("%s: extension ptr: 0x%x", __FUNCTION__, (unsigned int)extension);
+        if (extension)
+            filename_extension = extension->get_extension();
+
+        attr = (char *)prefs_get_string_attribute("dialogs.save_as", "path");
+        if (attr)
+            export_path = attr;
+
+        if (!Inkscape::IO::file_test(export_path.c_str(),
+              (GFileTest)(G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)))
+            export_path = "";
+
+        if (export_path.size()<1)
+            export_path = g_get_home_dir();
+
+        export_loc = export_path;
+        export_loc.append(G_DIR_SEPARATOR_S);
+        snprintf(formatBuf, 255, _("drawing%s"), filename_extension.c_str());
+        export_loc.append(formatBuf);
+
+        }
+    else
+        {
+        export_path = Glib::path_get_dirname(doc->uri);
+        }
+
+    // convert save_loc from utf-8 to locale
+    // is this needed any more, now that everything is handled in
+    // Inkscape::IO?
+    Glib::ustring export_path_local = Glib::filename_from_utf8(export_path);
+    if ( export_path_local.size() > 0) 
+        export_path = export_path_local;
+
+    //# Show the SaveAs dialog
+    if (!exportDialogInstance)
+        exportDialogInstance =
+             Inkscape::UI::Dialog::FileExportDialog::create(
+                 export_path,
+                 Inkscape::UI::Dialog::EXPORT_TYPES,
+                 (char const *) _("Select file to export to"),
+                 default_extension
+            );
+
+    bool success = exportDialogInstance->show();
+    if (!success)
+        return success;
+
+    Glib::ustring fileName = exportDialogInstance->getFilename();
+
+    Inkscape::Extension::Extension *selectionType =
+        exportDialogInstance->getSelectionType();
+
+
+    if (fileName.size() > 0) {
+        Glib::ustring newFileName = Glib::filename_to_utf8(fileName);
+
+        if ( newFileName.size()>0 )
+            fileName = newFileName;
+        else
+            g_warning( "Error converting save filename to UTF-8." );
+
+        success = file_save(doc, fileName, selectionType, TRUE, FALSE);
+
+        if (success)
+            prefs_set_recent_file(SP_DOCUMENT_URI(doc), SP_DOCUMENT_NAME(doc));
+
+        export_path = fileName;
+        prefs_set_string_attribute("dialogs.save_as", "path", export_path.c_str());
+
+        return success;
+    }
+
+
+    return false;
+}
+
+
+
+
+
+
+
+#else
+
+
+
 /**
  *
  */
-void
+bool
 sp_file_export_dialog(void *widget)
 {
     sp_export_dialog();
+    return true;
 }
 
-#include <display/nr-arena-item.h>
-#include <display/nr-arena.h>
-
-struct SPEBP {
-    int width, height, sheight;
-    guchar r, g, b, a;
-    NRArenaItem *root; // the root arena item to show; it is assumed that all unneeded items are hidden
-    guchar *px;
-    unsigned (*status)(float, void *);
-    void *data;
-};
-
-
-/**
- *
- */
-static int
-sp_export_get_rows(guchar const **rows, int row, int num_rows, void *data)
-{
-    struct SPEBP *ebp = (struct SPEBP *) data;
-
-    if (ebp->status) {
-        if (!ebp->status((float) row / ebp->height, ebp->data)) return 0;
-    }
-
-    num_rows = MIN(num_rows, ebp->sheight);
-    num_rows = MIN(num_rows, ebp->height - row);
-
-    /* Set area of interest */
-    NRRectL bbox;
-    bbox.x0 = 0;
-    bbox.y0 = row;
-    bbox.x1 = ebp->width;
-    bbox.y1 = row + num_rows;
-    /* Update to renderable state */
-    NRGC gc(NULL);
-    nr_matrix_set_identity(&gc.transform);
-
-    nr_arena_item_invoke_update(ebp->root, &bbox, &gc, NR_ARENA_ITEM_STATE_ALL, NR_ARENA_ITEM_STATE_NONE);
-
-    NRPixBlock pb;
-    nr_pixblock_setup_extern(&pb, NR_PIXBLOCK_MODE_R8G8B8A8N,
-                             bbox.x0, bbox.y0, bbox.x1, bbox.y1,
-                             ebp->px, 4 * ebp->width, FALSE, FALSE);
-
-    for (int r = 0; r < num_rows; r++) {
-        guchar *p = NR_PIXBLOCK_PX(&pb) + r * pb.rs;
-        for (int c = 0; c < ebp->width; c++) {
-            *p++ = ebp->r;
-            *p++ = ebp->g;
-            *p++ = ebp->b;
-            *p++ = ebp->a;
-        }
-    }
-
-    /* Render */
-    nr_arena_item_invoke_render(ebp->root, &bbox, &pb, 0);
-
-    for (int r = 0; r < num_rows; r++) {
-        rows[r] = NR_PIXBLOCK_PX(&pb) + r * pb.rs;
-    }
-
-    nr_pixblock_release(&pb);
-
-    return num_rows;
-}
-
-/**
-Hide all items which are not listed in list, recursively, skipping groups and defs
-*/
-void
-hide_other_items_recursively(SPObject *o, GSList *list, unsigned dkey)
-{
-    if (SP_IS_ITEM(o)
-        && !SP_IS_DEFS(o)
-        && !SP_IS_ROOT(o)
-        && !SP_IS_GROUP(o)
-        && !g_slist_find(list, o))
-    {
-        sp_item_invoke_hide(SP_ITEM(o), dkey);
-    }
-
-     // recurse
-    if (!g_slist_find(list, o)) {
-        for (SPObject *child = sp_object_first_child(o) ; child != NULL; child = SP_OBJECT_NEXT(child) ) {
-            hide_other_items_recursively(child, list, dkey);
-        }
-    }
-}
-
-
-/**
- *  Render the SVG drawing onto a PNG raster image, then save to
- *  a file.  Returns TRUE if succeeded in writing the file,
- *  FALSE otherwise.
- */
-int
-sp_export_png_file(SPDocument *doc, gchar const *filename,
-                   double x0, double y0, double x1, double y1,
-                   unsigned width, unsigned height, double xdpi, double ydpi,
-                   unsigned long bgcolor,
-                   unsigned (*status)(float, void *),
-                   void *data, bool force_overwrite,
-                   GSList *items_only)
-{
-    int write_status = TRUE;
-    g_return_val_if_fail(doc != NULL, FALSE);
-    g_return_val_if_fail(filename != NULL, FALSE);
-    g_return_val_if_fail(width >= 1, FALSE);
-    g_return_val_if_fail(height >= 1, FALSE);
-
-    if (!force_overwrite && !sp_ui_overwrite_file(filename)) {
-        return FALSE;
-    }
-
-    sp_document_ensure_up_to_date(doc);
-
-    /* Go to document coordinates */
-    gdouble t = y0;
-    y0 = sp_document_height(doc) - y1;
-    y1 = sp_document_height(doc) - t;
-
-    /*
-     * 1) a[0] * x0 + a[2] * y1 + a[4] = 0.0
-     * 2) a[1] * x0 + a[3] * y1 + a[5] = 0.0
-     * 3) a[0] * x1 + a[2] * y1 + a[4] = width
-     * 4) a[1] * x0 + a[3] * y0 + a[5] = height
-     * 5) a[1] = 0.0;
-     * 6) a[2] = 0.0;
-     *
-     * (1,3) a[0] * x1 - a[0] * x0 = width
-     * a[0] = width / (x1 - x0)
-     * (2,4) a[3] * y0 - a[3] * y1 = height
-     * a[3] = height / (y0 - y1)
-     * (1) a[4] = -a[0] * x0
-     * (2) a[5] = -a[3] * y1
-     */
-
-    NRMatrix affine;
-    affine.c[0] = width / (x1 - x0);
-    affine.c[1] = 0.0;
-    affine.c[2] = 0.0;
-    affine.c[3] = height / (y1 - y0);
-    affine.c[4] = -affine.c[0] * x0;
-    affine.c[5] = -affine.c[3] * y0;
-
-    //SP_PRINT_MATRIX("SVG2PNG", &affine);
-
-    struct SPEBP ebp;
-    ebp.width  = width;
-    ebp.height = height;
-    ebp.r      = NR_RGBA32_R(bgcolor);
-    ebp.g      = NR_RGBA32_G(bgcolor);
-    ebp.b      = NR_RGBA32_B(bgcolor);
-    ebp.a      = NR_RGBA32_A(bgcolor);
-
-    /* Create new arena */
-    NRArena *arena = NRArena::create();
-    unsigned dkey = sp_item_display_key_new(1);
-
-    /* Create ArenaItems and set transform */
-    ebp.root = sp_item_invoke_show(SP_ITEM(sp_document_root(doc)), arena, dkey, SP_ITEM_SHOW_DISPLAY);
-    nr_arena_item_set_transform(NR_ARENA_ITEM(ebp.root), NR::Matrix(&affine));
-
-    // We show all and then hide all items we don't want, instead of showing only requested items,
-    // because that would not work if the shown item references something in defs
-    if (items_only) {
-        hide_other_items_recursively(sp_document_root(doc), items_only, dkey);
-    }
-
-    ebp.status = status;
-    ebp.data   = data;
-
-    if ((width < 256) || ((width * height) < 32768)) {
-        ebp.px = nr_pixelstore_64K_new(FALSE, 0);
-        ebp.sheight = 65536 / (4 * width);
-        write_status = sp_png_write_rgba_striped(filename, width, height, xdpi, ydpi, sp_export_get_rows, &ebp);
-        nr_pixelstore_64K_free(ebp.px);
-    } else {
-        ebp.px = g_new(guchar, 4 * 64 * width);
-        ebp.sheight = 64;
-        write_status = sp_png_write_rgba_striped(filename, width, height, xdpi, ydpi, sp_export_get_rows, &ebp);
-        g_free(ebp.px);
-    }
-
-    // Hide items
-    sp_item_invoke_hide(SP_ITEM(sp_document_root(doc)), dkey);
-
-    /* Free Arena and ArenaItem */
-    nr_arena_item_unref(ebp.root);
-    nr_object_unref((NRObject *) arena);
-    return write_status;
-}
-
+#endif
 
 /*######################
 ## P R I N T
