@@ -19,32 +19,29 @@
  * - put in eof
  */
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 
-#include <cstdio>
 //#ifdef STDC_HEADERS
-#include <cstdlib>
 //#endif
 
 //#ifdef HAVE_UNISTD_H
-#include <unistd.h>
 //#endif
 
 //#ifdef HAVE_SYS_PARAM_H
-#include <sys/param.h>
 //#else
 //#define MAXPATHLEN 1024
 //#endif
 
 //#ifdef HAVE_DIRENT_H
-#include <dirent.h>
 //#endif
 
 //#ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 //#endif
 
-#include <cstring>
 
 #include <glib.h>
 
