@@ -19,44 +19,30 @@
 # include "config.h"
 #endif
 
-#include <string.h>
-#include <vector>
-#include <glib.h>
-#include <glib-object.h>
 #include <gtk/gtk.h>
-#include "inkscape.h"
 #include "inkscape-private.h"
-#include "extension/db.h"
 #include "extension/effect.h"
 #include "widgets/icon.h"
 #include "prefs-utils.h"
 #include "path-prefix.h"
 
-#include "verbs.h"
 #include "shortcuts.h"
 
 #include "document.h"
 #include "desktop-handles.h"
 #include "file.h"
-#include "help.h"
 #include "interface.h"
-#include "widgets/toolbox.h"
 #include "desktop.h"
-#include "object-ui.h"
+#include "ui/context-menu.h"
 #include "selection.h"
 #include "selection-chemistry.h"
-#include "path-chemistry.h"
-#include "zoom-context.h"
 #include "svg-view-widget.h"
 #include "widgets/desktop-widget.h"
 #include "sp-item-group.h"
 #include "sp-namedview.h"
 
-#include "dir-util.h"
-#include "memeq.h"
 #include "helper/action.h"
 #include "helper/gnome-utils.h"
-#include <glibmm/i18n.h>
 #include "helper/window.h"
 
 #include "io/sys.h"
@@ -67,6 +53,16 @@
 
 #include "message-context.h"
 
+// Added for color drag-n-drop
+#if ENABLE_LCMS
+#include "lcms.h"
+#endif // ENABLE_LCMS
+#include "display/sp-canvas.h"
+#include "color.h"
+#include "svg/svg-color.h"
+#include "desktop-style.h"
+#include "style.h"
+
 
 using Inkscape::IO::StringOutputStream;
 using Inkscape::IO::Base64OutputStream;
@@ -81,7 +77,9 @@ typedef enum {
     SVG_DATA,
     PNG_DATA,
     JPEG_DATA,
-    IMAGE_DATA
+    IMAGE_DATA,
+    APP_X_INKY_COLOR,
+    APP_X_COLOR
 } ui_drop_target_info;
 
 static GtkTargetEntry ui_drop_target_entries [] = {
@@ -90,6 +88,10 @@ static GtkTargetEntry ui_drop_target_entries [] = {
     {"image/svg",     0, SVG_DATA},
     {"image/png",     0, PNG_DATA},
     {"image/jpeg",    0, JPEG_DATA},
+#if ENABLE_MAGIC_COLORS
+    {"application/x-inkscape-color", 0, APP_X_INKY_COLOR},
+#endif // ENABLE_MAGIC_COLORS
+    {"application/x-color", 0, APP_X_COLOR}
 };
 
 static GtkTargetEntry *completeDropTargets = 0;
@@ -110,38 +112,35 @@ static void sp_ui_drag_data_received(GtkWidget *widget,
 static void sp_ui_menu_item_set_sensitive(SPAction *action,
                                           unsigned int sensitive,
                                           void *data);
+static void sp_ui_menu_item_set_name(SPAction *action, 
+                                     Glib::ustring name,
+                                     void *data);
 
 SPActionEventVector menu_item_event_vector = {
     {NULL},
     NULL,
     NULL, /* set_active */
     sp_ui_menu_item_set_sensitive, /* set_sensitive */
-    NULL  /* set_shortcut */
+    NULL, /* set_shortcut */
+    sp_ui_menu_item_set_name /* set_name */
 };
 
 void
 sp_create_window(SPViewWidget *vw, gboolean editable)
 {
-    GtkWidget *w, *hb;
-
     g_return_if_fail(vw != NULL);
     g_return_if_fail(SP_IS_VIEW_WIDGET(vw));
 
-    w = sp_window_new("", TRUE);
-    
+    GtkWidget *w = sp_window_new("", TRUE);
+
     if (editable) {
       g_object_set_data(G_OBJECT(vw), "window", w);
-      reinterpret_cast<SPDesktopWidget*>(vw)->window = 
+      reinterpret_cast<SPDesktopWidget*>(vw)->window =
         static_cast<GtkWindow*>((void*)w);
     }
 
-    hb = gtk_hbox_new(FALSE, 0);
-    gtk_widget_show(hb);
-    gtk_container_add(GTK_CONTAINER(w), hb);
-    g_object_set_data(G_OBJECT(w), "hbox", hb);
-
-    /* fixme: */
     if (editable) {
+        /* fixme: */
         gtk_window_set_default_size((GtkWindow *) w, 640, 480);
         g_object_set_data(G_OBJECT(w), "desktop", SP_DESKTOP_WIDGET(vw)->desktop);
         g_object_set_data(G_OBJECT(w), "desktopwidget", vw);
@@ -151,10 +150,9 @@ sp_create_window(SPViewWidget *vw, gboolean editable)
         gtk_window_set_policy(GTK_WINDOW(w), TRUE, TRUE, TRUE);
     }
 
-    gtk_box_pack_end(GTK_BOX(hb), GTK_WIDGET(vw), TRUE, TRUE, 0);
+    gtk_container_add(GTK_CONTAINER(w), GTK_WIDGET(vw));
     gtk_widget_show(GTK_WIDGET(vw));
 
-
     if ( completeDropTargets == 0 || completeDropTargetsCount == 0 )
     {
         std::vector<gchar*> types;
@@ -215,6 +213,7 @@ sp_ui_new_view()
 
     sp_create_window(dtw, TRUE);
     sp_namedview_window_from_document(static_cast<SPDesktop*>(dtw->view));
+    sp_namedview_update_layers_from_document(static_cast<SPDesktop*>(dtw->view));
 }
 
 /* TODO: not yet working */
@@ -235,19 +234,19 @@ sp_ui_new_view_preview()
     sp_create_window(dtw, FALSE);
 }
 
+/**
+ * \param widget unused
+ */
 void
 sp_ui_close_view(GtkWidget *widget)
 {
-    GtkWidget *w;
-
     if (SP_ACTIVE_DESKTOP == NULL) {
         return;
     }
-    w = static_cast<GtkWidget*>((void*)SP_DT_WIDGET(SP_ACTIVE_DESKTOP)->window);
     if ((SP_ACTIVE_DESKTOP)->shutdown()) {
         return;
     }
-    gtk_widget_destroy(w);
+    SP_ACTIVE_DESKTOP->destroyWidget();
 }
 
 
@@ -268,29 +267,11 @@ sp_ui_close_all(void)
     /* Iterate through all the windows, destroying each in the order they
        become active */
     while (SP_ACTIVE_DESKTOP) {
-        GtkWidget *w;
-        w = static_cast<GtkWidget*>((void*)SP_DT_WIDGET(SP_ACTIVE_DESKTOP)->window);
         if ((SP_ACTIVE_DESKTOP)->shutdown()) {
             /* The user cancelled the operation, so end doing the close */
             return FALSE;
         }
-        //gtk_widget_destroy(w);
-
-        // Spackle for Bug [ 1217361 ] "freeze on quitting with transform dialog"
-        // The following code is intended to be temporary and IS NOT NEEDED once this part
-        // of Inkscape is de-bugged, when the single line above should be re-instituted.
-
-        // At present, SP_ACTIVE_DESKTOP does not guarantee to return a gtk_widget: If we find
-        // that we have been issued an object other than a widget we assume that something is
-        // wrong upstream and quit immediately.
-        if ( GTK_IS_WIDGET(w) ) {
-            //fprintf( stderr, "sp_ui_close_all( ) About to call  gtk_widget_destroy( %08p )\n", w );
-            gtk_widget_destroy(w);
-        } else {
-            g_warning( "Desktop has passed us a non-widget! : %p", w );
-            gtk_main_quit();
-            break;
-        }
+        SP_ACTIVE_DESKTOP->destroyWidget();
     }
 
     return TRUE;
@@ -352,7 +333,7 @@ sp_ui_menuitem_add_icon( GtkWidget *item, gchar *icon_name )
 {
     GtkWidget *icon;
 
-    icon = sp_icon_new( GTK_ICON_SIZE_MENU, icon_name );
+    icon = sp_icon_new( Inkscape::ICON_SIZE_MENU, icon_name );
     gtk_widget_show(icon);
     gtk_image_menu_item_set_image((GtkImageMenuItem *) item, icon);
 } // end of sp_ui_menu_add_icon
@@ -398,22 +379,6 @@ sp_ui_menu_append_item( GtkMenu *menu, gchar const *stock,
 
 } // end of sp_ui_menu_append_item()
 
-
-static void
-sp_ui_menu_key_press(GtkMenuItem *item, GdkEventKey *event, void *data)
-{
-    if (event->state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK)) {
-        unsigned int shortcut;
-
-        shortcut = get_group0_keyval(event);
-        if (event->state & GDK_SHIFT_MASK) shortcut |= SP_SHORTCUT_SHIFT_MASK;
-        if (event->state & GDK_CONTROL_MASK) shortcut |= SP_SHORTCUT_CONTROL_MASK;
-        if (event->state & GDK_MOD1_MASK) shortcut |= SP_SHORTCUT_ALT_MASK;
-        sp_shortcut_set(shortcut, (Inkscape::Verb *)data, true);
-    }
-}
-
-
 /**
 \brief  a wrapper around gdk_keyval_name producing (when possible) characters, not names
  */
@@ -452,7 +417,7 @@ sp_ui_shortcut_string(unsigned const shortcut, gchar *const c)
 {
     /* TODO: This function shouldn't exist.  Our callers should use GtkAccelLabel instead of
      * a generic GtkLabel containing this string, and should call gtk_widget_add_accelerator.
-     * Will probably need to change sp_shortcut_invoke callers and sp_ui_menu_key_press.
+     * Will probably need to change sp_shortcut_invoke callers.
      *
      * The existing gtk_label_new_with_mnemonic call can be replaced with
      * g_object_new(GTK_TYPE_ACCEL_LABEL, NULL) followed by
@@ -522,7 +487,7 @@ sp_ui_dialog_title_string(Inkscape::Verb *verb, gchar *c)
  */
 
 static GtkWidget *
-sp_ui_menu_append_item_from_verb(GtkMenu *menu, Inkscape::Verb *verb, Inkscape::UI::View::View *view)
+sp_ui_menu_append_item_from_verb(GtkMenu *menu, Inkscape::Verb *verb, Inkscape::UI::View::View *view, bool radio = false, GSList *group = NULL)
 {
     SPAction *action;
     GtkWidget *item;
@@ -543,17 +508,30 @@ sp_ui_menu_append_item_from_verb(GtkMenu *menu, Inkscape::Verb *verb, Inkscape::
             gchar c[256];
             sp_ui_shortcut_string(shortcut, c);
             GtkWidget *const hb = gtk_hbox_new(FALSE, 16);
-            GtkWidget *const name_lbl = gtk_label_new_with_mnemonic(action->name);
+            GtkWidget *const name_lbl = gtk_label_new("");
+            gtk_label_set_markup_with_mnemonic(GTK_LABEL(name_lbl), action->name);
             gtk_misc_set_alignment((GtkMisc *) name_lbl, 0.0, 0.5);
             gtk_box_pack_start((GtkBox *) hb, name_lbl, TRUE, TRUE, 0);
             GtkWidget *const accel_lbl = gtk_label_new(c);
             gtk_misc_set_alignment((GtkMisc *) accel_lbl, 1.0, 0.5);
             gtk_box_pack_end((GtkBox *) hb, accel_lbl, FALSE, FALSE, 0);
             gtk_widget_show_all(hb);
-            item = gtk_image_menu_item_new();
+            if (radio) {
+                item = gtk_radio_menu_item_new (group);
+            } else {
+                item = gtk_image_menu_item_new();
+            }
             gtk_container_add((GtkContainer *) item, hb);
         } else {
-            item = gtk_image_menu_item_new_with_mnemonic(action->name);
+            if (radio) {
+                item = gtk_radio_menu_item_new (group);
+            } else {
+                item = gtk_image_menu_item_new ();
+            }
+            GtkWidget *const name_lbl = gtk_label_new("");
+            gtk_label_set_markup_with_mnemonic(GTK_LABEL(name_lbl), action->name);
+            gtk_misc_set_alignment((GtkMisc *) name_lbl, 0.0, 0.5);
+            gtk_container_add((GtkContainer *) item, name_lbl);
         }
 
         nr_active_object_add_listener((NRActiveObject *)action, (NRObjectEventVector *)&menu_item_event_vector, sizeof(SPActionEventVector), item);
@@ -567,8 +545,6 @@ sp_ui_menu_append_item_from_verb(GtkMenu *menu, Inkscape::Verb *verb, Inkscape::
         gtk_widget_set_events(item, GDK_KEY_PRESS_MASK);
         g_signal_connect( G_OBJECT(item), "activate",
                           G_CALLBACK(sp_ui_menu_activate), action );
-        g_signal_connect( G_OBJECT(item), "key_press_event",
-                          G_CALLBACK(sp_ui_menu_key_press), (void *) verb);
 
         g_signal_connect( G_OBJECT(item), "select", G_CALLBACK(sp_ui_menu_select_action), action );
         g_signal_connect( G_OBJECT(item), "deselect", G_CALLBACK(sp_ui_menu_deselect_action), action );
@@ -589,7 +565,7 @@ checkitem_toggled(GtkCheckMenuItem *menuitem, gpointer user_data)
     Inkscape::UI::View::View *view = (Inkscape::UI::View::View *) g_object_get_data(G_OBJECT(menuitem), "view");
 
     gchar const *pref_path;
-    if (static_cast<SPDesktop*>(view)->is_fullscreen)
+    if (reinterpret_cast<SPDesktop*>(view)->is_fullscreen)
         pref_path = g_strconcat("fullscreen.", pref, NULL);
     else
         pref_path = g_strconcat("window.", pref, NULL);
@@ -597,7 +573,7 @@ checkitem_toggled(GtkCheckMenuItem *menuitem, gpointer user_data)
     gboolean checked = gtk_check_menu_item_get_active(menuitem);
     prefs_set_int_attribute(pref_path, "state", checked);
 
-    sp_desktop_widget_layout(static_cast<SPDesktop*>(view)->owner);
+    reinterpret_cast<SPDesktop*>(view)->layoutWidget();
 }
 
 static gboolean
@@ -702,41 +678,58 @@ sp_file_new_from_template(GtkWidget *widget, gchar const *uri)
 void
 sp_menu_append_new_templates(GtkWidget *menu, Inkscape::UI::View::View *view)
 {
-    GDir *dir = g_dir_open(INKSCAPE_TEMPLATESDIR, 0, NULL);
-    if (!dir)
-        return;
-    for (gchar const *file = g_dir_read_name(dir); file != NULL; file = g_dir_read_name(dir)) {
-        if (!g_str_has_suffix(file, ".svg"))
-            continue; // skip non-svg files
+    std::list<gchar *> sources;
+    sources.push_back( profile_path("templates") ); // first try user's local dir
+    sources.push_back( g_strdup(INKSCAPE_TEMPLATESDIR) ); // then the system templates dir
 
-        gchar *basename = g_path_get_basename(file);
-        if (g_str_has_suffix(basename, ".svg") && g_str_has_prefix(basename, "default."))
-            continue; // skip default.*.svg (i.e. default.svg and translations) - it's in the menu already
+    // Use this loop to iterate through a list of possible document locations.
+    while (!sources.empty()) {
+        gchar *dirname = sources.front();
 
-        gchar const *filepath = g_build_filename(INKSCAPE_TEMPLATESDIR, file, NULL);
-        gchar *dupfile = g_strndup(file, strlen(file) - 4);
-        gchar *filename =  g_filename_to_utf8(dupfile,  -1, NULL, NULL, NULL);
-        g_free(dupfile);
-        GtkWidget *item = gtk_menu_item_new_with_label(filename);
-        g_free(filename);
+        if ( Inkscape::IO::file_test( dirname, (GFileTest)(G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR) ) ) {
+            GError *err = 0;
+            GDir *dir = g_dir_open(dirname, 0, &err);
 
-        gtk_widget_show(item);
-        // how does "filepath" ever get freed?
-        g_signal_connect(G_OBJECT(item),
-                         "activate",
-                         G_CALLBACK(sp_file_new_from_template),
-                         (gpointer) filepath);
+            if (dir) {
+                for (gchar const *file = g_dir_read_name(dir); file != NULL; file = g_dir_read_name(dir)) {
+                    if (!g_str_has_suffix(file, ".svg") && !g_str_has_suffix(file, ".svgz"))
+                        continue; // skip non-svg files
 
-        if (view) {
-            // set null tip for now; later use a description from the template file
-            g_object_set_data(G_OBJECT(item), "view", (gpointer) view);
-            g_signal_connect( G_OBJECT(item), "select", G_CALLBACK(sp_ui_menu_select), (gpointer) NULL );
-            g_signal_connect( G_OBJECT(item), "deselect", G_CALLBACK(sp_ui_menu_deselect), NULL);
+                    gchar *basename = g_path_get_basename(file);
+                    if (g_str_has_suffix(basename, ".svg") && g_str_has_prefix(basename, "default."))
+                        continue; // skip default.*.svg (i.e. default.svg and translations) - it's in the menu already
+
+                    gchar const *filepath = g_build_filename(dirname, file, NULL);
+                    gchar *dupfile = g_strndup(file, strlen(file) - 4);
+                    gchar *filename =  g_filename_to_utf8(dupfile,  -1, NULL, NULL, NULL);
+                    g_free(dupfile);
+                    GtkWidget *item = gtk_menu_item_new_with_label(filename);
+                    g_free(filename);
+
+                    gtk_widget_show(item);
+                    // how does "filepath" ever get freed?
+                    g_signal_connect(G_OBJECT(item),
+                                     "activate",
+                                     G_CALLBACK(sp_file_new_from_template),
+                                     (gpointer) filepath);
+
+                    if (view) {
+                        // set null tip for now; later use a description from the template file
+                        g_object_set_data(G_OBJECT(item), "view", (gpointer) view);
+                        g_signal_connect( G_OBJECT(item), "select", G_CALLBACK(sp_ui_menu_select), (gpointer) NULL );
+                        g_signal_connect( G_OBJECT(item), "deselect", G_CALLBACK(sp_ui_menu_deselect), NULL);
+                    }
+
+                    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+                }
+                g_dir_close(dir);
+            }
         }
 
-        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+        // toss the dirname
+        g_free(dirname);
+        sources.pop_front();
     }
-    g_dir_close(dir);
 }
 
 void
@@ -768,30 +761,6 @@ sp_menu_append_recent_documents(GtkWidget *menu, Inkscape::UI::View::View* /* vi
     }
 }
 
-/** Creates the effects menu.
-    \param  menu  The menu to append to.
-    \param  doc   Document being used.
-    \param  view  The view of the document this is being attached to.
-*/
-static void
-sp_ui_effect_menu(GtkMenu *menu, SPDocument *doc, Inkscape::UI::View::View *view)
-{
-    Inkscape::Extension::DB::EffectList effectlist;
-    Inkscape::Extension::db.get_effect_list(effectlist);
-
-    for (Inkscape::Extension::DB::EffectList::iterator current_item = effectlist.begin();
-         current_item != effectlist.end(); current_item++) {
-        Inkscape::Extension::Effect *emod = *current_item;
-
-        Inkscape::Verb *verb = emod->get_verb();
-
-        if (verb != NULL)
-            sp_ui_menu_append_item_from_verb(GTK_MENU(menu), verb, view);
-    }
-
-    return;
-}
-
 void
 sp_ui_checkboxes_menus(GtkMenu *m, Inkscape::UI::View::View *view)
 {
@@ -799,7 +768,7 @@ sp_ui_checkboxes_menus(GtkMenu *m, Inkscape::UI::View::View *view)
     //                                       checkitem_toggled, checkitem_update, 0);
     sp_ui_menu_append_check_item_from_verb(m, view, _("Commands Bar"), _("Show or hide the Commands bar (under the menu)"), "commands",
                                            checkitem_toggled, checkitem_update, 0);
-    sp_ui_menu_append_check_item_from_verb(m, view, _("Tool Controls"), _("Show or hide the Tool Controls panel"), "toppanel",
+    sp_ui_menu_append_check_item_from_verb(m, view, _("Tool Controls Bar"), _("Show or hide the Tool Controls bar"), "toppanel",
                                            checkitem_toggled, checkitem_update, 0);
     sp_ui_menu_append_check_item_from_verb(m, view, _("_Toolbox"), _("Show or hide the main toolbox (on the left)"), "toolbox",
                                            checkitem_toggled, checkitem_update, 0);
@@ -807,6 +776,8 @@ sp_ui_checkboxes_menus(GtkMenu *m, Inkscape::UI::View::View *view)
                                            checkitem_toggled, checkitem_update, Inkscape::Verb::get(SP_VERB_TOGGLE_RULERS));
     sp_ui_menu_append_check_item_from_verb(m, view, NULL, NULL, "scrollbars",
                                            checkitem_toggled, checkitem_update, Inkscape::Verb::get(SP_VERB_TOGGLE_SCROLLBARS));
+    sp_ui_menu_append_check_item_from_verb(m, view, _("_Palette"), _("Show or hide the color palette"), "panels",
+                                           checkitem_toggled, checkitem_update, 0);
     sp_ui_menu_append_check_item_from_verb(m, view, _("_Statusbar"), _("Show or hide the statusbar (at the bottom of the window)"), "statusbar",
                                            checkitem_toggled, checkitem_update, 0);
 }
@@ -831,14 +802,12 @@ sp_ui_build_dyn_menus(Inkscape::XML::Node *menus, GtkWidget *menu, Inkscape::UI:
 {
     if (menus == NULL) return;
     if (menu == NULL)  return;
+    GSList *group = NULL;
 
     for (Inkscape::XML::Node *menu_pntr = menus;
          menu_pntr != NULL;
          menu_pntr = menu_pntr->next()) {
         if (!strcmp(menu_pntr->name(), "submenu")) {
-            if (!strcmp(menu_pntr->attribute("name"), "Effects") && !prefs_get_int_attribute("extensions", "show-effects-menu", 0)) {
-                continue;
-            }
             GtkWidget *mitem = gtk_menu_item_new_with_mnemonic(_(menu_pntr->attribute("name")));
             GtkWidget *submenu = gtk_menu_new();
             sp_ui_build_dyn_menus(menu_pntr->firstChild(), submenu, view);
@@ -851,7 +820,16 @@ sp_ui_build_dyn_menus(Inkscape::XML::Node *menus, GtkWidget *menu, Inkscape::UI:
             Inkscape::Verb *verb = Inkscape::Verb::getbyid(verb_name);
 
             if (verb != NULL) {
-                sp_ui_menu_append_item_from_verb(GTK_MENU(menu), verb, view);
+                if (menu_pntr->attribute("radio") != NULL) {
+                    GtkWidget *item = sp_ui_menu_append_item_from_verb (GTK_MENU(menu), verb, view, true, group);
+                    group = gtk_radio_menu_item_get_group( GTK_RADIO_MENU_ITEM(item));
+                    if (menu_pntr->attribute("default") != NULL) {
+                        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item), TRUE);
+                    }
+                } else {
+                    sp_ui_menu_append_item_from_verb(GTK_MENU(menu), verb, view);
+                    group = NULL;
+                }
             } else {
                 gchar string[120];
                 g_snprintf(string, 120, _("Verb \"%s\" Unknown"), verb_name);
@@ -863,7 +841,11 @@ sp_ui_build_dyn_menus(Inkscape::XML::Node *menus, GtkWidget *menu, Inkscape::UI:
             }
             continue;
         }
-        if (!strcmp(menu_pntr->name(), "seperator")) {
+        if (!strcmp(menu_pntr->name(), "separator")
+                // This was spelt wrong in the original version
+                // and so this is for backward compatibility.  It can
+                // probably be dropped after the 0.44 release.
+             || !strcmp(menu_pntr->name(), "seperator")) {
             GtkWidget *item = gtk_separator_menu_item_new();
             gtk_widget_show(item);
             gtk_menu_append(GTK_MENU(menu), item);
@@ -877,10 +859,6 @@ sp_ui_build_dyn_menus(Inkscape::XML::Node *menus, GtkWidget *menu, Inkscape::UI:
             sp_menu_append_recent_documents(menu, view);
             continue;
         }
-        if (!strcmp(menu_pntr->name(), "effects-list")) {
-            sp_ui_effect_menu(GTK_MENU(menu), NULL, view);
-            continue;
-        }
         if (!strcmp(menu_pntr->name(), "objects-checkboxes")) {
             sp_ui_checkboxes_menus(GTK_MENU(menu), view);
             continue;
@@ -911,7 +889,7 @@ static void leave_group(GtkMenuItem *, SPDesktop *desktop) {
 
 static void enter_group(GtkMenuItem *mi, SPDesktop *desktop) {
     desktop->setCurrentLayer(reinterpret_cast<SPObject *>(g_object_get_data(G_OBJECT(mi), "group")));
-    SP_DT_SELECTION(desktop)->clear();
+    sp_desktop_selection(desktop)->clear();
 }
 
 GtkWidget *
@@ -997,6 +975,120 @@ sp_ui_drag_data_received(GtkWidget *widget,
                          gpointer user_data)
 {
     switch (info) {
+#if ENABLE_MAGIC_COLORS
+        case APP_X_INKY_COLOR:
+        {
+            SPDesktop *desktop = SP_ACTIVE_DESKTOP;
+            int destX = 0;
+            int destY = 0;
+            gtk_widget_translate_coordinates( widget, &(desktop->canvas->widget), x, y, &destX, &destY );
+            NR::Point where( sp_canvas_window_to_world( desktop->canvas, NR::Point( destX, destY ) ) );
+
+            SPItem *item = desktop->item_at_point( where, true );
+            if ( item )
+            {
+                if ( data->length >= 8 ) {
+                    cmsHPROFILE srgbProf = cmsCreate_sRGBProfile();
+
+                    gchar c[64] = {0};
+                    // Careful about endian issues.
+                    guint16* dataVals = (guint16*)data->data;
+                    sp_svg_write_color( c, 64,
+                                        SP_RGBA32_U_COMPOSE(
+                                            0x0ff & (dataVals[0] >> 8),
+                                            0x0ff & (dataVals[1] >> 8),
+                                            0x0ff & (dataVals[2] >> 8),
+                                            0xff // can't have transparency in the color itself
+                                            //0x0ff & (data->data[3] >> 8),
+                                            ));
+                    SPCSSAttr *css = sp_repr_css_attr_new();
+                    bool updatePerformed = false;
+
+                    if ( data->length > 14 ) {
+                        int flags = dataVals[4];
+
+                        // piggie-backed palette entry info
+                        int index = dataVals[5];
+                        Glib::ustring palName;
+                        for ( int i = 0; i < dataVals[6]; i++ ) {
+                            palName += (gunichar)dataVals[7+i];
+                        }
+
+                        // Now hook in a magic tag of some sort.
+                        if ( !palName.empty() && (flags & 1) ) {
+                            gchar* str = g_strdup_printf("%d|", index);
+                            palName.insert( 0, str );
+                            g_free(str);
+                            str = 0;
+
+                            sp_object_setAttribute( SP_OBJECT(item),
+                                                    (drag_context->action != GDK_ACTION_MOVE) ? "inkscape:x-fill-tag":"inkscape:x-stroke-tag",
+                                                    palName.c_str(),
+                                                    false );
+                            item->updateRepr();
+
+                            sp_repr_css_set_property( css, (drag_context->action != GDK_ACTION_MOVE) ? "fill":"stroke", c );
+                            updatePerformed = true;
+                        }
+                    }
+
+                    if ( !updatePerformed ) {
+                        sp_repr_css_set_property( css, (drag_context->action != GDK_ACTION_MOVE) ? "fill":"stroke", c );
+                    }
+
+                    sp_desktop_apply_css_recursive( item, css, true );
+                    item->updateRepr();
+
+                    SPDocument *doc = SP_ACTIVE_DOCUMENT;
+                    sp_document_done( doc , SP_VERB_NONE, 
+                                      _("Drop color"));
+
+                    if ( srgbProf ) {
+                        cmsCloseProfile( srgbProf );
+                    }
+                }
+            }
+        }
+        break;
+#endif // ENABLE_MAGIC_COLORS
+
+        case APP_X_COLOR:
+        {
+            SPDesktop *desktop = SP_ACTIVE_DESKTOP;
+            int destX = 0;
+            int destY = 0;
+            gtk_widget_translate_coordinates( widget, &(desktop->canvas->widget), x, y, &destX, &destY );
+            NR::Point where( sp_canvas_window_to_world( desktop->canvas, NR::Point( destX, destY ) ) );
+
+            SPItem *item = desktop->item_at_point( where, true );
+            if ( item )
+            {
+                if ( data->length == 8 ) {
+                    gchar c[64] = {0};
+                    // Careful about endian issues.
+                    guint16* dataVals = (guint16*)data->data;
+                    sp_svg_write_color( c, 64,
+                                        SP_RGBA32_U_COMPOSE(
+                                            0x0ff & (dataVals[0] >> 8),
+                                            0x0ff & (dataVals[1] >> 8),
+                                            0x0ff & (dataVals[2] >> 8),
+                                            0xff // can't have transparency in the color itself
+                                            //0x0ff & (data->data[3] >> 8),
+                                            ));
+                    SPCSSAttr *css = sp_repr_css_attr_new();
+                    sp_repr_css_set_property( css, (drag_context->action != GDK_ACTION_MOVE) ? "fill":"stroke", c );
+
+                    sp_desktop_apply_css_recursive( item, css, true );
+                    item->updateRepr();
+
+                    SPDocument *doc = SP_ACTIVE_DOCUMENT;
+                    sp_document_done( doc , SP_VERB_NONE, 
+                                      _("Drop color"));
+                }
+            }
+        }
+        break;
+
         case SVG_DATA:
         case SVG_XML_DATA: {
             gchar *svgdata = (gchar *)data->data;
@@ -1014,7 +1106,7 @@ sp_ui_drag_data_received(GtkWidget *widget,
             gchar const *style = repr->attribute("style");
 
             Inkscape::XML::Node *newgroup = sp_repr_new("svg:g");
-            sp_repr_set_attr(newgroup, "style", style);
+            newgroup->setAttribute("style", style);
 
             for (Inkscape::XML::Node *child = repr->firstChild(); child != NULL; child = child->next()) {
                 Inkscape::XML::Node *newchild = child->duplicate();
@@ -1025,26 +1117,27 @@ sp_ui_drag_data_received(GtkWidget *widget,
 
             SPDesktop *desktop = SP_ACTIVE_DESKTOP;
             // Add it to the current layer
-           
+
             // Greg's edits to add intelligent positioning of svg drops
             SPObject *new_obj = NULL;
             new_obj = desktop->currentLayer()->appendChildRepr(newgroup);
 
-            Inkscape::Selection *selection = SP_DT_SELECTION(desktop);
+            Inkscape::Selection *selection = sp_desktop_selection(desktop);
             selection->set(SP_ITEM(new_obj));
             // To move the imported object, we must temporarily set the "transform pattern with
             // object" option.
             {
                 int const saved_pref = prefs_get_int_attribute("options.transform", "pattern", 1);
                 prefs_set_int_attribute("options.transform", "pattern", 1);
-                sp_document_ensure_up_to_date(SP_DT_DOCUMENT(desktop));
+                sp_document_ensure_up_to_date(sp_desktop_document(desktop));
                 NR::Point m( desktop->point() - selection->bounds().midpoint() );
                 sp_selection_move_relative(selection, m);
                 prefs_set_int_attribute("options.transform", "pattern", saved_pref);
             }
 
-            sp_repr_unref(newgroup);
-            sp_document_done(doc);
+            Inkscape::GC::release(newgroup);
+            sp_document_done(doc, SP_VERB_NONE, 
+                             _("Drop SVG"));
             break;
         }
 
@@ -1077,7 +1170,7 @@ sp_ui_drag_data_received(GtkWidget *widget,
 
             snprintf( tmp, sizeof(tmp), "data:%s;base64,", gdk_atom_name(data->type) );
             str.insert( 0, tmp );
-            sp_repr_set_attr( newImage, "xlink:href", str.c_str() );
+            newImage->setAttribute("xlink:href", str.c_str());
 
             GError *error = NULL;
             GdkPixbufLoader *loader = gdk_pixbuf_loader_new_with_mime_type( gdk_atom_name(data->type), &error );
@@ -1089,10 +1182,10 @@ sp_ui_drag_data_received(GtkWidget *widget,
                         int width = gdk_pixbuf_get_width(pbuf);
                         int height = gdk_pixbuf_get_height(pbuf);
                         snprintf( tmp, sizeof(tmp), "%d", width );
-                        sp_repr_set_attr( newImage, "width", tmp );
+                        newImage->setAttribute("width", tmp);
 
                         snprintf( tmp, sizeof(tmp), "%d", height );
-                        sp_repr_set_attr( newImage, "height", tmp );
+                        newImage->setAttribute("height", tmp);
                     }
                 }
             }
@@ -1102,8 +1195,9 @@ sp_ui_drag_data_received(GtkWidget *widget,
             // Add it to the current layer
             desktop->currentLayer()->appendChildRepr(newImage);
 
-            sp_repr_unref( newImage );
-            sp_document_done( doc );
+            Inkscape::GC::release(newImage);
+            sp_document_done( doc , SP_VERB_NONE, 
+                              _("Drop bitmap image"));
             break;
         }
     }
@@ -1184,7 +1278,10 @@ sp_ui_overwrite_file(gchar const *filename)
         gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
 
         hbox = gtk_hbox_new(FALSE, 5);
+
+        // TODO - replace with Inkscape-specific call
         boxdata = gtk_image_new_from_stock(GTK_STOCK_DIALOG_QUESTION, GTK_ICON_SIZE_DIALOG);
+
         gtk_widget_show(boxdata);
         gtk_box_pack_start(GTK_BOX(hbox), boxdata, TRUE, TRUE, 5);
         text = g_strdup_printf(_("The file %s already exists.  Do you want to overwrite that file with the current document?"), filename);
@@ -1217,6 +1314,15 @@ sp_ui_menu_item_set_sensitive(SPAction *action, unsigned int sensitive, void *da
     return gtk_widget_set_sensitive(GTK_WIDGET(data), sensitive);
 }
 
+static void
+sp_ui_menu_item_set_name(SPAction *action, Glib::ustring name, void *data)
+{
+    gtk_label_set_markup_with_mnemonic(
+        GTK_LABEL (gtk_container_get_children(GTK_CONTAINER (GTK_BIN (data)->child))->data), 
+        name.c_str());
+}
+
+
 /*
   Local Variables:
   mode:c++
