@@ -48,8 +48,6 @@
 #include <gtk/gtkwindow.h>
 #include <gtk/gtkbox.h>
 
-#include <gtk/gtkmain.h>
-
 #include "gc-core.h"
 
 #include "macros.h"
@@ -81,6 +79,8 @@
 
 #include "debug/logger.h"
 
+#include "helper/png-write.h"
+
 #include <extension/extension.h>
 #include <extension/system.h>
 #include <extension/db.h>
@@ -132,6 +132,7 @@ enum {
     SP_ARG_EXPORT_EPS,
     SP_ARG_EXPORT_PDF,
     SP_ARG_EXPORT_TEXT_TO_PATH,
+    SP_ARG_EXPORT_FONT,
     SP_ARG_EXPORT_BBOX_PAGE,
     SP_ARG_EXTENSIONDIR,
     SP_ARG_SLIDESHOW,
@@ -141,7 +142,6 @@ enum {
     SP_ARG_QUERY_HEIGHT,
     SP_ARG_QUERY_ID,
     SP_ARG_VERSION,
-    SP_ARG_NEW_GUI,
     SP_ARG_VACUUM_DEFS,
     SP_ARG_LAST
 };
@@ -174,6 +174,7 @@ static gchar *sp_export_ps = NULL;
 static gchar *sp_export_eps = NULL;
 static gchar *sp_export_pdf = NULL;
 static gboolean sp_export_text_to_path = FALSE;
+static gboolean sp_export_font = FALSE;
 static gboolean sp_export_bbox_page = FALSE;
 static gboolean sp_query_x = FALSE;
 static gboolean sp_query_y = FALSE;
@@ -309,6 +310,11 @@ struct poptOption options[] = {
      N_("Convert text object to paths on export (EPS)"),
      NULL},
 
+    {"export-embed-fonts", 'F',
+     POPT_ARG_NONE, &sp_export_font, SP_ARG_EXPORT_FONT,
+     N_("Embed fonts on export (Type 1 only) (EPS)"),
+     NULL},
+
     {"export-bbox-page", 'B',
      POPT_ARG_NONE, &sp_export_bbox_page, SP_ARG_EXPORT_BBOX_PAGE,
      N_("Export files with the bounding box set to the page size (EPS)"),
@@ -354,11 +360,6 @@ struct poptOption options[] = {
      N_("Show given files one-by-one, switch to next on any key/mouse event"),
      NULL},
 
-    {"new-gui", 'G',
-     POPT_ARG_NONE, &sp_new_gui, SP_ARG_NEW_GUI,
-     N_("Use the new Gtkmm GUI interface"),
-     NULL},
-
     {"vacuum-defs", 0,
      POPT_ARG_NONE, &sp_vacuum_defs, SP_ARG_VACUUM_DEFS,
      N_("Remove unused definitions from the defs section(s) of the document"),
@@ -465,9 +466,6 @@ main(int argc, char **argv)
         } else if (!strcmp(argv[i], "-g") || !strcmp(argv[i], "--with-gui")) {
             use_gui = TRUE;
             break;
-        } else if (!strcmp(argv[i], "-G") || !strcmp(argv[i], "--new-gui")) {
-            sp_new_gui = TRUE;
-            break;
         }
     }
 
@@ -1001,6 +999,7 @@ static void do_export_ps(SPDocument* doc, gchar const* uri, char const* mime)
     }
 
     bool old_text_to_path = false;
+    bool old_font_embedded = false;
     bool old_bbox_page = false;
 
     try {
@@ -1011,6 +1010,14 @@ static void do_export_ps(SPDocument* doc, gchar const* uri, char const* mime)
         g_warning ("Could not set export-text-to-path option for this export.");
     }
 
+    try {
+        old_font_embedded = (*i)->get_param_bool("fontEmbedded");
+        (*i)->set_param_bool("fontEmbedded", sp_export_font);
+    }
+    catch (...) {
+        g_warning ("Could not set export-font option for this export.");
+    }
+
     try {
         old_bbox_page = (*i)->get_param_bool("pageBoundingBox");
         (*i)->set_param_bool("pageBoundingBox", sp_export_bbox_page);
@@ -1023,6 +1030,7 @@ static void do_export_ps(SPDocument* doc, gchar const* uri, char const* mime)
 
     try {
         (*i)->set_param_bool("textToPath", old_text_to_path);
+        (*i)->set_param_bool("fontEmbedded", old_font_embedded);
         (*i)->set_param_bool("pageBoundingBox", old_bbox_page);
     }
     catch (...) {
