@@ -25,6 +25,7 @@ static char const menus_skeleton[] =
 "       <verb verb-id=\"FileRevert\" />\n"
 "       <verb verb-id=\"FileSave\" />\n"
 "       <verb verb-id=\"FileSaveAs\" />\n"
+"       <verb verb-id=\"FileSaveACopy\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"FileImport\" />\n"
 "       <verb verb-id=\"FileExport\" />\n"
@@ -54,6 +55,7 @@ static char const menus_skeleton[] =
 "   <submenu name=\"" N_("_Edit") "\">\n"
 "       <verb verb-id=\"EditUndo\" />\n"
 "       <verb verb-id=\"EditRedo\" />\n"
+"       <verb verb-id=\"DialogUndoHistory\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"EditCut\" />\n"
 "       <verb verb-id=\"EditCopy\" />\n"
@@ -93,7 +95,7 @@ static char const menus_skeleton[] =
 "     <submenu name=\"" N_("_Zoom") "\">\n"
 "       <verb verb-id=\"ZoomIn\" />\n"
 "       <verb verb-id=\"ZoomOut\" />\n"
-"       <seperator/>\n"
+"       <separator/>\n"
 "       <verb verb-id=\"Zoom1:0\" />\n"
 "       <verb verb-id=\"Zoom1:2\" />\n"
 "       <verb verb-id=\"Zoom2:1\" />\n"
@@ -106,30 +108,31 @@ static char const menus_skeleton[] =
 "       <verb verb-id=\"ZoomPrev\" />\n"
 "       <verb verb-id=\"ZoomNext\" />\n"
 "     </submenu>\n"
+"       <submenu name=\"" N_("_Display mode") "\">\n"
+"         <verb verb-id=\"ViewModeToggle\"/>\n"
+"         <verb verb-id=\"ViewModeNormal\" radio=\"yes\" default=\"yes\"/>\n"
+"         <verb verb-id=\"ViewModeOutline\" radio=\"yes\"/>\n"
+"       </submenu>\n"
+"       <separator/>\n"
+"       <verb verb-id=\"ToggleGrid\" />\n"
+"       <verb verb-id=\"ToggleGuides\" />\n"
 "       <separator/>\n"
 "       <submenu name=\"" N_("Show/Hide") "\">\n"
 "           <objects-checkboxes/>\n"
 "       </submenu>\n"
 "       <verb verb-id=\"DialogsToggle\" />\n"
 "       <separator/>\n"
-"   <submenu name=\"" N_("_Display mode") "\">\n"
-"       <verb verb-id=\"ViewModeNormal\" radio=\"yes\" default=\"yes\"/>\n"
-"       <verb verb-id=\"ViewModeOutline\" radio=\"yes\"/>\n"
-"       </submenu>\n"
-"       <seperator/>\n"
-"       <verb verb-id=\"ToggleGrid\" />\n"
-"       <verb verb-id=\"ToggleGuides\" />\n"
-"       <separator/>\n"
-"       <verb verb-id=\"FullScreen\" />\n"
-"       <separator/>\n"
+"       <verb verb-id=\"DialogSwatches\" />\n"
 "       <verb verb-id=\"DialogDebug\" />\n"
 "       <verb verb-id=\"DialogScript\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"PrevWindow\" />\n"
 "       <verb verb-id=\"NextWindow\" />\n"
 "       <separator/>\n"
-"       <verb verb-id=\"ViewNew\" />\n"
 "       <verb verb-id=\"ViewIconPreview\" />\n"
+"       <verb verb-id=\"ViewNew\" />\n"
+"       <separator/>\n"
+"       <verb verb-id=\"FullScreen\" />\n"
 "   </submenu>\n"
 "   <submenu name=\"" N_("_Layer") "\">\n"
 "       <verb verb-id=\"LayerNew\" />\n"
@@ -152,7 +155,6 @@ static char const menus_skeleton[] =
 "   </submenu>\n"
 "   <submenu name=\"" N_("_Object") "\">\n"
 "       <verb verb-id=\"DialogFillStroke\" />\n"
-"       <verb verb-id=\"DialogSwatches\" />\n"
 "       <verb verb-id=\"DialogObjectProperties\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"SelectionGroup\" />\n"
@@ -220,7 +222,7 @@ static char const menus_skeleton[] =
 "       <separator/>\n"
 "       <verb verb-id=\"SelectionTextRemoveKerns\" />\n"
 "   </submenu>\n"
-"   <submenu name=\"" N_("Effects") "\">\n"
+"   <submenu name=\"" N_("Effe_cts") "\">\n"
 "       <verb verb-id=\"EffectLast\" />\n"
 "       <verb verb-id=\"EffectLastPref\" />\n"
 "       <separator/>\n"
@@ -228,19 +230,11 @@ static char const menus_skeleton[] =
 "   </submenu>\n"
 #ifdef WITH_INKBOARD
 "	<submenu name=\"" N_("Whiteboa_rd") "\">\n"
-"		<verb verb-id=\"DialogWhiteboardConnect\" />\n"
-"		<verb verb-id=\"DialogWhiteboardDisconnectSession\" />\n"
-"		<verb verb-id=\"DialogWhiteboardDisconnectServer\" />\n"
-"		<separator/>\n"
-"		<verb verb-id=\"DialogWhiteboardShareWithUser\" />\n"
-"		<verb verb-id=\"DialogWhiteboardShareWithChat\" />\n"
-"		<separator/>\n"
-"		<verb verb-id=\"DialogWhiteboardOpenSessionFile\" />\n"
-"		<separator/>\n"
-"		<verb verb-id=\"DialogWhiteboardDumpXMLTracker\" />\n"
+"		<verb verb-id=\"DialogXmppClient\" />\n"
 "	</submenu>\n"
 #endif
 "   <submenu name=\"" N_("_Help") "\">\n"
+"       <verb verb-id=\"org.inkscape.help.manual\" />\n"
 "       <verb verb-id=\"HelpKeys\" />\n"
 "       <submenu name=\"" N_("Tutorials") "\">\n"
 "           <verb verb-id=\"TutorialsBasic\" />\n"
@@ -252,6 +246,12 @@ static char const menus_skeleton[] =
 "           <verb verb-id=\"TutorialsTips\" />\n"
 "       </submenu>\n"
 //"       <verb verb-id=\"org.inkscape.dialogs.extensioneditor\" />\n"
+"       <verb verb-id=\"org.inkscape.help.commandline\" />\n"
+"       <verb verb-id=\"org.inkscape.help.faq\" />\n"
+"       <verb verb-id=\"org.inkscape.help.relnotes\" />\n"
+"       <verb verb-id=\"org.inkscape.help.reportabug\" />\n"
+"       <verb verb-id=\"org.inkscape.help.svgspec\" />\n"
+"       <separator/>\n"
 "       <verb verb-id=\"HelpAboutMemory\" />\n"
 "       <verb verb-id=\"HelpAbout\" />\n"
 //"       <verb verb-id=\"ShowLicense\" />\n"
