@@ -105,7 +105,7 @@ sp_node_context_init(SPNodeContext *node_context)
     node_context->rightalt = FALSE;
     node_context->leftctrl = FALSE;
     node_context->rightctrl = FALSE;
-
+    
     new (&node_context->sel_changed_connection) sigc::connection();
 }
 
@@ -134,6 +134,7 @@ sp_node_context_dispose(GObject *object)
     }
 
     if (nc->nodepath) {
+        nc->grab_node = -1;
         sp_nodepath_destroy(nc->nodepath);
         nc->nodepath = NULL;
     }
@@ -164,6 +165,7 @@ sp_node_context_setup(SPEventContext *ec)
     Inkscape::Selection *selection = sp_desktop_selection(ec->desktop);
     SPItem *item = selection->singleItem();
 
+    nc->grab_node = -1;
     nc->nodepath = NULL;
     ec->shape_knot_holder = NULL;
 
@@ -173,6 +175,8 @@ sp_node_context_setup(SPEventContext *ec)
 
     nc->added_node = false;
 
+    nc->current_state = SP_NODE_CONTEXT_INACTIVE;
+
     if (item) {
         nc->nodepath = sp_nodepath_new(ec->desktop, item, (prefs_get_int_attribute("tools.nodes", "show_handles", 1) != 0));
         if ( nc->nodepath) {
@@ -221,6 +225,7 @@ sp_node_context_selection_changed(Inkscape::Selection *selection, gpointer data)
 
     if (nc->nodepath) {
         old_repr = nc->nodepath->repr;
+        nc->grab_node = -1;
         sp_nodepath_destroy(nc->nodepath);
         nc->nodepath = NULL;
     }
@@ -238,6 +243,7 @@ sp_node_context_selection_changed(Inkscape::Selection *selection, gpointer data)
     SPItem *item = selection->singleItem();
 
     SPDesktop *desktop = selection->desktop();
+    nc->grab_node = -1;
     nc->nodepath = NULL;
     ec->shape_knot_holder = NULL;
     if (item) {
@@ -279,6 +285,7 @@ sp_nodepath_update_from_item(SPNodeContext *nc, SPItem *item)
     g_assert(desktop);
 
     if (nc->nodepath) {
+        nc->grab_node = -1;
         sp_nodepath_destroy(nc->nodepath);
         nc->nodepath = NULL;
     }
@@ -371,6 +378,7 @@ sp_node_context_is_over_stroke (SPNodeContext *nc, SPItem *item, NR::Point event
     nc->curvepoint_doc *= sp_item_dt2i_affine(item);
     nc->curvepoint_doc *= sp_item_i2doc_affine(item);
 
+    sp_nodepath_ensure_livarot_path(nc->nodepath);
     NR::Maybe<Path::cut_position> position = get_nearest_position_on_Path(nc->nodepath->livarot_path, nc->curvepoint_doc);
     NR::Point nearest = get_point_on_Path(nc->nodepath->livarot_path, position.assume().piece, position.assume().t);
     NR::Point delta = nearest - nc->curvepoint_doc;
@@ -392,7 +400,7 @@ sp_node_context_is_over_stroke (SPNodeContext *nc, SPItem *item, NR::Point event
         nc->curvepoint_event[NR::Y] = (gint) event_p [NR::Y];
         nc->hit = true;
         nc->grab_t = position.assume().t;
-        nc->grab_node = sp_nodepath_get_node_by_index(position.assume().piece);
+        nc->grab_node = position.assume().piece;
     }
 
     return close;
@@ -520,6 +528,11 @@ sp_node_context_root_handler(SPEventContext *event_context, GdkEvent *event)
     SPDesktop *desktop = event_context->desktop;
     Inkscape::Selection *selection = sp_desktop_selection (desktop);
 
+    // fixme:  nc->nodepath can potentially become NULL after retrieving nc.
+    // A general method for handling this possibility should be created.
+    // For now, the number of checks for a NULL nc->nodepath have been
+    // increased, both here and in the called sp_nodepath_* functions.
+
     SPNodeContext *nc = SP_NODE_CONTEXT(event_context);
     double const nudge = prefs_get_double_attribute_limited("options.nudgedistance", "value", 2, 0, 1000); // in px
     event_context->tolerance = prefs_get_int_attribute_limited("options.dragtolerance", "value", 0, 0, 100); // read every time, to make prefs changes really live
@@ -541,6 +554,7 @@ sp_node_context_root_handler(SPEventContext *event_context, GdkEvent *event)
                                          event->button.y);
                 NR::Point const button_dt(desktop->w2d(button_w));
                 Inkscape::Rubberband::get()->start(desktop, button_dt);
+                nc->current_state = SP_NODE_CONTEXT_INACTIVE;
                 desktop->updateNow();
                 ret = TRUE;
             }
@@ -553,25 +567,69 @@ sp_node_context_root_handler(SPEventContext *event_context, GdkEvent *event)
                      && ( abs( (gint) event->motion.y - event_context->yp ) < event_context->tolerance ) ) {
                     break; // do not drag if we're within tolerance from origin
                 }
+
+                // The path went away while dragging; throw away any further motion
+                // events until the mouse pointer is released.
+                if (nc->hit && (nc->nodepath == NULL)) {                  
+                  break;
+                }
+
                 // Once the user has moved farther than tolerance from the original location
                 // (indicating they intend to move the object, not click), then always process the
                 // motion notify coordinates as given (no snapping back to origin)
                 event_context->within_tolerance = false;
 
-                if (nc->nodepath && nc->hit) {
-                    NR::Point const delta_w(event->motion.x - nc->curvepoint_event[NR::X],
-                                         event->motion.y - nc->curvepoint_event[NR::Y]);
-                    NR::Point const delta_dt(desktop->w2d(delta_w));
-                    sp_nodepath_curve_drag (nc->grab_node, nc->grab_t, delta_dt);
-                    nc->curvepoint_event[NR::X] = (gint) event->motion.x;
-                    nc->curvepoint_event[NR::Y] = (gint) event->motion.y;
-                    gobble_motion_events(GDK_BUTTON1_MASK);
-                } else {
-                    NR::Point const motion_w(event->motion.x,
-                                         event->motion.y);
-                    NR::Point const motion_dt(desktop->w2d(motion_w));
-                    Inkscape::Rubberband::get()->move(motion_dt);
+                // Once we determine what the user is doing (dragging either a node or the
+                // selection rubberband), make sure we continue to perform that operation
+                // until the mouse pointer is lifted.
+                if (nc->current_state == SP_NODE_CONTEXT_INACTIVE) {
+                    if (nc->nodepath && nc->hit) {
+                        nc->current_state = SP_NODE_CONTEXT_NODE_DRAGGING;
+                    } else {
+                        nc->current_state = SP_NODE_CONTEXT_RUBBERBAND_DRAGGING;
+                    }
                 }
+
+                switch (nc->current_state) {
+                    case SP_NODE_CONTEXT_NODE_DRAGGING:
+                        {
+                            if (nc->grab_node == -1) // don't know which segment to drag
+                                break;
+
+                            // We round off the extra precision in the motion coordinates provided
+                            // by some input devices (like tablets). As we'll store the coordinates
+                            // as integers in curvepoint_event we need to do this rounding before
+                            // comparing them with the last coordinates from curvepoint_event.
+                            // See bug #1593499 for details.
+
+                            gint x = (gint) Inkscape::round(event->motion.x);
+                            gint y = (gint) Inkscape::round(event->motion.y);
+
+                            // The coordinates hasn't changed since the last motion event, abort
+                            if (nc->curvepoint_event[NR::X] == x &&
+                                nc->curvepoint_event[NR::Y] == y)
+                                break;
+
+                            NR::Point const delta_w(event->motion.x - nc->curvepoint_event[NR::X],
+                                                    event->motion.y - nc->curvepoint_event[NR::Y]);
+                            NR::Point const delta_dt(desktop->w2d(delta_w));
+
+                            sp_nodepath_curve_drag (nc->grab_node, nc->grab_t, delta_dt);
+                            nc->curvepoint_event[NR::X] = x;
+                            nc->curvepoint_event[NR::Y] = y;
+                            gobble_motion_events(GDK_BUTTON1_MASK);
+                            break;
+                        }
+                    case SP_NODE_CONTEXT_RUBBERBAND_DRAGGING:
+                        if (Inkscape::Rubberband::get()->is_started()) {
+                            NR::Point const motion_w(event->motion.x,
+                                                event->motion.y);
+                            NR::Point const motion_dt(desktop->w2d(motion_w));
+                            Inkscape::Rubberband::get()->move(motion_dt);
+                        }
+                        break;
+                }
+
                 nc->drag = TRUE;
                 ret = TRUE;
             } else {
@@ -608,7 +666,9 @@ sp_node_context_root_handler(SPEventContext *event_context, GdkEvent *event)
                 NR::Maybe<NR::Rect> b = Inkscape::Rubberband::get()->getRectangle();
 
                 if (nc->hit && !event_context->within_tolerance) { //drag curve
-                    sp_nodepath_update_repr (nc->nodepath);
+                    if (nc->nodepath) {
+                        sp_nodepath_update_repr (nc->nodepath, _("Drag curve"));
+                    }
                 } else if (b != NR::Nothing() && !event_context->within_tolerance) { // drag to select
                     if (nc->nodepath) {
                         sp_nodepath_select_rect(nc->nodepath, b.assume(), event->button.state & GDK_SHIFT_MASK);
@@ -627,6 +687,7 @@ sp_node_context_root_handler(SPEventContext *event_context, GdkEvent *event)
                 nc->rb_escaped = false;
                 nc->drag = FALSE;
                 nc->hit = false;
+                nc->current_state = SP_NODE_CONTEXT_INACTIVE;
                 break;
             }
             break;
@@ -774,23 +835,12 @@ sp_node_context_root_handler(SPEventContext *event_context, GdkEvent *event)
                         ret = TRUE;
                     }
                     break;
-                case GDK_Tab: // Tab - cycle selection forward
-                    if (!(MOD__CTRL_ONLY || (MOD__CTRL && MOD__SHIFT))) {
-                        sp_nodepath_select_next(nc->nodepath);
-                        ret = TRUE;
-                    }
-                    break;
-                case GDK_ISO_Left_Tab:  // Shift Tab - cycle selection backward
-                    if (!(MOD__CTRL_ONLY || (MOD__CTRL && MOD__SHIFT))) {
-                        sp_nodepath_select_prev(nc->nodepath);
-                        ret = TRUE;
-                    }
-                    break;
                 case GDK_Escape:
                 {
                     NR::Maybe<NR::Rect> const b = Inkscape::Rubberband::get()->getRectangle();
                     if (b != NR::Nothing()) {
                         Inkscape::Rubberband::get()->stop();
+                        nc->current_state = SP_NODE_CONTEXT_INACTIVE;
                         nc->rb_escaped = true;
                     } else {
                         if (nc->nodepath && nc->nodepath->selected) {
