@@ -1,14 +1,9 @@
-#include <glib.h>
-#include <vector>
 
 #include <approx-equal.h>
-#include <libnr/nr-point-fns.h>
 #include <remove-last.h>
 #include <sp-guide.h>
-#include <sp-guide-attachment.h>
 #include <sp-guide-constraint.h>
 #include <sp-item.h>
-#include <sp-item-rm-unsatisfied-cns.h>
 using std::vector;
 
 void sp_item_rm_unsatisfied_cns(SPItem &item)
@@ -37,9 +32,9 @@ void sp_item_rm_unsatisfied_cns(SPItem &item)
   Local Variables:
   mode:c++
   c-file-style:"stroustrup"
-  c-file-offsets:((innamespace . 0)(inline-open . 0))
+  c-file-offsets:((innamespace . 0)(inline-open . 0)(case-label . +))
   indent-tabs-mode:nil
   fill-column:99
   End:
 */
-// vim: filetype=c++:expandtab:shiftwidth=4:tabstop=8:softtabstop=4 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
