@@ -7,8 +7,9 @@
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *   bulia byak <buliabyak@users.sf.net>
+ *   Johan Engelen <j.b.c.engelen@ewi.utwente.nl>
  *
- * Copyright (C) 2001-2005 authors
+ * Copyright (C) 2001-2006 authors
  * Copyright (C) 2001 Ximian, Inc.
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
@@ -149,16 +150,14 @@ void SPItem::init() {
     this->display = NULL;
 
     this->clip_ref = new SPClipPathReference(this);
-		{
-			sigc::signal<void, SPObject *, SPObject *> cs1=this->clip_ref->changedSignal();
-			sigc::slot2<void,SPObject*, SPObject *> sl1=sigc::bind(sigc::ptr_fun(clip_ref_changed), this);
-			cs1.connect(sl1);
-		}
+		sigc::signal<void, SPObject *, SPObject *> cs1=this->clip_ref->changedSignal();
+		sigc::slot2<void,SPObject*, SPObject *> sl1=sigc::bind(sigc::ptr_fun(clip_ref_changed), this);
+    _clip_ref_connection = cs1.connect(sl1);
 
     this->mask_ref = new SPMaskReference(this);
 		sigc::signal<void, SPObject *, SPObject *> cs2=this->mask_ref->changedSignal();
 		sigc::slot2<void,SPObject*, SPObject *> sl2=sigc::bind(sigc::ptr_fun(mask_ref_changed), this);
-    cs2.connect(sl2);
+    _mask_ref_connection = cs2.connect(sl2);
 
     if (!this->style) this->style = sp_style_new_from_object(this);
 
@@ -402,6 +401,9 @@ sp_item_release(SPObject *object)
 {
     SPItem *item = (SPItem *) object;
 
+    item->_clip_ref_connection.disconnect();
+    item->_mask_ref_connection.disconnect();
+
     if (item->clip_ref) {
         item->clip_ref->detach();
         delete item->clip_ref;
@@ -861,12 +863,12 @@ sp_item_description(SPItem *item)
     if (((SPItemClass *) G_OBJECT_GET_CLASS(item))->description) {
         gchar *s = ((SPItemClass *) G_OBJECT_GET_CLASS(item))->description(item);
         if (s && item->clip_ref->getObject()) {
-            gchar *snew = g_strdup_printf ("%s; <i>clipped</i>", s);
+            gchar *snew = g_strdup_printf (_("%s; <i>clipped</i>"), s);
             g_free (s);
             s = snew;
         }
         if (s && item->mask_ref->getObject()) {
-            gchar *snew = g_strdup_printf ("%s; <i>masked</i>", s);
+            gchar *snew = g_strdup_printf (_("%s; <i>masked</i>"), s);
             g_free (s);
             s = snew;
         }
@@ -1179,7 +1181,7 @@ sp_item_write_transform(SPItem *item, Inkscape::XML::Node *repr, NRMatrix const
  * the repr is updated with the new transform.
  */
 void
-sp_item_write_transform(SPItem *item, Inkscape::XML::Node *repr, NR::Matrix const &transform, NR::Matrix const *adv)
+sp_item_write_transform(SPItem *item, Inkscape::XML::Node *repr, NR::Matrix const &transform, NR::Matrix const *adv, bool compensate)
 {
     g_return_if_fail(item != NULL);
     g_return_if_fail(SP_IS_ITEM(item));
@@ -1192,37 +1194,46 @@ sp_item_write_transform(SPItem *item, Inkscape::XML::Node *repr, NR::Matrix cons
     } else {
         advertized_transform = sp_item_transform_repr (item).inverse() * transform;
     }
+    
+    if (compensate) {
+        
+         // recursively compensate for stroke scaling, depending on user preference
+        if (prefs_get_int_attribute("options.transform", "stroke", 1) == 0) {
+            double const expansion = 1. / NR::expansion(advertized_transform);
+            sp_item_adjust_stroke_width_recursive(item, expansion);
+        }
+    
+        // recursively compensate rx/ry of a rect if requested
+        if (prefs_get_int_attribute("options.transform", "rectcorners", 1) == 0) {
+            sp_item_adjust_rects_recursive(item, advertized_transform);
+        }
+    
+        // recursively compensate pattern fill if it's not to be transformed
+        if (prefs_get_int_attribute("options.transform", "pattern", 1) == 0) {
+            sp_item_adjust_paint_recursive (item, advertized_transform.inverse(), NR::identity(), true);
+        }
+        /// \todo FIXME: add the same else branch as for gradients below, to convert patterns to userSpaceOnUse as well
+        /// recursively compensate gradient fill if it's not to be transformed
+        if (prefs_get_int_attribute("options.transform", "gradient", 1) == 0) {
+            sp_item_adjust_paint_recursive (item, advertized_transform.inverse(), NR::identity(), false);
+        } else {
+            // this converts the gradient/pattern fill/stroke, if any, to userSpaceOnUse; we need to do
+            // it here _before_ the new transform is set, so as to use the pre-transform bbox
+            sp_item_adjust_paint_recursive (item, NR::identity(), NR::identity(), false);
+        }          
+        
+    } // endif(compensate)
 
-     // recursively compensate for stroke scaling, depending on user preference
-    if (prefs_get_int_attribute("options.transform", "stroke", 1) == 0) {
-        double const expansion = 1. / NR::expansion(advertized_transform);
-        sp_item_adjust_stroke_width_recursive(item, expansion);
-    }
-
-    // recursively compensate rx/ry of a rect if requested
-    if (prefs_get_int_attribute("options.transform", "rectcorners", 1) == 0) {
-        sp_item_adjust_rects_recursive(item, advertized_transform);
-    }
-
-    // recursively compensate pattern fill if it's not to be transformed
-    if (prefs_get_int_attribute("options.transform", "pattern", 1) == 0) {
-        sp_item_adjust_paint_recursive (item, advertized_transform.inverse(), NR::identity(), true);
-    }
-    /// \todo FIXME: add the same else branch as for gradients below, to convert patterns to userSpaceOnUse as well
-    /// recursively compensate gradient fill if it's not to be transformed
-    if (prefs_get_int_attribute("options.transform", "gradient", 1) == 0) {
-        sp_item_adjust_paint_recursive (item, advertized_transform.inverse(), NR::identity(), false);
-    } else {
-        // this converts the gradient/pattern fill/stroke, if any, to userSpaceOnUse; we need to do
-        // it here _before_ the new transform is set, so as to use the pre-transform bbox
-        sp_item_adjust_paint_recursive (item, NR::identity(), NR::identity(), false);
-    }
-
-    // run the object's set_transform if transforms are stored optimized and there's no clippath or mask
     gint preserve = prefs_get_int_attribute("options.preservetransform", "value", 0);
     NR::Matrix transform_attr (transform);
-    if (((SPItemClass *) G_OBJECT_GET_CLASS(item))->set_transform 
-                && !preserve && !item->clip_ref->getObject() && !item->mask_ref->getObject()) {
+    if ( // run the object's set_transform (i.e. embed transform) only if:
+         ((SPItemClass *) G_OBJECT_GET_CLASS(item))->set_transform && // it does have a set_transform method
+             !preserve && // user did not chose to preserve all transforms
+             !item->clip_ref->getObject() && // the object does not have a clippath
+             !item->mask_ref->getObject() && // the object does not have a mask
+             !(!transform.is_translation() && SP_OBJECT_STYLE(item) && SP_OBJECT_STYLE(item)->filter.filter) 
+             // the object does not have a filter, or the transform is translation (which is supposed to not affect filters)
+        ) {
         transform_attr = ((SPItemClass *) G_OBJECT_GET_CLASS(item))->set_transform(item, transform);
     }
     sp_item_set_item_transform(item, transform_attr);
