@@ -99,8 +99,6 @@ static gchar *sp_object_get_unique_id(SPObject *object, gchar const *defid);
 
 guint update_in_progress = 0; // guard against update-during-update
 
-enum {RELEASE, MODIFIED, LAST_SIGNAL};
-
 Inkscape::XML::NodeEventVector object_event_vector = {
     sp_object_repr_child_added,
     sp_object_repr_child_removed,
@@ -110,7 +108,6 @@ Inkscape::XML::NodeEventVector object_event_vector = {
 };
 
 static GObjectClass *parent_class;
-static guint object_signals[LAST_SIGNAL] = {0};
 
 /**
  * Registers the SPObject class with Gdk and returns its type number.
@@ -147,21 +144,6 @@ sp_object_class_init(SPObjectClass *klass)
 
     parent_class = (GObjectClass *) g_type_class_ref(G_TYPE_OBJECT);
 
-    object_signals[RELEASE] =  g_signal_new("release",
-                                            G_TYPE_FROM_CLASS(klass),
-                                            (GSignalFlags)(G_SIGNAL_RUN_CLEANUP | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS),
-                                            G_STRUCT_OFFSET(SPObjectClass, release),
-                                            NULL, NULL,
-                                            sp_marshal_VOID__VOID,
-                                            G_TYPE_NONE, 0);
-    object_signals[MODIFIED] = g_signal_new("modified",
-                                            G_TYPE_FROM_CLASS(klass),
-                                            G_SIGNAL_RUN_FIRST,
-                                            G_STRUCT_OFFSET(SPObjectClass, modified),
-                                            NULL, NULL,
-                                            sp_marshal_NONE__UINT,
-                                            G_TYPE_NONE, 1, G_TYPE_UINT);
-
     object_class->finalize = sp_object_finalize;
 
     klass->child_added = sp_object_child_added;
@@ -195,6 +177,8 @@ sp_object_init(SPObject *object)
 
     object->_collection_policy = SPObject::COLLECT_WITH_PARENT;
 
+    new (&object->_release_signal) sigc::signal<void, SPObject *>();
+    new (&object->_modified_signal) sigc::signal<void, SPObject *, unsigned int>();
     new (&object->_delete_signal) sigc::signal<void, SPObject *>();
     new (&object->_position_changed_signal) sigc::signal<void, SPObject *>();
     object->_successor = NULL;
@@ -225,6 +209,8 @@ sp_object_finalize(GObject *object)
         (* ((GObjectClass *) (parent_class))->finalize)(object);
     }
 
+    spobject->_release_signal.~signal();
+    spobject->_modified_signal.~signal();
     spobject->_delete_signal.~signal();
     spobject->_position_changed_signal.~signal();
 }
@@ -458,6 +444,22 @@ SPObject::appendChildRepr(Inkscape::XML::Node *repr) {
     }
 }
 
+/**
+ * Retrieves the children as a GSList object, optionally ref'ing the children
+ * in the process, if add_ref is specified.
+ */
+GSList *SPObject::childList(bool add_ref, Action) {
+    GSList *l = NULL;
+    for (SPObject *child = sp_object_first_child(this) ; child != NULL; child = SP_OBJECT_NEXT(child) ) {
+        if (add_ref)
+            g_object_ref (G_OBJECT (child));
+
+        l = g_slist_prepend (l, child);
+    }
+    return l;
+
+}
+
 /** Gets the label property for the object or a default if no label
  *  is defined.
  */
@@ -637,7 +639,7 @@ sp_object_detach(SPObject *parent, SPObject *object) {
     g_return_if_fail(SP_IS_OBJECT(object));
     g_return_if_fail(object->parent == parent);
 
-    sp_object_invoke_release(object);
+    object->releaseReferences();
 
     SPObject *prev=NULL;
     for ( SPObject *child = parent->children ; child && child != object ;
@@ -823,10 +825,10 @@ sp_object_invoke_build(SPObject *object, SPDocument *document, Inkscape::XML::No
 
             /* Redefine ID, if required */
             if ((id == NULL) || (strcmp(id, realid) != 0)) {
-                gboolean undo_sensitive=sp_document_get_undo_sensitive(document);
-                sp_document_set_undo_sensitive(document, FALSE);
+                bool saved = sp_document_get_undo_sensitive(document);
+                sp_document_set_undo_sensitive(document, false);
                 object->repr->setAttribute("id", realid);
-                sp_document_set_undo_sensitive(document, undo_sensitive);
+                sp_document_set_undo_sensitive(document, saved);
             }
         }
     } else {
@@ -843,45 +845,44 @@ sp_object_invoke_build(SPObject *object, SPDocument *document, Inkscape::XML::No
     sp_repr_add_listener(repr, &object_event_vector, object);
 }
 
-void
-sp_object_invoke_release(SPObject *object)
-{
-    g_assert(object != NULL);
-    g_assert(SP_IS_OBJECT(object));
+void SPObject::releaseReferences() {
+    g_assert(this->document);
+    g_assert(this->repr);
 
-    g_assert(object->document);
-    g_assert(object->repr);
+    sp_repr_remove_listener_by_data(this->repr, this);
 
-    sp_repr_remove_listener_by_data(object->repr, object);
-
-    g_signal_emit(G_OBJECT(object), object_signals[RELEASE], 0);
+    this->_release_signal.emit(this);
+    SPObjectClass *klass=(SPObjectClass *)G_OBJECT_GET_CLASS(this);
+    if (klass->release) {
+        klass->release(this);
+    }
 
     /* all hrefs should be released by the "release" handlers */
-    g_assert(object->hrefcount == 0);
+    g_assert(this->hrefcount == 0);
 
-    if (!SP_OBJECT_IS_CLONED(object)) {
-        if (object->id) {
-            object->document->bindObjectToId(object->id, NULL);
+    if (!SP_OBJECT_IS_CLONED(this)) {
+        if (this->id) {
+            this->document->bindObjectToId(this->id, NULL);
         }
-        g_free(object->id);
-        object->id = NULL;
+        g_free(this->id);
+        this->id = NULL;
 
-        g_free(object->_default_label);
-        object->_default_label = NULL;
+        g_free(this->_default_label);
+        this->_default_label = NULL;
 
-        object->document->bindObjectToRepr(object->repr, NULL);
+        this->document->bindObjectToRepr(this->repr, NULL);
     } else {
-        g_assert(!object->id);
+        g_assert(!this->id);
     }
 
-    if (object->style) {
-        object->style = sp_style_unref(object->style);
+    if (this->style) {
+        this->style = sp_style_unref(this->style);
     }
 
-    Inkscape::GC::release(object->repr);
+    Inkscape::GC::release(this->repr);
 
-    object->document = NULL;
-    object->repr = NULL;
+    this->document = NULL;
+    this->repr = NULL;
 }
 
 /**
@@ -1168,30 +1169,38 @@ SPObject::updateRepr(Inkscape::XML::Node *repr, unsigned int flags) {
 void
 SPObject::requestDisplayUpdate(unsigned int flags)
 {
+    g_return_if_fail( this->document != NULL );
+
     if (update_in_progress) {
         g_print("WARNING: Requested update while update in progress, counter = %d\n", update_in_progress);
     }
 
+    /* requestModified must be used only to set one of SP_OBJECT_MODIFIED_FLAG or
+     * SP_OBJECT_CHILD_MODIFIED_FLAG */
     g_return_if_fail(!(flags & SP_OBJECT_PARENT_MODIFIED_FLAG));
     g_return_if_fail((flags & SP_OBJECT_MODIFIED_FLAG) || (flags & SP_OBJECT_CHILD_MODIFIED_FLAG));
     g_return_if_fail(!((flags & SP_OBJECT_MODIFIED_FLAG) && (flags & SP_OBJECT_CHILD_MODIFIED_FLAG)));
 
-    /* Check for propagate before we set any flags */
-    /* Propagate means, that this is not passed through by modification request cascade yet */
-    unsigned int propagate = (!(this->uflags & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG)));
+    bool already_propagated = (!(this->uflags & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG)));
 
-    /* Just set this flags safe even if some have been set before */
     this->uflags |= flags;
 
-    if (propagate) {
-        if (this->parent) {
-            this->parent->requestDisplayUpdate(SP_OBJECT_CHILD_MODIFIED_FLAG);
+    /* If requestModified has already been called on this object or one of its children, then we
+     * don't need to set CHILD_MODIFIED on our ancestors because it's already been done.
+     */
+    if (already_propagated) {
+        SPObject *parent = SP_OBJECT_PARENT(this);
+        if (parent) {
+            parent->requestDisplayUpdate(SP_OBJECT_CHILD_MODIFIED_FLAG);
         } else {
-            sp_document_request_modified(this->document);
+            sp_document_request_modified(SP_OBJECT_DOCUMENT(this));
         }
     }
 }
 
+/**
+ * Update views
+ */
 void
 SPObject::updateDisplay(SPCtx *ctx, unsigned int flags)
 {
@@ -1228,29 +1237,30 @@ SPObject::updateDisplay(SPCtx *ctx, unsigned int flags)
     update_in_progress --;
 }
 
+/**
+ * Request modified always bubbles *up* the tree, as opposed to 
+ * request display update, which trickles down and relies on the 
+ * flags set during this pass...
+ */
 void
 SPObject::requestModified(unsigned int flags)
 {
     g_return_if_fail( this->document != NULL );
 
-    /* PARENT_MODIFIED is computed later on and is not intended to be
-     * "manually" queued */
+    /* requestModified must be used only to set one of SP_OBJECT_MODIFIED_FLAG or
+     * SP_OBJECT_CHILD_MODIFIED_FLAG */
     g_return_if_fail(!(flags & SP_OBJECT_PARENT_MODIFIED_FLAG));
-
-    /* we should be setting either MODIFIED or CHILD_MODIFIED... */
     g_return_if_fail((flags & SP_OBJECT_MODIFIED_FLAG) || (flags & SP_OBJECT_CHILD_MODIFIED_FLAG));
-
-    /* ...but not both */
     g_return_if_fail(!((flags & SP_OBJECT_MODIFIED_FLAG) && (flags & SP_OBJECT_CHILD_MODIFIED_FLAG)));
 
-    unsigned int old_mflags=this->mflags;
+    bool already_propagated = (!(this->mflags & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG)));
+
     this->mflags |= flags;
 
-    /* If we already had MODIFIED or CHILD_MODIFIED queued, we will
-     * have already queued CHILD_MODIFIED with our ancestors and
-     * need not disturb them again.
+    /* If requestModified has already been called on this object or one of its children, then we
+     * don't need to set CHILD_MODIFIED on our ancestors because it's already been done.
      */
-    if (!( old_mflags & ( SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG ) )) {
+    if (already_propagated) {
         SPObject *parent=SP_OBJECT_PARENT(this);
         if (parent) {
             parent->requestModified(SP_OBJECT_CHILD_MODIFIED_FLAG);
@@ -1260,6 +1270,12 @@ SPObject::requestModified(unsigned int flags)
     }
 }
 
+/** 
+ *  Emits the MODIFIED signal with the object's flags.
+ *  The object's mflags are the original set aside during the update pass for 
+ *  later delivery here.  Once emitModified() is called, those flags don't
+ *  need to be stored any longer.
+ */
 void
 SPObject::emitModified(unsigned int flags)
 {
@@ -1277,7 +1293,11 @@ SPObject::emitModified(unsigned int flags)
     this->mflags = 0;
 
     g_object_ref(G_OBJECT(this));
-    g_signal_emit(G_OBJECT(this), object_signals[MODIFIED], 0, flags);
+    SPObjectClass *klass=(SPObjectClass *)G_OBJECT_GET_CLASS(this);
+    if (klass->modified) {
+        klass->modified(this, flags);
+    }
+    _modified_signal.emit(this, flags);
     g_object_unref(G_OBJECT(this));
 }
 
