@@ -14,12 +14,8 @@
  */
 
 #include <libnr/nr-pixblock.h>
-
-#include "forward.h"
 #include "sp-object.h"
 
-#include <glib.h>
-
 
 
 class SPPainter;
