@@ -24,14 +24,15 @@
 #include <libnr/nr-matrix-translate-ops.h>
 #include <libnr/nr-scale-matrix-ops.h>
 
+#include <sigc++/functors/ptr_fun.h>
+#include <sigc++/adaptors/bind.h>
 
 #include "macros.h"
 #include "display/nr-arena-shape.h"
 #include "print.h"
 #include "document.h"
-#include "marker-status.h"
 #include "style.h"
-#include "sp-marker.h"
+#include "marker.h"
 #include "sp-path.h"
 #include "prefs-utils.h"
 
@@ -39,6 +40,7 @@
 
 static void sp_shape_class_init (SPShapeClass *klass);
 static void sp_shape_init (SPShape *shape);
+static void sp_shape_finalize (GObject *object);
 
 static void sp_shape_build (SPObject * object, SPDocument * document, Inkscape::XML::Node * repr);
 static void sp_shape_release (SPObject *object);
@@ -86,16 +88,20 @@ sp_shape_get_type (void)
 static void
 sp_shape_class_init (SPShapeClass *klass)
 {
+        GObjectClass *gobject_class;
 	SPObjectClass *sp_object_class;
 	SPItemClass * item_class;
 	SPPathClass * path_class;
 
+        gobject_class = (GObjectClass *) klass;
 	sp_object_class = (SPObjectClass *) klass;
 	item_class = (SPItemClass *) klass;
 	path_class = (SPPathClass *) klass;
 
 	parent_class = (SPItemClass *)g_type_class_peek_parent (klass);
 
+        gobject_class->finalize = sp_shape_finalize;
+
 	sp_object_class->build = sp_shape_build;
 	sp_object_class->release = sp_shape_release;
 	sp_object_class->update = sp_shape_update;
@@ -109,13 +115,32 @@ sp_shape_class_init (SPShapeClass *klass)
 }
 
 /**
- * Initializes an SPShape object.  Nothing particular is needed to initialize
- * an SPShape object; this is just here as a stub.
+ * Initializes an SPShape object.
  */
 static void
 sp_shape_init (SPShape *shape)
 {
-	/* Nothing here */
+    for ( int i = 0 ; i < SP_MARKER_LOC_QTY ; i++ ) {
+        new (&shape->release_connect[i]) sigc::connection();
+        new (&shape->modified_connect[i]) sigc::connection();
+    }
+}
+
+static void
+sp_shape_finalize (GObject *object)
+{
+    SPShape *shape=(SPShape *)object;
+
+    for ( int i = 0 ; i < SP_MARKER_LOC_QTY ; i++ ) {
+        shape->release_connect[i].disconnect();
+        shape->release_connect[i].~connection();
+        shape->modified_connect[i].disconnect();
+        shape->modified_connect[i].~connection();
+    }
+
+    if (((GObjectClass *) (parent_class))->finalize) {
+        (* ((GObjectClass *) (parent_class))->finalize)(object);
+    }
 }
 
 /**
@@ -533,8 +558,6 @@ sp_shape_update_marker_view (SPShape *shape, NRArenaItem *ai)
 {
 	SPStyle *style = ((SPObject *) shape)->style;
 
-	marker_status("sp_shape_update_marker_view:  Updating views of markers");
-
         for (int i = SP_MARKER_LOC_START; i < SP_MARKER_LOC_QTY; i++) {
             if (shape->marker[i] == NULL) {
                 continue;
@@ -835,7 +858,6 @@ sp_shape_marker_release (SPObject *marker, SPShape *shape)
 
 	item = (SPItem *) shape;
 
-	marker_status("sp_shape_marker_release:  Releasing markers");
 	for (i = SP_MARKER_LOC_START; i < SP_MARKER_LOC_QTY; i++) {
 	  if (marker == shape->marker[i]) {
 	    SPItemView *v;
@@ -885,8 +907,8 @@ sp_shape_set_marker (SPObject *object, unsigned int key, const gchar *value)
             SPItemView *v;
 
             /* Detach marker */
-            g_signal_handler_disconnect (shape->marker[key], shape->release_connect[key]);
-            g_signal_handler_disconnect (shape->marker[key], shape->modified_connect[key]);
+            shape->release_connect[key].disconnect();
+            shape->modified_connect[key].disconnect();
 
             /* Hide marker */
             for (v = item->display; v != NULL; v = v->next) {
@@ -901,10 +923,8 @@ sp_shape_set_marker (SPObject *object, unsigned int key, const gchar *value)
         }
         if (SP_IS_MARKER (mrk)) {
             shape->marker[key] = sp_object_href (mrk, object);
-            shape->release_connect[key] = g_signal_connect (G_OBJECT (shape->marker[key]), "release",
-                              G_CALLBACK (sp_shape_marker_release), shape);
-            shape->modified_connect[key] = g_signal_connect (G_OBJECT (shape->marker[key]), "modified",
-                              G_CALLBACK (sp_shape_marker_modified), shape);
+            shape->release_connect[key] = mrk->connectRelease(sigc::bind<1>(sigc::ptr_fun(&sp_shape_marker_release), shape));
+            shape->modified_connect[key] = mrk->connectModified(sigc::bind<2>(sigc::ptr_fun(&sp_shape_marker_modified), shape));
         }
     }
 }
