@@ -16,21 +16,12 @@
 
 #include "config.h"
 
-#include <math.h>
-#include <string.h>
-#include <stdlib.h>
+#include <glibmm/i18n.h>
 
 #include "svg/svg.h"
 #include "attributes.h"
-#include "dialogs/object-attributes.h"
 #include "display/curve.h"
-#include <glibmm/i18n.h>
 #include "xml/repr.h"
-#include "libnr/nr-point.h"
-#include "libnr/nr-point-fns.h"
-#include "libnr/nr-point-ops.h"
-#include "libnr/nr-point-l.h"
-#include "libnr/nr-point-matrix-ops.h"
 
 #include "sp-star.h"
 
@@ -139,7 +130,7 @@ sp_star_write (SPObject *object, Inkscape::XML::Node *repr, guint flags)
 	}
 
 	if (flags & SP_OBJECT_WRITE_EXT) {
-		sp_repr_set_attr (repr, "sodipodi:type", "star");
+		repr->setAttribute("sodipodi:type", "star");
 		sp_repr_set_int (repr, "sodipodi:sides", star->sides);
 		sp_repr_set_svg_double(repr, "sodipodi:cx", star->center[NR::X]);
 		sp_repr_set_svg_double(repr, "sodipodi:cy", star->center[NR::Y]);
@@ -153,8 +144,8 @@ sp_star_write (SPObject *object, Inkscape::XML::Node *repr, guint flags)
 	}
 
 	sp_star_set_shape ((SPShape *) star);
-	char *d = sp_svg_write_path (((SPShape *) star)->curve->bpath);
-	sp_repr_set_attr (repr, "d", d);
+	char *d = sp_svg_write_path (SP_CURVE_BPATH(((SPShape *) star)->curve));
+	repr->setAttribute("d", d);
 	g_free (d);
 
 	if (((SPObjectClass *) (parent_class))->write)
@@ -166,7 +157,7 @@ sp_star_write (SPObject *object, Inkscape::XML::Node *repr, guint flags)
 static void
 sp_star_set (SPObject *object, unsigned int key, const gchar *value)
 {
-	SPSVGLengthUnit unit;
+	SVGLength::Unit unit;
 
 	SPStar *star = SP_STAR (object);
 
@@ -183,27 +174,27 @@ sp_star_set (SPObject *object, unsigned int key, const gchar *value)
 		break;
 	case SP_ATTR_SODIPODI_CX:
 		if (!sp_svg_length_read_ldd (value, &unit, NULL, &star->center[NR::X]) ||
-		    (unit == SP_SVG_UNIT_EM) ||
-		    (unit == SP_SVG_UNIT_EX) ||
-		    (unit == SP_SVG_UNIT_PERCENT)) {
+		    (unit == SVGLength::EM) ||
+		    (unit == SVGLength::EX) ||
+		    (unit == SVGLength::PERCENT)) {
 			star->center[NR::X] = 0.0;
 		}
 		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_SODIPODI_CY:
 		if (!sp_svg_length_read_ldd (value, &unit, NULL, &star->center[NR::Y]) ||
-		    (unit == SP_SVG_UNIT_EM) ||
-		    (unit == SP_SVG_UNIT_EX) ||
-		    (unit == SP_SVG_UNIT_PERCENT)) {
+		    (unit == SVGLength::EM) ||
+		    (unit == SVGLength::EX) ||
+		    (unit == SVGLength::PERCENT)) {
 			star->center[NR::Y] = 0.0;
 		}
 		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_SODIPODI_R1:
 		if (!sp_svg_length_read_ldd (value, &unit, NULL, &star->r[0]) ||
-		    (unit == SP_SVG_UNIT_EM) ||
-		    (unit == SP_SVG_UNIT_EX) ||
-		    (unit == SP_SVG_UNIT_PERCENT)) {
+		    (unit == SVGLength::EM) ||
+		    (unit == SVGLength::EX) ||
+		    (unit == SVGLength::PERCENT)) {
 			star->r[0] = 1.0;
 		}
 		/* fixme: Need CLAMP (Lauris) */
@@ -211,9 +202,9 @@ sp_star_set (SPObject *object, unsigned int key, const gchar *value)
 		break;
 	case SP_ATTR_SODIPODI_R2:
 		if (!sp_svg_length_read_ldd (value, &unit, NULL, &star->r[1]) ||
-		    (unit == SP_SVG_UNIT_EM) ||
-		    (unit == SP_SVG_UNIT_EX) ||
-		    (unit == SP_SVG_UNIT_PERCENT)) {
+		    (unit == SVGLength::EM) ||
+		    (unit == SVGLength::EX) ||
+		    (unit == SVGLength::PERCENT)) {
 			star->r[1] = 0.0;
 		}
 		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
@@ -266,8 +257,8 @@ sp_star_set (SPObject *object, unsigned int key, const gchar *value)
 static void
 sp_star_update (SPObject *object, SPCtx *ctx, guint flags)
 {
-	if (flags & (SP_OBJECT_MODIFIED_FLAG | 
-		     SP_OBJECT_STYLE_MODIFIED_FLAG | 
+	if (flags & (SP_OBJECT_MODIFIED_FLAG |
+		     SP_OBJECT_STYLE_MODIFIED_FLAG |
 		     SP_OBJECT_VIEWPORT_MODIFIED_FLAG)) {
 		sp_shape_set_shape ((SPShape *) object);
 	}
@@ -312,9 +303,9 @@ Obvious (but acceptable for my purposes) limits to uniqueness:
 static guint32
 point_unique_int (NR::Point o)
 {
-	return ((guint32) 
-	65536 * 
-		(((int) floor (o[NR::X] * 64)) % 1024 + ((int) floor (o[NR::X] * 1024)) % 64) 
+	return ((guint32)
+	65536 *
+		(((int) floor (o[NR::X] * 64)) % 1024 + ((int) floor (o[NR::X] * 1024)) % 64)
 	+
      		(((int) floor (o[NR::Y] * 64)) % 1024 + ((int) floor (o[NR::Y] * 1024)) % 64)
 	);
@@ -365,7 +356,7 @@ sp_star_get_curvepoint (SPStar *star, SPStarPoint point, gint index, bool previ)
 
 	// point to which we direct the bissector of the curve handles;
 	// it's far enough outside the star on the perpendicular to prev-next through mid
-	NR::Point biss =  mid + 100000 * rot90_rel (mid, next); 
+	NR::Point biss =  mid + 100000 * rot90_rel (mid, next);
 
 	// lengths of vectors to prev and next
 	gdouble prev_len = NR::L2 (prev - o);
@@ -423,7 +414,7 @@ sp_star_set_shape (SPShape *shape)
 		if (not_rounded) {
 			sp_curve_lineto (c, sp_star_get_xy (star, SP_STAR_POINT_KNOT2, 0, true));
 		} else {
-			sp_curve_curveto (c, 
+			sp_curve_curveto (c,
 				sp_star_get_curvepoint (star, SP_STAR_POINT_KNOT1, 0, NEXT),
 				sp_star_get_curvepoint (star, SP_STAR_POINT_KNOT2, 0, PREV),
 				sp_star_get_xy (star, SP_STAR_POINT_KNOT2, 0, true));
@@ -436,12 +427,12 @@ sp_star_set_shape (SPShape *shape)
 			sp_curve_lineto (c, sp_star_get_xy (star, SP_STAR_POINT_KNOT1, i, true));
 		} else {
 			if (star->flatsided == false) {
-				sp_curve_curveto (c, 
+				sp_curve_curveto (c,
 						sp_star_get_curvepoint (star, SP_STAR_POINT_KNOT2, i - 1, NEXT),
 						sp_star_get_curvepoint (star, SP_STAR_POINT_KNOT1, i, PREV),
 						sp_star_get_xy (star, SP_STAR_POINT_KNOT1, i, true));
 			} else {
-				sp_curve_curveto (c, 
+				sp_curve_curveto (c,
 						sp_star_get_curvepoint (star, SP_STAR_POINT_KNOT1, i - 1, NEXT),
 						sp_star_get_curvepoint (star, SP_STAR_POINT_KNOT1, i, PREV),
 						sp_star_get_xy (star, SP_STAR_POINT_KNOT1, i, true));
@@ -465,12 +456,12 @@ sp_star_set_shape (SPShape *shape)
 			sp_curve_lineto (c, sp_star_get_xy (star, SP_STAR_POINT_KNOT1, 0, true));
 		} else {
 			if (star->flatsided == false) {
-			sp_curve_curveto (c, 
+			sp_curve_curveto (c,
 				sp_star_get_curvepoint (star, SP_STAR_POINT_KNOT2, sides - 1, NEXT),
 				sp_star_get_curvepoint (star, SP_STAR_POINT_KNOT1, 0, PREV),
 				sp_star_get_xy (star, SP_STAR_POINT_KNOT1, 0, true));
 			} else {
-			sp_curve_curveto (c, 
+			sp_curve_curveto (c,
 				sp_star_get_curvepoint (star, SP_STAR_POINT_KNOT1, sides - 1, NEXT),
 				sp_star_get_curvepoint (star, SP_STAR_POINT_KNOT1, 0, PREV),
 				sp_star_get_xy (star, SP_STAR_POINT_KNOT1, 0, true));
