@@ -23,7 +23,7 @@
 #include "svg/svg.h"
 #include "sp-path.h"
 #include "sp-shape.h"
-#include "sp-marker.h"
+#include "marker.h"
 #include "enums.h"
 #include "sp-text.h"
 #include "sp-item-group.h"
@@ -56,60 +56,65 @@
 
 bool   Ancetre(Inkscape::XML::Node *a, Inkscape::XML::Node *who);
 
-void sp_selected_path_boolop(bool_op bop);
+void sp_selected_path_boolop(bool_op bop, const unsigned int verb=SP_VERB_NONE, const Glib::ustring description="");
 void sp_selected_path_do_offset(bool expand, double prefOffset);
 void sp_selected_path_create_offset_object(int expand, bool updating);
 
 void
 sp_selected_path_union()
 {
-    sp_selected_path_boolop(bool_op_union);
+    sp_selected_path_boolop(bool_op_union, SP_VERB_SELECTION_UNION, _("Union"));
 }
 
 void
 sp_selected_path_intersect()
 {
-    sp_selected_path_boolop(bool_op_inters);
+    sp_selected_path_boolop(bool_op_inters, SP_VERB_SELECTION_INTERSECT, _("Intersection"));
 }
 
 void
 sp_selected_path_diff()
 {
-    sp_selected_path_boolop(bool_op_diff);
+    sp_selected_path_boolop(bool_op_diff, SP_VERB_SELECTION_DIFF, _("Difference"));
 }
 
 void
 sp_selected_path_symdiff()
 {
-    sp_selected_path_boolop(bool_op_symdiff);
+    sp_selected_path_boolop(bool_op_symdiff, SP_VERB_SELECTION_SYMDIFF, _("Exclusion"));
 }
 void
 sp_selected_path_cut()
 {
-    sp_selected_path_boolop(bool_op_cut);
+    sp_selected_path_boolop(bool_op_cut, SP_VERB_SELECTION_CUT, _("Division"));
 }
 void
 sp_selected_path_slice()
 {
-    sp_selected_path_boolop(bool_op_slice);
+    sp_selected_path_boolop(bool_op_slice, SP_VERB_SELECTION_SLICE,  _("Cut path"));
 }
 
 
 // boolean operations
 // take the source paths from the file, do the operation, delete the originals and add the results
 void
-sp_selected_path_boolop(bool_op bop)
+sp_selected_path_boolop(bool_op bop, const unsigned int verb, const Glib::ustring description)
 {
     SPDesktop *desktop = SP_ACTIVE_DESKTOP;
 
     Inkscape::Selection *selection = sp_desktop_selection(desktop);
-
+    
     GSList *il = (GSList *) selection->itemList();
-
-    if (g_slist_length(il) < 2) {
+    
+    // allow union on a single object for the purpose of removing self overlapse (svn log, revision 13334)
+    if ( (g_slist_length(il) < 2) && (bop != bool_op_union)) {
         desktop->messageStack()->flash(Inkscape::ERROR_MESSAGE, _("Select <b>at least 2 paths</b> to perform a boolean operation."));
         return;
     }
+    else if ( g_slist_length(il) < 1 ) {
+        desktop->messageStack()->flash(Inkscape::ERROR_MESSAGE, _("Select <b>at least 1 path</b> to perform a boolean union."));
+        return;
+    }
 
     if (g_slist_length(il) > 2) {
         if (bop == bool_op_diff || bop == bool_op_symdiff || bop == bool_op_cut || bop == bool_op_slice ) {
@@ -405,7 +410,8 @@ sp_selected_path_boolop(bool_op bop)
         {
             SP_OBJECT(l->data)->deleteObject();
         }
-        sp_document_done(sp_desktop_document(desktop));
+        sp_document_done(sp_desktop_document(desktop), SP_VERB_NONE, 
+                         description);
         selection->clear();
 
         delete res;
@@ -413,27 +419,43 @@ sp_selected_path_boolop(bool_op bop)
         return;
     }
 
-    // remember important aspects of the source path, to be restored
-    Inkscape::XML::Node *repr_source;
+    // get the source path object
+    SPObject *source;
     if ( bop == bool_op_diff || bop == bool_op_symdiff || bop == bool_op_cut || bop == bool_op_slice ) {
         if (reverseOrderForOp) {
-             repr_source = SP_OBJECT_REPR(il->data);
+             source = SP_OBJECT(il->data);
         } else {
-             repr_source = SP_OBJECT_REPR(il->next->data);
+             source = SP_OBJECT(il->next->data);
         }
     } else {
         // find out the bottom object
         GSList *sorted = g_slist_copy((GSList *) selection->reprList());
 
         sorted = g_slist_sort(sorted, (GCompareFunc) sp_repr_compare_position);
-        repr_source = ((Inkscape::XML::Node *) sorted->data);
+
+        source = sp_desktop_document(desktop)->
+            getObjectByRepr((Inkscape::XML::Node *)sorted->data);
+
         g_slist_free(sorted);
     }
+
+    // adjust style properties that depend on a possible transform in the source object in order
+    // to get a correct style attribute for the new path
+    SPItem* item_source = SP_ITEM(source);
+    NR::Matrix i2d = sp_item_i2d_affine(item_source);
+    sp_item_adjust_stroke(item_source, i2d.expansion());
+    sp_item_adjust_pattern(item_source, i2d);
+    sp_item_adjust_gradient(item_source, i2d);
+
+    Inkscape::XML::Node *repr_source = SP_OBJECT_REPR(source);
+
+    // remember important aspects of the source path, to be restored
     gint pos = repr_source->position();
     Inkscape::XML::Node *parent = sp_repr_parent(repr_source);
-    char const *id = repr_source->attribute("id");
-    char const *style = repr_source->attribute("style");
-
+    gchar const *id = repr_source->attribute("id");
+    gchar const *style = repr_source->attribute("style");
+    gchar const *mask = repr_source->attribute("mask");
+    gchar const *clip_path = repr_source->attribute("clip-path");
 
     // remove source paths
     selection->clear();
@@ -486,6 +508,11 @@ sp_selected_path_boolop(bool_op bop)
 
             Inkscape::XML::Node *repr = sp_repr_new("svg:path");
             repr->setAttribute("style", style);
+            if (mask)
+                repr->setAttribute("mask", mask);
+            if (clip_path)
+                repr->setAttribute("clip-path", clip_path);
+
             repr->setAttribute("d", d);
             g_free(d);
 
@@ -527,6 +554,12 @@ sp_selected_path_boolop(bool_op bop)
         Inkscape::XML::Node *repr = sp_repr_new("svg:path");
         repr->setAttribute("style", style);
 
+        if ( mask )
+            repr->setAttribute("mask", mask);
+
+        if ( clip_path )
+            repr->setAttribute("clip-path", clip_path);
+
         repr->setAttribute("d", d);
         g_free(d);
 
@@ -540,7 +573,7 @@ sp_selected_path_boolop(bool_op bop)
         Inkscape::GC::release(repr);
     }
 
-    sp_document_done(sp_desktop_document(desktop));
+    sp_document_done(sp_desktop_document(desktop), verb, description);
 
     delete res;
 }
@@ -554,8 +587,7 @@ sp_selected_path_outline()
     Inkscape::Selection *selection = sp_desktop_selection(desktop);
 
     if (selection->isEmpty()) {
-        // TRANSLATORS: "to outline" means "to convert stroke to path"
-        desktop->messageStack()->flash(Inkscape::WARNING_MESSAGE, _("Select <b>path(s)</b> to outline."));
+        desktop->messageStack()->flash(Inkscape::WARNING_MESSAGE, _("Select <b>stroked path(s)</b> to convert stroke to path."));
         return;
     }
 
@@ -618,6 +650,8 @@ sp_selected_path_outline()
         float const scale = transform.expansion();
         gchar *style = g_strdup(SP_OBJECT_REPR(item)->attribute("style"));
         SPStyle *i_style = SP_OBJECT(item)->style;
+        gchar const *mask = SP_OBJECT_REPR(item)->attribute("mask");
+        gchar const *clip_path = SP_OBJECT_REPR(item)->attribute("clip-path");
 
         float o_width, o_miter;
         JoinType o_join;
@@ -749,6 +783,10 @@ sp_selected_path_outline()
             repr->setAttribute("d", str);
             g_free(str);
 
+            if (mask)
+                repr->setAttribute("mask", mask);
+            if (clip_path)
+                repr->setAttribute("clip-path", clip_path);
 
             if (SP_IS_SHAPE(item) && sp_shape_has_markers (SP_SHAPE(item))) {
 
@@ -830,10 +868,11 @@ sp_selected_path_outline()
     }
 
     if (did) {
-        sp_document_done(sp_desktop_document(desktop));
+        sp_document_done(sp_desktop_document(desktop), SP_VERB_SELECTION_OUTLINE, 
+                         _("Convert stroke to path"));
     } else {
         // TRANSLATORS: "to outline" means "to convert stroke to path"
-        desktop->messageStack()->flash(Inkscape::ERROR_MESSAGE, _("<b>No stroked paths</b> to outline in the selection."));
+        desktop->messageStack()->flash(Inkscape::ERROR_MESSAGE, _("<b>No stroked paths</b> in the selection."));
         return;
     }
 }
@@ -1034,7 +1073,11 @@ sp_selected_path_create_offset_object(int expand, bool updating)
     {
         // pas vraiment de points sur le resultat
         // donc il ne reste rien
-        sp_document_done(sp_desktop_document(desktop));
+        sp_document_done(sp_desktop_document(desktop), 
+                         (updating ? SP_VERB_SELECTION_LINKED_OFFSET 
+                          : SP_VERB_SELECTION_DYNAMIC_OFFSET),
+                         (updating ? _("Create linked offset")
+                          : _("Create dynamic offset")));
         selection->clear();
 
         delete res;
@@ -1098,7 +1141,11 @@ sp_selected_path_create_offset_object(int expand, bool updating)
         selection->set(nitem);
     }
 
-    sp_document_done(sp_desktop_document(desktop));
+    sp_document_done(sp_desktop_document(desktop), 
+                     (updating ? SP_VERB_SELECTION_LINKED_OFFSET 
+                      : SP_VERB_SELECTION_DYNAMIC_OFFSET),
+                     (updating ? _("Create linked offset")
+                      : _("Create dynamic offset")));
 
     delete res;
     delete orig;
@@ -1340,7 +1387,9 @@ sp_selected_path_do_offset(bool expand, double prefOffset)
     }
 
     if (did) {
-        sp_document_done(sp_desktop_document(desktop));
+        sp_document_done(sp_desktop_document(desktop), 
+                         (expand ? SP_VERB_SELECTION_OFFSET : SP_VERB_SELECTION_INSET),
+                         (expand ? _("Outset path") : _("Inset path")));
     } else {
         desktop->messageStack()->flash(Inkscape::ERROR_MESSAGE, _("<b>No paths</b> to inset/outset in the selection."));
         return;
@@ -1348,10 +1397,18 @@ sp_selected_path_do_offset(bool expand, double prefOffset)
 }
 
 
+static bool
+sp_selected_path_simplify_items(SPDesktop *desktop,
+                                Inkscape::Selection *selection, GSList *items,
+                                float threshold,  bool justCoalesce,
+                                float angleLimit, bool breakableAngles,
+                                bool modifySelection);
+
 
 //return true if we changed something, else false
 bool
-sp_selected_path_simplify_item(SPDesktop *desktop, Inkscape::Selection *selection, SPItem *item,
+sp_selected_path_simplify_item(SPDesktop *desktop,
+                 Inkscape::Selection *selection, SPItem *item,
                  float threshold,  bool justCoalesce,
                  float angleLimit, bool breakableAngles,
                  gdouble size,     bool modifySelection)
@@ -1361,18 +1418,12 @@ sp_selected_path_simplify_item(SPDesktop *desktop, Inkscape::Selection *selectio
 
     //If this is a group, do the children instead
     if (SP_IS_GROUP(item)) {
-
-        bool didSomething = false;
-
-        for ( GSList *children = sp_item_group_item_list(SP_GROUP(item));
-                 children  ; children = children->next) {
-
-            SPItem *child = (SPItem *) children->data;
-            didSomething |= sp_selected_path_simplify_item(desktop, selection, child, threshold, justCoalesce,
-                   angleLimit, breakableAngles, size, false);
-        }
-
-        return didSomething;
+        GSList *items = sp_item_group_item_list(SP_GROUP(item));
+        
+        return sp_selected_path_simplify_items(desktop, selection, items,
+                                               threshold, justCoalesce,
+                                               angleLimit, breakableAngles,
+                                               false);
     }
 
 
@@ -1401,6 +1452,8 @@ sp_selected_path_simplify_item(SPDesktop *desktop, Inkscape::Selection *selectio
     sp_item_write_transform(item, SP_OBJECT_REPR(item), NR::identity());
 
     gchar *style = g_strdup(SP_OBJECT_REPR(item)->attribute("style"));
+    gchar *mask = g_strdup(SP_OBJECT_REPR(item)->attribute("mask"));
+    gchar *clip_path = g_strdup(SP_OBJECT_REPR(item)->attribute("clip-path"));
 
     Path *orig = Path_for_item(item, false);
     if (orig == NULL) {
@@ -1432,8 +1485,21 @@ sp_selected_path_simplify_item(SPDesktop *desktop, Inkscape::Selection *selectio
 
     Inkscape::XML::Node *repr = sp_repr_new("svg:path");
 
+    // restore style, mask and clip-path
     repr->setAttribute("style", style);
+    g_free(style);
 
+    if ( mask ) {
+        repr->setAttribute("mask", mask);
+        g_free(mask);
+    }
+
+    if ( clip_path ) {
+        repr->setAttribute("clip-path", clip_path);
+        g_free(clip_path);
+    }
+
+    // path
     gchar *str = orig->svg_dump_path();
     repr->setAttribute("d", str);
     g_free(str);
@@ -1465,6 +1531,68 @@ sp_selected_path_simplify_item(SPDesktop *desktop, Inkscape::Selection *selectio
 }
 
 
+bool
+sp_selected_path_simplify_items(SPDesktop *desktop,
+                                Inkscape::Selection *selection, GSList *items,
+                                float threshold,  bool justCoalesce,
+                                float angleLimit, bool breakableAngles,
+                                bool modifySelection)
+{
+  bool simplifyIndividualPaths =
+    (bool) prefs_get_int_attribute("options.simplifyindividualpaths", "value", 0);
+  
+  gchar *simplificationType;
+  if (simplifyIndividualPaths) {
+    simplificationType = "individual paths";
+  } else {
+    simplificationType = "as a group";
+  }
+
+  bool didSomething = false;
+
+  NR::Rect selectionBbox = selection->bounds();
+  gdouble selectionSize  = L2(selectionBbox.dimensions());
+
+  gdouble simplifySize  = selectionSize;
+  
+  int pathsSimplified = 0;
+  int totalPathCount  = g_slist_length(items);
+  
+  desktop->disableInteraction();
+  
+  for (; items != NULL; items = items->next) {
+      SPItem *item = (SPItem *) items->data;
+      
+      if (!(SP_IS_GROUP(item) || SP_IS_SHAPE(item) || SP_IS_TEXT(item)))
+          continue;
+
+      if (simplifyIndividualPaths) {
+          NR::Rect itemBbox = item->invokeBbox(sp_item_i2d_affine(item));        
+          simplifySize      = L2(itemBbox.dimensions());
+      }
+
+
+      pathsSimplified++;
+
+      if (pathsSimplified % 20 == 0) {
+        gchar *message = g_strdup_printf(_("Simplifying %s - <b>%d</b> of <b>%d</b> paths simplified..."), simplificationType, pathsSimplified, totalPathCount);
+        desktop->messageStack()->flash(Inkscape::NORMAL_MESSAGE, message);
+        desktop->updateCanvasNow();
+      }
+
+      didSomething |= sp_selected_path_simplify_item(desktop, selection, item,
+                          threshold, justCoalesce, angleLimit, breakableAngles, simplifySize, modifySelection);
+  }
+
+  desktop->enableInteraction();
+  
+  if (pathsSimplified > 20) {
+    desktop->messageStack()->flash(Inkscape::NORMAL_MESSAGE, g_strdup_printf(_("Done - <b>%d</b> paths simplified."), pathsSimplified));
+  }
+  
+  return didSomething;
+}
+
 void
 sp_selected_path_simplify_selection(float threshold, bool justCoalesce,
                        float angleLimit, bool breakableAngles)
@@ -1479,28 +1607,17 @@ sp_selected_path_simplify_selection(float threshold, bool justCoalesce,
         return;
     }
 
-    // remember selection size
-    NR::Rect bbox = selection->bounds();
-    gdouble size  = L2(bbox.dimensions());
-
-    bool didSomething = false;
-
-    //Loop through all of the items in the selection
-    for (GSList *items = g_slist_copy((GSList *) selection->itemList());
-                        items != NULL; items = items->next) {
-
-        SPItem *item = (SPItem *) items->data;
-
-        if (!(SP_IS_GROUP(item) || SP_IS_SHAPE(item) || SP_IS_TEXT(item)))
-            continue;
-
-        didSomething |= sp_selected_path_simplify_item(desktop, selection, item,
-                           threshold, justCoalesce, angleLimit, breakableAngles, size, true);
-    }
+    GSList *items = g_slist_copy((GSList *) selection->itemList());
 
+    bool didSomething = sp_selected_path_simplify_items(desktop, selection,
+                                                        items, threshold,
+                                                        justCoalesce,
+                                                        angleLimit,
+                                                        breakableAngles, true);
 
     if (didSomething)
-        sp_document_done(sp_desktop_document(desktop));
+        sp_document_done(sp_desktop_document(desktop), SP_VERB_SELECTION_SIMPLIFY, 
+                         _("Simplify"));
     else
         desktop->messageStack()->flash(Inkscape::ERROR_MESSAGE, _("<b>No paths</b> to simplify in the selection."));
 
