@@ -2,15 +2,13 @@
    This file is part of potrace. It is free software and it is covered
    by the GNU General Public License. See the file COPYING for details. */
 
-/* $Id: render.cpp,v 1.4 2005/03/06 13:43:38 ishmal Exp $ */
+/* $Id: render.cpp 10570 2006-01-10 12:26:08Z rwst $ */
 
-#include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
 #include <string.h>
 
 #include "render.h"
-#include "greymap.h"
 #include "auxiliary.h"
 
 /* ---------------------------------------------------------------------- */
