@@ -4,8 +4,9 @@
  * Authors:
  *   Carl Hetherington
  *   Marco Scholten
+ *   Johan Engelen <j.b.c.engelen@ewi.utwente.nl>
  *
- * Copyright (C) 2004, 2006 Authors
+ * Copyright (C) 2004-2007 Authors
  *
  * Released under GNU GPL.  Read the file 'COPYING' for more information.
  */ 
@@ -34,6 +35,7 @@
 #include "selection-chemistry.h"
 #include "xml/repr.h"
 #include "ui/widget/style-swatch.h"
+#include "display/nr-filter-gaussian.h"
 
 namespace Inkscape {
 namespace UI {
@@ -95,6 +97,7 @@ InkscapePreferences::InkscapePreferences()
     initPageWindows();
     initPageClones();
     initPageTransforms();
+    initPageFilters();
     initPageSelecting();
     initPageMisc();
 
@@ -406,21 +409,33 @@ void InkscapePreferences::initPageWindows()
     _win_ontop_normal.init ( _("Normal"), "options.transientpolicy", "value", 1, true, &_win_ontop_none);
     _win_ontop_agressive.init ( _("Aggressive"), "options.transientpolicy", "value", 2, false, &_win_ontop_none);
 
+// FIXME: Temporary Win32 special code to enable transient dialogs
+#ifdef WIN32 
+    _win_ontop_win32.init ( _("Dialogs stay on top (experimental!)"), "options.dialogsontopwin32", "value", false);
+#endif 
+
     _page_windows.add_line( false, "", _win_save_geom, "", 
                             _("Save the window size and position with each document (only for Inkscape SVG format)"));
+#ifndef WIN32 // FIXME: Temporary Win32 special code to enable transient dialogs
     _page_windows.add_line( false, "", _win_hide_task, "", 
                             _("Whether dialog windows are to be hidden in the window manager taskbar"));
+#endif                           
     _page_windows.add_line( false, "", _win_zoom_resize, "", 
                             _("Zoom drawing when document window is resized, to keep the same area visible (this is the default which can be changed in any window using the button above the right scrollbar)"));
     _page_windows.add_line( false, "", _win_show_close, "", 
                             _("Whether dialog windows have a close button (requires restart)"));
     _page_windows.add_group_header( _("Dialogs on top:"));
+#ifndef WIN32 // FIXME: Temporary Win32 special code to enable transient dialogs
     _page_windows.add_line( true, "", _win_ontop_none, "", 
                             _("Dialogs are treated as regular windows"));
     _page_windows.add_line( true, "", _win_ontop_normal, "", 
                             _("Dialogs stay on top of document windows"));
     _page_windows.add_line( true, "", _win_ontop_agressive, "", 
                             _("Same as Normal but may work better with some window managers"));
+#else
+    _page_windows.add_line( false, "", _win_ontop_win32, "", 
+                            _("Whether dialogs should stay on top of document windows. Read the ReleaseNotes on this issue! (Rightclick the taskbar button and press 'Restore' to bring back a minimized document window)"));
+#endif                            
 
     this->AddPage(_page_windows, _("Windows"), PREFS_PAGE_WINDOWS);
 }
@@ -480,6 +495,35 @@ void InkscapePreferences::initPageTransforms()
     this->AddPage(_page_transforms, _("Transforms"), PREFS_PAGE_TRANSFORMS);
 }
 
+void InkscapePreferences::initPageFilters()
+{
+    _blur_quality_best.init ( _("Best quality (slowest)"), "options.blurquality", "value", 
+                                  BLUR_QUALITY_BEST, false, 0);
+    _blur_quality_better.init ( _("Better quality (slower)"), "options.blurquality", "value", 
+                                  BLUR_QUALITY_BETTER, false, &_blur_quality_best);
+    _blur_quality_normal.init ( _("Average quality"), "options.blurquality", "value", 
+                                  BLUR_QUALITY_NORMAL, true, &_blur_quality_best);
+    _blur_quality_worse.init ( _("Lower quality (faster)"), "options.blurquality", "value", 
+                                  BLUR_QUALITY_WORSE, false, &_blur_quality_best);
+    _blur_quality_worst.init ( _("Lowest quality (fastest)"), "options.blurquality", "value", 
+                                  BLUR_QUALITY_WORST, false, &_blur_quality_best);
+
+    _page_filters.add_group_header( _("Gaussian blur quality for display:"));
+    _page_filters.add_line( true, "", _blur_quality_best, "", 
+                           _("Best quality, but display may be very slow at high zooms (bitmap export always uses best quality)"));
+    _page_filters.add_line( true, "", _blur_quality_better, "", 
+                           _("Better quality, but slower display"));
+    _page_filters.add_line( true, "", _blur_quality_normal, "", 
+                           _("Average quality, acceptable display speed"));
+    _page_filters.add_line( true, "", _blur_quality_worse, "", 
+                           _("Lower quality (some artifacts), but display is faster"));
+    _page_filters.add_line( true, "", _blur_quality_worst, "", 
+                           _("Lowest quality (considerable artifacts), but display is fastest"));
+
+    this->AddPage(_page_filters, _("Filters"), PREFS_PAGE_FILTERS);
+}
+
+
 void InkscapePreferences::initPageSelecting()
 {
     _sel_all.init ( _("Select in all layers"), "options.kbselection", "inlayer", PREFS_SELECTION_ALL, false, 0);
@@ -539,6 +583,9 @@ void InkscapePreferences::initPageMisc()
     _misc_mask_remove.init ( _("Remove clipping path or mask after applying"), "options.maskobject", "remove", true);
     _page_misc.add_line(true, "", _misc_mask_remove, "", 
                         _("After applying, remove the object used as the clipping path or mask from the drawing"));
+    _misc_use_ext_input.init( _("Use a pressure sensitive tablet or other device (requires restart)"), "options.useextinput", "value", true);
+    _page_misc.add_line(true, "",_misc_use_ext_input, "",
+                        _("Use the capabilities of a tablet or other pressure sensitive device. Disable this only if you have problems with the tablet."));
 
     this->AddPage(_page_misc, _("Misc"), PREFS_PAGE_MISC);
 }
