@@ -134,6 +134,15 @@ Transformation::Transformation()
     _check_apply_separately.set_active(prefs_get_int_attribute_limited ("dialogs.transformation", "applyseparately", 0, 0, 1));
     _check_apply_separately.signal_toggled().connect(sigc::mem_fun(*this, &Transformation::onApplySeparatelyToggled));
 
+    // make sure all spinbuttons activate Apply on pressing Enter
+      ((Gtk::Entry *) (_scalar_move_horizontal.getWidget()))->set_activates_default(true);
+      ((Gtk::Entry *) (_scalar_move_vertical.getWidget()))->set_activates_default(true);
+      ((Gtk::Entry *) (_scalar_scale_horizontal.getWidget()))->set_activates_default(true);
+      ((Gtk::Entry *) (_scalar_scale_vertical.getWidget()))->set_activates_default(true);
+      ((Gtk::Entry *) (_scalar_rotate.getWidget()))->set_activates_default(true);
+      ((Gtk::Entry *) (_scalar_skew_horizontal.getWidget()))->set_activates_default(true);
+      ((Gtk::Entry *) (_scalar_skew_vertical.getWidget()))->set_activates_default(true);
+
     updateSelection(PAGE_MOVE, _getSelection());
 
     resetButton = add_button(Gtk::Stock::CLEAR, 0);
@@ -147,6 +156,7 @@ Transformation::Transformation()
     if (applyButton) {
         tooltips.set_tip((*applyButton), _("Apply transformation to selection"));
         applyButton->set_sensitive(false);
+        set_default (*applyButton); // activable by Enter in spinbuttons
     }
 
     // Connect to the global selection changed & modified signals
@@ -581,7 +591,8 @@ Transformation::applyPageMove(Inkscape::Selection *selection)
             x - bbox.min()[NR::X], y - bbox.min()[NR::Y]);
     }
 
-    sp_document_done ( sp_desktop_document (selection->desktop()) );
+    sp_document_done ( sp_desktop_document (selection->desktop()) , SP_VERB_DIALOG_TRANSFORM, 
+                       _("Move"));
 }
 
 void
@@ -632,7 +643,8 @@ Transformation::applyPageScale(Inkscape::Selection *selection)
         sp_selection_scale_relative(selection, center, scale);
     }
 
-    sp_document_done(sp_desktop_document(selection->desktop()));
+    sp_document_done(sp_desktop_document(selection->desktop()), SP_VERB_DIALOG_TRANSFORM, 
+                     _("Scale"));
 }
 
 void
@@ -650,7 +662,8 @@ Transformation::applyPageRotate(Inkscape::Selection *selection)
         sp_selection_rotate_relative(selection, center, angle);
     }
 
-    sp_document_done(sp_desktop_document(selection->desktop()));
+    sp_document_done(sp_desktop_document(selection->desktop()), SP_VERB_DIALOG_TRANSFORM, 
+                     _("Rotate"));
 }
 
 void
@@ -702,7 +715,8 @@ Transformation::applyPageSkew(Inkscape::Selection *selection)
         }
     }
 
-    sp_document_done(sp_desktop_document(selection->desktop()));
+    sp_document_done(sp_desktop_document(selection->desktop()), SP_VERB_DIALOG_TRANSFORM, 
+                     _("Skew"));
 }
 
 
@@ -728,7 +742,8 @@ Transformation::applyPageTransform(Inkscape::Selection *selection)
         sp_selection_apply_affine(selection, displayed); // post-multiply each object's transform
     }
 
-    sp_document_done(sp_desktop_document(selection->desktop()));
+    sp_document_done(sp_desktop_document(selection->desktop()), SP_VERB_DIALOG_TRANSFORM, 
+                     _("Edit transformation matrix"));
 }
 
 
