@@ -11,9 +11,8 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <string.h>
-#include <stdlib.h>
 #include <stdio.h>
+#include <glib/gstrfuncs.h>
 #include "version.h"
 
 gboolean sp_version_from_string(const gchar *string, Inkscape::Version *version)
