@@ -16,27 +16,16 @@
 #endif
 #include "path-prefix.h"
 
-#include <algorithm>
-#include <stdlib.h>
-#include <string.h>
-#include <libnr/nr-macros.h>
-#include <libnr/nr-rect.h>
-#include <libnr/nr-matrix.h>
-#include <libnr/nr-pixblock.h>
-#include <libnr/nr-pixblock-pattern.h>
-#include <libnr/nr-pixops.h>
 
-#include <map>
 
+#include <glib/gmem.h>
 #include <gtk/gtkiconfactory.h>
 #include <gtk/gtkstock.h>
 #include <gtk/gtkimage.h>
 
 #include <gtkmm/image.h>
 
-#include "forward.h"
 #include "prefs-utils.h"
-#include "inkscape-private.h"
 #include "inkscape.h"
 #include "document.h"
 #include "sp-item.h"
@@ -46,6 +35,10 @@
 
 #include "icon.h"
 
+static gboolean icon_prerender_task(gpointer data);
+
+static void addPreRender( Inkscape::IconSize lsize, gchar const *name );
+
 static void sp_icon_class_init(SPIconClass *klass);
 static void sp_icon_init(SPIcon *icon);
 static void sp_icon_destroy(GtkObject *object);
@@ -73,10 +66,24 @@ static int sp_icon_get_phys_size(int size);
 static void sp_icon_overlay_pixels( guchar *px, int width, int height, int stride,
                                     unsigned r, unsigned g, unsigned b );
 
+static void injectCustomSize();
+
 static GtkWidgetClass *parent_class;
 
 static bool sizeDirty = true;
 
+static bool sizeMapDone = false;
+static GtkIconSize iconSizeLookup[] = {
+    GTK_ICON_SIZE_INVALID,
+    GTK_ICON_SIZE_MENU,
+    GTK_ICON_SIZE_SMALL_TOOLBAR,
+    GTK_ICON_SIZE_LARGE_TOOLBAR,
+    GTK_ICON_SIZE_BUTTON,
+    GTK_ICON_SIZE_DND,
+    GTK_ICON_SIZE_DIALOG,
+    GTK_ICON_SIZE_MENU, // for Inkscape::ICON_SIZE_DECORATION
+};
+
 GtkType
 sp_icon_get_type()
 {
@@ -120,7 +127,7 @@ static void
 sp_icon_init(SPIcon *icon)
 {
     GTK_WIDGET_FLAGS(icon) |= GTK_NO_WINDOW;
-    icon->lsize = GTK_ICON_SIZE_BUTTON;
+    icon->lsize = Inkscape::ICON_SIZE_BUTTON;
     icon->psize = 0;
     icon->name = 0;
     icon->pb = 0;
@@ -182,6 +189,19 @@ static int sp_icon_expose(GtkWidget *widget, GdkEventExpose *event)
 {
     if ( GTK_WIDGET_DRAWABLE(widget) ) {
         SPIcon *icon = SP_ICON(widget);
+        if ( !icon->pb ) {
+            sp_icon_fetch_pixbuf( icon );
+        }
+
+        sp_icon_paint(SP_ICON(widget), &event->area);
+    }
+
+    return TRUE;
+}
+
+void sp_icon_fetch_pixbuf( SPIcon *icon )
+{
+    if ( icon ) {
         if ( !icon->pb ) {
             guchar *pixels = 0;
 
@@ -190,12 +210,12 @@ static int sp_icon_expose(GtkWidget *widget, GdkEventExpose *event)
             pixels = sp_icon_image_load( icon, icon->name );
 
             if (pixels) {
-                // don't pass the nr_free because we're caching the pixel
+                // don't pass the g_free because we're caching the pixel
                 // space loaded through ...
                 // I just changed this. make sure sp_icon_image_load still does the right thing.
                 icon->pb = gdk_pixbuf_new_from_data(pixels, GDK_COLORSPACE_RGB, TRUE, 8,
                                                     icon->psize, icon->psize, icon->psize * 4,
-                                                    /*(GdkPixbufDestroyNotify)nr_free*/NULL, NULL);
+                                                    /*(GdkPixbufDestroyNotify)g_free*/NULL, NULL);
                 icon->pb_faded = gdk_pixbuf_copy(icon->pb);
 
                 pixels = gdk_pixbuf_get_pixels(icon->pb_faded);
@@ -214,14 +234,9 @@ static int sp_icon_expose(GtkWidget *widget, GdkEventExpose *event)
                 g_warning ("failed to load icon '%s'", icon->name);
             }
         }
-
-        sp_icon_paint(SP_ICON(widget), &event->area);
     }
-
-    return TRUE;
 }
 
-
 static void sp_icon_screen_changed( GtkWidget *widget, GdkScreen *previous_screen )
 {
     if ( GTK_WIDGET_CLASS( parent_class )->screen_changed ) {
@@ -250,11 +265,13 @@ static void sp_icon_theme_changed( SPIcon *icon )
 
 
 static GtkWidget *
-sp_icon_new_full( GtkIconSize lsize, gchar const *name )
+sp_icon_new_full( Inkscape::IconSize lsize, gchar const *name )
 {
     static gint dump = prefs_get_int_attribute_limited( "debug.icons", "dumpGtk", 0, 0, 1 );
     static gint fallback = prefs_get_int_attribute_limited( "debug.icons", "checkNames", 0, 0, 1 );
 
+    addPreRender( lsize, name );
+
     GtkStockItem stock;
     gboolean tryLoad = gtk_stock_lookup( name, &stock );
     if ( !tryLoad && fallback ) {
@@ -266,7 +283,13 @@ sp_icon_new_full( GtkIconSize lsize, gchar const *name )
 
     GtkWidget *widget = 0;
     if ( tryLoad ) {
-        GtkWidget *img = gtk_image_new_from_stock( name, lsize );
+        gint trySize = CLAMP( static_cast<gint>(lsize), 0, static_cast<gint>(G_N_ELEMENTS(iconSizeLookup) - 1) );
+
+        if ( !sizeMapDone ) {
+            injectCustomSize();
+        }
+
+        GtkWidget *img = gtk_image_new_from_stock( name, iconSizeLookup[trySize] );
         if ( img ) {
             GtkImageType type = gtk_image_get_storage_type( GTK_IMAGE(img) );
             if ( type == GTK_IMAGE_STOCK ) {
@@ -299,12 +322,12 @@ sp_icon_new_full( GtkIconSize lsize, gchar const *name )
 }
 
 GtkWidget *
-sp_icon_new( GtkIconSize lsize, gchar const *name )
+sp_icon_new( Inkscape::IconSize lsize, gchar const *name )
 {
     return sp_icon_new_full( lsize, name );
 }
 
-Gtk::Widget *sp_icon_get_icon( Glib::ustring const &oid, GtkIconSize size )
+Gtk::Widget *sp_icon_get_icon( Glib::ustring const &oid, Inkscape::IconSize size )
 {
     Gtk::Widget *result = 0;
     GtkWidget *widget = sp_icon_new_full( size, oid.c_str() );
@@ -347,13 +370,46 @@ sp_icon_get_gtk_size(int size)
     return map[size];
 }
 
+static void injectCustomSize()
+{
+    // TODO - still need to handle the case of theme changes and resize, especially as we can't re-register a string.
+    if ( !sizeMapDone )
+    {
+        gint dump = prefs_get_int_attribute_limited( "debug.icons", "dumpDefault", 0, 0, 1 );
+        gint width = 0;
+        gint height = 0;
+        if ( gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height ) ) {
+            gint newWidth = ((width * 3) / 4);
+            gint newHeight = ((height * 3) / 4);
+            GtkIconSize newSizeEnum = gtk_icon_size_register( "inkscape-decoration", newWidth, newHeight );
+            if ( newSizeEnum ) {
+                if ( dump ) {
+                    g_message("Registered (%d, %d) <= (%d, %d) as index %d", newWidth, newHeight, width, height, newSizeEnum);
+                }
+                guint index = static_cast<guint>(Inkscape::ICON_SIZE_DECORATION);
+                if ( index < G_N_ELEMENTS(iconSizeLookup) ) {
+                    iconSizeLookup[index] = newSizeEnum;
+                } else if ( dump ) {
+                    g_message("size lookup array too small to store entry");
+                }
+            }
+        }
+        sizeMapDone = true;
+    }
+
+}
+
 static int sp_icon_get_phys_size(int size)
 {
     static bool init = false;
-    static int lastSys[GTK_ICON_SIZE_DIALOG + 1];
-    static int vals[GTK_ICON_SIZE_DIALOG + 1];
+    static int lastSys[Inkscape::ICON_SIZE_DECORATION + 1];
+    static int vals[Inkscape::ICON_SIZE_DECORATION + 1];
 
-    size = CLAMP( size, GTK_ICON_SIZE_MENU, GTK_ICON_SIZE_DIALOG );
+    size = CLAMP( size, GTK_ICON_SIZE_MENU, Inkscape::ICON_SIZE_DECORATION );
+
+    if ( !sizeMapDone ) {
+        injectCustomSize();
+    }
 
     if ( sizeDirty && init ) {
         GtkIconSize const gtkSizes[] = {
@@ -362,10 +418,14 @@ static int sp_icon_get_phys_size(int size)
             GTK_ICON_SIZE_LARGE_TOOLBAR,
             GTK_ICON_SIZE_BUTTON,
             GTK_ICON_SIZE_DND,
-            GTK_ICON_SIZE_DIALOG
+            GTK_ICON_SIZE_DIALOG,
+            static_cast<guint>(Inkscape::ICON_SIZE_DECORATION) < G_N_ELEMENTS(iconSizeLookup) ?
+                iconSizeLookup[static_cast<int>(Inkscape::ICON_SIZE_DECORATION)] :
+                GTK_ICON_SIZE_MENU
         };
         for (unsigned i = 0; i < G_N_ELEMENTS(gtkSizes) && init; ++i) {
-            unsigned const val_ix(gtkSizes[i]);
+            guint const val_ix = (gtkSizes[i] <= GTK_ICON_SIZE_DIALOG) ? (guint)gtkSizes[i] : (guint)Inkscape::ICON_SIZE_DECORATION;
+
             g_assert( val_ix < G_N_ELEMENTS(vals) );
 
             gint width = 0;
@@ -379,6 +439,7 @@ static int sp_icon_get_phys_size(int size)
     if ( !init ) {
         sizeDirty = false;
         gint dump = prefs_get_int_attribute_limited( "debug.icons", "dumpDefault", 0, 0, 1 );
+
         if ( dump ) {
             g_message( "Default icon sizes:" );
         }
@@ -390,7 +451,10 @@ static int sp_icon_get_phys_size(int size)
             GTK_ICON_SIZE_LARGE_TOOLBAR,
             GTK_ICON_SIZE_BUTTON,
             GTK_ICON_SIZE_DND,
-            GTK_ICON_SIZE_DIALOG
+            GTK_ICON_SIZE_DIALOG,
+            static_cast<guint>(Inkscape::ICON_SIZE_DECORATION) < G_N_ELEMENTS(iconSizeLookup) ?
+                iconSizeLookup[static_cast<int>(Inkscape::ICON_SIZE_DECORATION)] :
+                GTK_ICON_SIZE_MENU
         };
         gchar const *const names[] = {
             "GTK_ICON_SIZE_MENU",
@@ -398,13 +462,15 @@ static int sp_icon_get_phys_size(int size)
             "GTK_ICON_SIZE_LARGE_TOOLBAR",
             "GTK_ICON_SIZE_BUTTON",
             "GTK_ICON_SIZE_DND",
-            "GTK_ICON_SIZE_DIALOG"
+            "GTK_ICON_SIZE_DIALOG",
+            "inkscape-decoration"
         };
 
         GtkWidget *icon = (GtkWidget *)g_object_new(SP_TYPE_ICON, NULL);
 
         for (unsigned i = 0; i < G_N_ELEMENTS(gtkSizes); ++i) {
-            unsigned const val_ix(gtkSizes[i]);
+            guint const val_ix = (gtkSizes[i] <= GTK_ICON_SIZE_DIALOG) ? (guint)gtkSizes[i] : (guint)Inkscape::ICON_SIZE_DECORATION;
+
             g_assert( val_ix < G_N_ELEMENTS(vals) );
 
             gint width = 0;
@@ -518,7 +584,7 @@ sp_icon_image_load_pixmap(gchar const *name, unsigned lsize, unsigned psize)
         }
         guchar *spx = gdk_pixbuf_get_pixels(pb);
         int srs = gdk_pixbuf_get_rowstride(pb);
-        px = nr_new(guchar, 4 * psize * psize);
+        px = g_new(guchar, 4 * psize * psize);
         for (unsigned y = 0; y < psize; y++) {
             memcpy(px + 4 * y * psize, spx + y * srs, 4 * psize);
         }
@@ -543,20 +609,16 @@ sp_icon_doc_icon( SPDocument *doc, NRArenaItem *root,
         if (object && SP_IS_ITEM(object)) {
             /* Find bbox in document */
             NR::Matrix const i2doc(sp_item_i2doc_affine(SP_ITEM(object)));
-            NRRect dbox;
-            sp_item_invoke_bbox(SP_ITEM(object), &dbox, i2doc, TRUE);
+            NR::Rect dbox = SP_ITEM(object)->invokeBbox(i2doc);
 
             if ( SP_OBJECT_PARENT(object) == NULL )
             {
-                dbox.x0 = 0;
-                dbox.y0 = 0;
-
-                dbox.x1 = sp_document_width( doc );
-                dbox.y1 = sp_document_height( doc );
+                dbox = NR::Rect(NR::Point(0, 0),
+                                NR::Point(sp_document_width(doc), sp_document_height(doc)));
             }
 
             /* This is in document coordinates, i.e. pixels */
-            if (!nr_rect_d_test_empty(&dbox)) {
+            if (dbox.isEmpty() == false) {
                 NRGC gc(NULL);
                 /* Update to renderable state */
                 double sf = 1.0;
@@ -569,10 +631,10 @@ sp_icon_doc_icon( SPDocument *doc, NRArenaItem *root,
                                              NR_ARENA_ITEM_STATE_NONE );
                 /* Item integer bbox in points */
                 NRRectL ibox;
-                ibox.x0 = (int) floor(sf * dbox.x0 + 0.5);
-                ibox.y0 = (int) floor(sf * dbox.y0 + 0.5);
-                ibox.x1 = (int) floor(sf * dbox.x1 + 0.5);
-                ibox.y1 = (int) floor(sf * dbox.y1 + 0.5);
+                ibox.x0 = (int) floor(sf * dbox.min()[NR::X] + 0.5);
+                ibox.y0 = (int) floor(sf * dbox.min()[NR::Y] + 0.5);
+                ibox.x1 = (int) floor(sf * dbox.max()[NR::X] + 0.5);
+                ibox.y1 = (int) floor(sf * dbox.max()[NR::Y] + 0.5);
 
                 if ( dump ) {
                     g_message( "   box    --'%s'  (%f,%f)-(%f,%f)", name, (double)ibox.x0, (double)ibox.y0, (double)ibox.x1, (double)ibox.y1 );
@@ -601,10 +663,10 @@ sp_icon_doc_icon( SPDocument *doc, NRArenaItem *root,
                                                      NR_ARENA_ITEM_STATE_ALL,
                                                      NR_ARENA_ITEM_STATE_NONE );
                         /* Item integer bbox in points */
-                        ibox.x0 = (int) floor(sf * dbox.x0 + 0.5);
-                        ibox.y0 = (int) floor(sf * dbox.y0 + 0.5);
-                        ibox.x1 = (int) floor(sf * dbox.x1 + 0.5);
-                        ibox.y1 = (int) floor(sf * dbox.y1 + 0.5);
+                        ibox.x0 = (int) floor(sf * dbox.min()[NR::X] + 0.5);
+                        ibox.y0 = (int) floor(sf * dbox.min()[NR::Y] + 0.5);
+                        ibox.x1 = (int) floor(sf * dbox.max()[NR::X] + 0.5);
+                        ibox.y1 = (int) floor(sf * dbox.max()[NR::Y] + 0.5);
 
                         if ( dump ) {
                             g_message( "   box2   --'%s'  (%f,%f)-(%f,%f)", name, (double)ibox.x0, (double)ibox.y0, (double)ibox.x1, (double)ibox.y1 );
@@ -642,7 +704,7 @@ sp_icon_doc_icon( SPDocument *doc, NRArenaItem *root,
                     g_message( "   ua     --'%s'  (%f,%f)-(%f,%f)", name, (double)ua.x0, (double)ua.y0, (double)ua.x1, (double)ua.y1 );
                 }
                 /* Set up pixblock */
-                px = nr_new(guchar, 4 * psize * psize);
+                px = g_new(guchar, 4 * psize * psize);
                 memset(px, 0x00, 4 * psize * psize);
                 /* Render */
                 NRPixBlock B;
@@ -674,41 +736,35 @@ struct svg_doc_cache_t
     NRArenaItem *root;
 };
 
+static std::map<Glib::ustring, svg_doc_cache_t *> doc_cache;
+static std::map<Glib::ustring, guchar *> px_cache;
 
-static guchar *
-sp_icon_image_load_svg(gchar const *name, unsigned lsize, unsigned psize)
+static Glib::ustring icon_cache_key(gchar const *name,
+                                    unsigned lsize, unsigned psize)
 {
-    gint const dump = prefs_get_int_attribute_limited("debug.icons", "dumpSvg", 0, 0, 1);
+    Glib::ustring key=name;
+    key += ":";
+    key += lsize;
+    key += ":";
+    key += psize;
+    return key;
+}
 
-    // It would be nice to figure out how to attach "desctructors" to
-    // these maps to keep mem-watching tools like valgrind happy.
-    static std::map<Glib::ustring, svg_doc_cache_t *> doc_cache;
-    static std::map<Glib::ustring, guchar *> px_cache;
+static guchar *get_cached_pixels(Glib::ustring const &key) {
+    std::map<Glib::ustring, guchar *>::iterator found=px_cache.find(key);
+    if ( found != px_cache.end() ) {
+        return found->second;
+    }
+    return NULL;
+}
+
+static guchar *load_svg_pixels(gchar const *name,
+                               unsigned lsize, unsigned psize)
+{
     SPDocument *doc = NULL;
     NRArenaItem *root = NULL;
     svg_doc_cache_t *info = NULL;
 
-    Glib::ustring icon_index = name;
-    icon_index += ":";
-    icon_index += lsize;
-    icon_index += ":";
-    icon_index += psize;
-
-    // did we already load this icon at this scale/size?
-    guchar *px = 0;
-    {
-        std::map<Glib::ustring, guchar *>::iterator i = px_cache.find(icon_index);
-        if ( i != px_cache.end() ) {
-            px = i->second;
-        }
-    }
-    if (px) {
-        if ( dump ) {
-            g_message( "svg icon cached (%s).", icon_index.c_str() );
-        }
-        return px;
-    }
-
     // Fall back from user prefs dir into system locations.
     Glib::ustring iconsvg = name;
     iconsvg += ".svg";
@@ -718,8 +774,9 @@ sp_icon_image_load_svg(gchar const *name, unsigned lsize, unsigned psize)
     sources.push_back(g_build_filename(INKSCAPE_PIXMAPDIR, iconsvg.c_str(), NULL));
     sources.push_back(g_build_filename(INKSCAPE_PIXMAPDIR, "icons.svg", NULL));
 
-    // Use this loop to iterate through a list of possible document locations.
-    while (!sources.empty()) {
+    // Try each document in turn until we successfully load the icon from one
+    guchar *px=NULL;
+    while ( !sources.empty() && !px ) {
         gchar *doc_filename = sources.front();
 
         // Did we already load this doc?
@@ -764,21 +821,46 @@ sp_icon_image_load_svg(gchar const *name, unsigned lsize, unsigned psize)
         g_free(doc_filename);
         sources.pop_front();
 
-        // move on to the next document
+        // move on to the next document if we couldn't get anything
         if (!info && !doc) continue;
 
-        if ( dump ) {
-            g_message( "loading svg '%s' (%d:%d)", name, lsize, psize );
-        }
         px = sp_icon_doc_icon( doc, root, name, psize );
-        if (px) {
-            px_cache[icon_index] = px;
-            break;
-        }
     }
 
     return px;
-} // end of sp_icon_image_load_svg()
+}
+
+// returns true if icon needed preloading, false if nothing was done
+static bool prerender_icon(gchar const *name, unsigned lsize, unsigned psize)
+{
+    Glib::ustring key=icon_cache_key(name, lsize, psize);
+    guchar *px=get_cached_pixels(key);
+    if (px) {
+        return false;
+    } else {
+        px = load_svg_pixels(name, lsize, psize);
+        if (px) {
+            px_cache[key] = px;
+        }
+        return true;
+    }
+}
+
+static guchar *
+sp_icon_image_load_svg(gchar const *name, unsigned lsize, unsigned psize)
+{
+    Glib::ustring key=icon_cache_key(name, lsize, psize);
+
+    // did we already load this icon at this scale/size?
+    guchar *px=get_cached_pixels(key);
+    if (!px) {
+        px = load_svg_pixels(name, lsize, psize);
+        if (px) {
+            px_cache[key] = px;
+        }
+    }
+    return px;
+}
 
 void sp_icon_overlay_pixels(guchar *px, int width, int height, int stride,
                             unsigned r, unsigned g, unsigned b)
@@ -835,6 +917,51 @@ void sp_icon_overlay_pixels(guchar *px, int width, int height, int stride,
     }
 }
 
+class preRenderItem
+{
+public:
+    preRenderItem( Inkscape::IconSize lsize, gchar const *name ) :
+        _lsize( lsize ),
+        _name( name )
+    {}
+    Inkscape::IconSize _lsize;
+    Glib::ustring _name;
+};
+
+
+#include <queue>
+
+static std::queue<preRenderItem> pendingRenders;
+static bool callbackHooked = false;
+
+static void addPreRender( Inkscape::IconSize lsize, gchar const *name )
+{
+
+    if ( !callbackHooked )
+    {
+        callbackHooked = true;
+        g_idle_add_full( G_PRIORITY_LOW, &icon_prerender_task, NULL, NULL );
+    }
+
+    pendingRenders.push(preRenderItem(lsize, name));
+}
+
+gboolean icon_prerender_task(gpointer data) {
+    if (!pendingRenders.empty()) {
+        preRenderItem single=pendingRenders.front();
+        pendingRenders.pop();
+        int psize = sp_icon_get_phys_size(single._lsize);
+        prerender_icon(single._name.c_str(), single._lsize, psize);
+    }
+
+    if (!pendingRenders.empty()) {
+        return TRUE;
+    } else {
+        callbackHooked = false;
+        return FALSE;
+    }
+}
+
 /*
   Local Variables:
   mode:c++
