@@ -6,13 +6,9 @@
 # include "config.h"
 #endif
 #include <math.h>
-#include <string.h>
-#include <stdlib.h>
 #include <gtk/gtk.h>
-#include "../color.h"
 #include <glibmm/i18n.h>
 #include "../dialogs/dialog-events.h"
-#include "sp-color-preview.h"
 #include "sp-color-scales.h"
 
 #define CSC_CHANNEL_R (1 << 0)
