@@ -143,11 +143,13 @@ Inkscape::XML::Document *sp_repr_document_new(gchar const *rootname);
 
 /// Returns root node of document.
 inline Inkscape::XML::Node *sp_repr_document_root(Inkscape::XML::Document const *doc) {
+    if (!doc) return NULL;
     return const_cast<Inkscape::XML::Node *>(doc->root());
 }
 
 /// Returns the node's document.
 inline Inkscape::XML::Document *sp_repr_document(Inkscape::XML::Node const *repr) {
+    if (!repr) return NULL;
     return const_cast<Inkscape::XML::Document *>(repr->document());
 }
 
@@ -180,7 +182,7 @@ inline Inkscape::XML::Node *sp_repr_next(Inkscape::XML::Node *repr) {
 Inkscape::XML::Document *sp_repr_read_file(gchar const *filename, gchar const *default_ns);
 Inkscape::XML::Document *sp_repr_read_mem(gchar const *buffer, int length, gchar const *default_ns);
 void sp_repr_save_stream(Inkscape::XML::Document *doc, FILE *to_file, gchar const *default_ns=NULL, bool compress = false);
-gboolean sp_repr_save_file(Inkscape::XML::Document *doc, gchar const *filename, gchar const *default_ns=NULL);
+bool sp_repr_save_file(Inkscape::XML::Document *doc, gchar const *filename, gchar const *default_ns=NULL);
 
 void sp_repr_print(Inkscape::XML::Node *repr);
 
@@ -227,7 +229,7 @@ unsigned sp_repr_set_svg_double(Inkscape::XML::Node *repr, gchar const *key, dou
 /// \deprecated !
 double sp_repr_get_double_attribute(Inkscape::XML::Node *repr, gchar const *key, double def);
 /// \deprecated !
-int sp_repr_get_int_attribute(Inkscape::XML::Node *repr, gchar const *key, int def);
+long long int sp_repr_get_int_attribute(Inkscape::XML::Node *repr, gchar const *key, long long int def);
 /* End Deprecated? */
 
 int sp_repr_compare_position(Inkscape::XML::Node *first, Inkscape::XML::Node *second);
